//----------------------------------------------------------------------------
//  Body: CGRegVal.cpp,v 1.0                                 2004/11/16     //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2004                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <fstream>
#include <sstream>
#include <cstring>
#include <cstdlib>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <cstdio>
#include <list>

#include <TString.h>

#include "CGSendSMS.h"

using namespace std;

ostream & operator<<(ostream &stream_out, CGSendSMS &data_in)
{
  // input data in HEX
  stream_out << "               |    Info                         " << endl;
  stream_out << "===============|=================================" << endl;
  for(int nM=0; nM<data_in.numberOfusersMail; ++nM) {
     stream_out << "Mail User " << nM << "    | " << data_in.MailToUser[nM].Data() << endl;
  }
  for(int nP=0; nP<data_in.numberOfusersPhone; ++nP) {
     stream_out << "Phone User " << nP << "   | " << data_in.CERNPhoneNumberUsers[nP].Data() << endl;
  }

  return stream_out;
}

CGSendSMS::CGSendSMS()
{
   CERNPhoneNumberUsers = NULL;
   MailToUser           = NULL;
   DefValue();
}

CGSendSMS::~CGSendSMS()
{
   Clear();
}

void CGSendSMS::DefValue()
{
   filePath    = "/data/GOOFIE_DATA/";
   fileName    = "RAW_Data.txt";
   processName = "goofieOnLine";
   
   numberOfusersMail  = 3;
   numberOfusersPhone = 2;
   CERNPhoneNumberUsers  = new TString[numberOfusersPhone];
   CERNPhoneNumberUsers[0] = "41764875327@mail2sms.cern.ch";
   CERNPhoneNumberUsers[1] = "41764873368@mail2sms.cern.ch";
    
      
   MailToUser            = new TString[numberOfusersMail];
   MailToUser[0]         = "C.Garabatos@gsi.de";
   MailToUser[2]         = "D.Antonczyk@gsi.de";
   MailToUser[1]         = "joerg.lehnert@cern.ch";

   fSendMail             = 1;
   fFileSize             = 0;
   fPreviousFileSize     = 0;
}

void CGSendSMS::Clear()
{
   numberOfusersMail  = 0;
   numberOfusersPhone = 0;

   if(CERNPhoneNumberUsers) {
      delete [] CERNPhoneNumberUsers;
      CERNPhoneNumberUsers = NULL;
   }
   if(MailToUser) {
      delete [] MailToUser;
      MailToUser = NULL;
   }
   
   fSendMail             = 1;
   fFileSize             = 0;
   fPreviousFileSize     = 0;
}


void CGSendSMS::Initialization(TString configFile)
{
   static char   *fRegName[] = {"numberOfusersMail",
				"numberOfusersPhone",
				"CERNPhoneNumberUsers"
				"MailToUser"};
   
   static int     *varrINT[]     = {&numberOfusersPhone, &numberOfusersMail};
   static TString *varrSTRING[]  = {CERNPhoneNumberUsers, MailToUser};

   
//   ifstream fin(configFile.Data(),  ios::in);
//   if (fin.is_open())
//   {
//     TString parse;
//     TString temp;

//     while(1) {
//       parse.ReadLine(fin);
//       if(fin.eof())
//         break;

//       temp = parse;
//       temp.Remove(5);

//       for(int i=0; i<10; i++) {
//         if(!temp.CompareTo(fRegName[i])) {
//           parse.Remove(0,(parse.Index("=")+1));
//           *varr[i]=atoi(parse.Data());
//           break;
//         }
//       }
//     }
   //  fin.clear();
//     fin.close();
//   } else {
//     cerr << "Warning in <CGRegVal::SetValueFromFile>: an input file doesn't found, default register value" << flush << endl;
//     DefValue();
//   }
}

int  CGSendSMS::Run()
{
   try {
      if(IsProccesRunning()) {
	 // check run mode
	 fFileSize = GetFileSize();
	 if(fFileSize==0) throw (int)-2;
	 else {
	    fPreviousFileSize   = fFileSize;
	    // wait 15 s
	    sleep(15);
	 }
      } else throw (int)-1;
      // check file size 
      fFileSize = GetFileSize();
      if(fFileSize>fPreviousFileSize) {
	 fPreviousFileSize = fFileSize;
      } else throw (int)-3;
      
   } catch(int err) {
      switch(err) {
      case -1: //wait for proccess
	 //restart sending flag
	 fSendMail = 1;
	 do {
	    if(_DEBUGLEVEL >= 3)
	       cout << " waiting for process " << endl;
	    sleep(40); // 40 sec
	 } while(!IsProccesRunning());
	 break;
	 
      case -2:
	 do {
	    if(_DEBUGLEVEL >= 3)
	       cout << " waiting for file " << endl;
	    sleep(20); // 20 sec
	 } while(!GetFileSize() && IsProccesRunning());
	 break;

      case -3:
	 if(fSendMail) {
	    if(_DEBUGLEVEL >= 3)
	       cout << "sending mail " << endl;
	    SendMails();
	 }
	 fSendMail = 0;
	 break;
      }
      if(err!=-1 && err!=-2 && err!=-3) return 0; 
   }
   return 1;
}

int CGSendSMS::GetFileSize()
{ 
   // check for '/' in the fpathName
   TString fname = "/";
   if(fname.CompareTo(filePath.operator[](filePath.Length()-1))!=0) {
      filePath.Append("/");
   }
   TString command = filePath + fileName;
   struct stat sbuf;
   stat(command.Data(), &sbuf);
   return sbuf.st_size;
}

bool CGSendSMS::IsProccesRunning()
{
   struct stat sbuf;
   TString command;
   try {
      command = "ps -A | awk '{print $4}' | grep '" + processName + "' > /tmp/filePS.tmp";
      int valRet = system(command.Data());
      if(valRet!=256 && valRet!=0) {
	 if(_DEBUGLEVEL > 0)
	    cerr << "Error in <CGSendSMS::IsProccesRunning>: status of procces unknown" << endl;
	 throw (int)-1;
      }
      stat("/tmp/filePS.tmp", &sbuf);
   } catch(int err) {
      sbuf.st_size = 10;
   }
   // delete temp file
   command = "rm /tmp/filePS.tmp";
   if(system(command.Data())) {
      if(_DEBUGLEVEL > 0)
	 cerr << "Error in <CGSendSMS::IsProccesRunning>: temp file not deleted" << endl;
   }
   return sbuf.st_size ? 1 : 0;
}

int CGSendSMS::SendMails()
{
   int errStat = 0;
   TString command = "";
   // create temporary file with information
   char fileName[] = "/tmp/GOOFIE_XXXXXX";
   mkstemp(fileName);
   fstream filestr;
   filestr.open(fileName, fstream::out);
   // get time of event
   time_t rawtime;
   struct tm * timeinfo;
   time(&rawtime);
   timeinfo = localtime(&rawtime);
   // email body
   filestr << "GOOFIE Problems - " << asctime(timeinfo);
   filestr << "Please take care of this issue" << endl;
   filestr.clear();
   filestr.close();
   // send email
   for(int nM = 0; nM<numberOfusersMail; ++nM) {
      command = "mailx -s \\\"Warning_GOOFIE\\\" " + MailToUser[nM] + " < " + fileName;
      if(system(command.Data())) {
	 if(_DEBUGLEVEL > 0)
	    cerr << "Error in <CGSendSMS::SendMails>: sending mail fail" << endl;
	 errStat = -100; 
      }
      if(_DEBUGLEVEL >= 3)
	 cout << "mail = " << command << endl;
   }
   for(int nP = 0; nP<numberOfusersPhone; ++nP) {
      command = "mailx -s \\\"Warning_GOOFIE\\\" " + CERNPhoneNumberUsers[nP] + " < " + fileName;
      if(system(command.Data())) {
	 if(_DEBUGLEVEL > 0)
	    cerr << "Error in <CGSendSMS::SendMails>: sending SMS fail" << endl;
	 errStat = -101;
      }
      if(_DEBUGLEVEL >= 3)
	 cout << "phone = " << command << endl;
   }
   // delete temp file
   command = "rm "; command += fileName;
   if(system(command.Data())) {
      if(_DEBUGLEVEL > 0)
 	 cerr << "Error in <CGSendSMS::SendMails>: sending mail fail" << endl;
      errStat = -1000;
   }
   return errStat;
}
