//----------------------------------------------------------------------------
//  Header: CCFileParser.h, v 1.0                             2007/08/24    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Configuration file reader                                               //
//                                                                          //
//----------------------------------------------------------------------------

#include <algorithm>
#include <stdexcept>
#include <map>
#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <sstream>
#include <set>
#include <iterator>

typedef std::map<std::string, double>      MapStringDbl_t;
typedef std::map<std::string, std::string> MapStringString_t;
typedef std::set<std::string>              SetString_t;

template<class _T>
class CCFileParser
{
public:
   
   typedef _T                                 container_t;
   typedef typename container_t::mapped_type  value_type_t;
   typedef typename container_t::key_type     key_type_t;

public:
   CCFileParser( const std::string &_file_name )
   {
      _read_file(_file_name);
   }

   void read_file( const std::string &_file_name )
   {
      _read_file(_file_name);
   }

   // TODO: add method, which returns iterator to the found element
   value_type_t get_value( const key_type_t &_key )
   {
      if( m_container.find(_key) == m_container.end() )
      {
         std::stringstream ss;
         ss << _key;
         throw std::runtime_error("Can't find the key in the container. Key: " + ss.str());
      }
      return m_container[_key];
   }

   void get_all( SetString_t * _Set ) const
   {
      if ( !_Set )
         return;

      // TODO: change this algorithm by using for_each algorithm
      typename container_t::const_iterator iter = m_container.begin();
      typename container_t::const_iterator iter_end = m_container.end();
      for(; iter != iter_end; ++iter)
         {
            std::stringstream ss;
            ss << iter->first << ":" << iter->second;
            _Set->insert(ss.str());
         }
   }

    
private:
   void _read_file( const std::string &_file_name )  throw(std::exception)
   {
      m_container.clear();

      std::ifstream f(_file_name.c_str());
      if(!f.is_open())
         throw std::runtime_error("Can't open for reading a file: "+_file_name);

      const size_t buf_size = 1024;
      std::vector<char> buf(buf_size);
      while( f.good())
         {
            f.getline( &buf[0], buf.capacity(), '\n' );
            _fill_container( std::string(&buf[0]) );
	    buf.clear();
            buf.resize(buf_size);
         }
   }

   void _fill_container( const std::string &_str)
   {
      std::string strLine( _str );
      // Clean all spaces
      strLine.erase(std::remove(strLine.begin(), strLine.end(), ' '), strLine.end());
      // Find =
      const std::string::size_type equal_pos = strLine.find('=');
      // Validate the string
      if( strLine.npos == equal_pos )
         return;
      // remove comments "//"
      const std::string::size_type comment_pos = strLine.rfind("//");
      if( strLine.npos > comment_pos ) {
	 strLine.erase(comment_pos, strLine.size() - comment_pos );
      }
      // Get Key and Value
      const std::string strKey( strLine.substr(0, equal_pos) );
      const std::string strValue( strLine.substr((equal_pos+1), strLine.size() - (equal_pos+1) ) );
      // Key
      key_type_t key;
      {
         std::stringstream ss;
         ss << strKey;
         ss >> key;
      }
      // Value
      value_type_t val;
      {
         std::stringstream ss;
         ss << strValue;
         ss >> val;
      }

      m_container.insert( std::make_pair(key, val) );
   }

private:
   container_t m_container;
};


template <typename _Elem, typename _Traits, typename _T>
std::basic_ostream<_Elem, _Traits> &operator << ( std::basic_ostream<_Elem, _Traits> &_stream, const CCFileParser<_T> &_ch )
{
   using namespace std;
   ostringstream str;
   str << "********************" << "\n"
       << "CCFileParser contains:" << "\n";

   SetString_t tmp;
   _ch.get_all( &tmp );
   copy( tmp.begin(), tmp.end(), ostream_iterator<std::string>( str, "\n" ) );
   str << "********************" << "\n"
       << endl;

   _stream << str.str();

   return _stream;
}
