//----------------------------------------------------------------------------
//  Body: COnLineExeption.cpp, v 1.0                          2007/03/14    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <fstream>

#include "CMacroDef.h"
#include "COnLineExeption.h"

using namespace std;

//set as default cerr 
//ostream COnLineExeption::fstreamSet = cerr;
//ostream COnLineExeption::fstreamGet = cout;

pthread_mutex_t  COnLineExeption::flogMutex;
bool             COnLineExeption::fmutexInitialized = 0;
unsigned int     COnLineExeption::fOBJcounter       = 0;

COnLineExeption::COnLineExeption()
{
   // error state/info
   feInfo  = "";
   feCode  = 0x00000000;
   // counter
   ++fOBJcounter;
   // only at first call
   CreateMutexObject();
}

COnLineExeption::~COnLineExeption()
{
   if(fOBJcounter>0) {
      --fOBJcounter;
   }
}

void COnLineExeption::CreateMutexObject()
{
   unsigned int err = 0;
   // mutex initialization
   try {
      if(!fmutexInitialized) {
	 pthread_mutexattr_t  lmutexarrt;
	 // setting mutex attribute
	 err = pthread_mutexattr_init(&lmutexarrt);
	 DEBUGINFO("Debuge Info => COnLineExeption::CreateMutexObject => mutex attribute initialization",err)
	    if(err) throw err;
	 // attribute ...
	 
	 // initialize mutex
	 err = pthread_mutex_init(&flogMutex, &lmutexarrt);
	 DEBUGINFO("Debuge Info => COnLineExeption::CreateMutexObject => initialize mutex",err)
	 if(err) throw err;
	 
	 err = pthread_mutexattr_destroy(&lmutexarrt);
	 DEBUGINFO("Debuge Info => COnLineExeption::CreateMutexObject => attribute mutex destroyed",err)
	 if(err) throw err;
	 
	 // mutex created
	 fmutexInitialized = 1;
      } else {
	 DEBUGINFO("Debuge Info => COnLineExeption::CreateMutexObject => mutex already initialized", 0)
      }
   } catch (unsigned int _err) {
      SetExeptionError(_err, "<COnLineExeption::CreateMutexObject()>: Initialization of mutex failed");
   }
}


void COnLineExeption::PrintExeptionError()
{
   cerr << "Code Error=" << hex << feCode << ", " << feInfo << flush << endl;
}
