//----------------------------------------------------------------------------
//  Body: SConfigData.cpp, v 1.0                              2007/04/16    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>

#include "CMacroDef.h"
#include "COnLineExeption.h"
#include "CCFileParser.h"
#include "SConfigData.h"

using namespace std;

// loaded from file
int    SConfigData::fEventsInPack                 = 500;         // how many events per vector (pack)
int    SConfigData::fEventsUsedForAnalysis        = 2500;        // how many events used for analysis
int    SConfigData::fRawDataSize                  = 5;           // write tree to root file after # of pack (int)
int    SConfigData::fGenerationMode               = 1;           // 1=True Event, 2=True event Smeared, 3=intiger numbers, work only with compilation option "NO_HARDWARE_REQUIRED"
bool   SConfigData::fStoreRawData                 = true;        // storring RAW data on local disk
bool   SConfigData::fPrintGasAnalysisResults      = true;        // dump on screan gas analysis results

// root/dim - output files
string SConfigData::fRootFileName                 = "ALICE____"; // prefix for ROOT files _RunNumber, each run=250000ev.
string SConfigData::fPathToRootFile               = "./";        // location where files should by copy
string SConfigData::fTrueEventDataFileLocation    = "./";
string SConfigData::fDIMServerPrefix              = "ALICE.???"; // ALICE.TRD, ALICE.TPC, etc

//constans
const int SConfigData::fNumberOfADCSampels        = 512;         // how many ADC bins in goofie board
int       SConfigData::fNIUpdataRate              = 20;          // updata ni values after given # of events from GOOFIE borad

// status
bool   SConfigData::fInitializationState          = false;


SConfigData::SConfigData()
{
   // error / exeption handling
   fexeptionStatus  = new COnLineExeption();
}

SConfigData::~SConfigData()
{
   if(fexeptionStatus) {
      delete fexeptionStatus;
      fexeptionStatus = NULL;
   }
}

unsigned int SConfigData::InitializeFromFile(const string _filename)
{
  unsigned int error = 0;
  if(!fInitializationState) {
    try {
       // Numbers
       CCFileParser<MapStringDbl_t> cfgNumber(_filename);      
       // set values from file
       fEventsInPack            = (int)cfgNumber.get_value("EventsInPack");
       fEventsUsedForAnalysis   = (int)cfgNumber.get_value("EventsUsedForAnalysis");
       fRawDataSize             = (int)cfgNumber.get_value("RawDataSize");
       fGenerationMode          = (int)cfgNumber.get_value("GenerationMode");
       fStoreRawData            = (bool)cfgNumber.get_value("StoreRawData");
       fPrintGasAnalysisResults = (bool)cfgNumber.get_value("PrintGasAnalysisResults");

       fNIUpdataRate            = (int)cfgNumber.get_value("NIUpdataRate");
  
       // Strings
       CCFileParser<MapStringString_t> cfgString(_filename);
       // setvalue from file
       fRootFileName              = cfgString.get_value("RootFileName");
       fPathToRootFile            = cfgString.get_value("PathToRootFile");
       fTrueEventDataFileLocation = cfgString.get_value("TrueEventDataFileLocation"); 
       fDIMServerPrefix           = cfgString.get_value("DIMServerPrefix");
      	  
       // initialization sucesfull
       fInitializationState = true;
      
    } catch(exception& e) {
       string errorInfp = "<SConfigData::InitializeFromFile()>:";
       errorInfp +=  e.what();
       error = 0x0000001;
       fexeptionStatus->SetExeptionError(error, errorInfp.c_str());
       fInitializationState = false;
       fexeptionStatus->SetExeptionError(error, "<SConfigData::InitializeFromFile()>: default values are going to be use");
    }
  }
  DEBUGINFO("Debuge Info =>  SConfigData::InitializeFromFile => initialization state is ", error)
  return error;
}
