//----------------------------------------------------------------------------
//  Header: SConfigData.h, v 1.0                              2007/04/16    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef SCONFIGDATA
#define SCONFIGDATA

#include <string>

class COnLineExeption; // for error\exeption handling


class SConfigData
{
 public:
   SConfigData();
   virtual ~SConfigData();

   unsigned int InitializeFromFile(const std::string _filename);
 
   int  GetNumberOfADCSampels()    { return fNumberOfADCSampels;}
   int  GetEventsInPack()          { return fEventsInPack;} 
   int  GetEventsUsedForAnalysis() { return fEventsUsedForAnalysis;} 

public:
   
   // common
   static int         fEventsInPack;              // how many events readed per vector
   static int         fEventsUsedForAnalysis;     // how many events used for analysis
   static int         fRawDataSize;               // write tree to root file after # of pack (int)
   static int         fGenerationMode;            // 1=True Event, 2=True event Smeared, 3=intiger numbers, work only with compilation option "NO_HARDWARE_REQUIRED"
   static bool        fStoreRawData;              // 0=No, 1=yes
   static bool        fPrintGasAnalysisResults;   // dump on screan gas analysis results

   static std::string fRootFileName;              // prefix for ROOT files _RunNumber, each run=250000ev.
   static std::string fPathToRootFile;            // location where files should by copy
   static std::string fTrueEventDataFileLocation; // location of the input data for simulation  
   static std::string fDIMServerPrefix;           // ALICE.TRD, ALICE.TPC, etc
 
   // hardware
   // GOOFIE borad
   const static int fNumberOfADCSampels;           // how many sampels is taken by readout board per event
   
   // NI     borad
   static int       fNIUpdataRate;                 // updata NI values after # of events from GOOFIE borad

   // error\exeption status
   COnLineExeption *fexeptionStatus;               // handling error mesages

private:
   // initialization from file on/off
   static bool fInitializationState;               //
};

#endif
