//----------------------------------------------------------------------------
//  Header: AliGoofieGasResult.h, v 1.0                       2007/11/19    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef ALIGOOFIEGASRESULT
#define ALIGOOFIEGASRESULT

#include <TObject.h>
#include <iostream>

class AliGoofieGasResult : public TObject
{
public:
   typedef Float_t typeGoofie;
   
   AliGoofieGasResult ();
   AliGoofieGasResult (const AliGoofieGasResult &_val);
   AliGoofieGasResult & operator= (const AliGoofieGasResult &_val);

   virtual ~AliGoofieGasResult() {;}

   virtual void Clear();

   virtual void SetResults(const AliGoofieGasResult *_val);
   
   // test function
   virtual void FillDummyValue();
   void         DumpValues(std::ostream &strm);
   
public:
   typeGoofie    fFieldCageHV;               // field voltage in monitor read through last resistor
   typeGoofie    fPickupHV;                  // anod voltage on pickup detector
   typeGoofie    fPeakPosNear;               // near peak position in pickup spectrum
   typeGoofie    fPeakPosFar;                // far peak position in pickup spectrum
   typeGoofie    fPeakAreaNear;              // near peak area in pickup spectrum
   typeGoofie    fPeakAreaFar;               // far peak area in pickup spectrum
   typeGoofie    fTemp1;                     // temperature from sensor 1
   typeGoofie    fTemp2;                     // temperature from sensor 2
   typeGoofie    fPressure;                  // pressure inside monitor
   typeGoofie    fGainNear;                  // gain calculated using Near area
   typeGoofie    fGainNearCorrected;         // gain calculated using Near area corected for P/T U variations
   typeGoofie    fGainFar;                   // gain calculated using Far area
   typeGoofie    fGainFarCorrected;          // gain calculated using Far area corected for P/T U variations
   typeGoofie    fFractionCO2;               // CO_2 fraction in gas mixture
   typeGoofie    fFractionN2;                // N_2 fraction in gas mixture
   typeGoofie    fDriftVelocity;             // drift velocity of electron
   typeGoofie    fDriftVelocityCorrected;    // drift velocity of electron corected for P/T U variations
   typeGoofie    fCorrectionFactor;          // calculated from P/T U variations
   int           fNumberOfDischarge;         // how many discharges happend during this time window

   ClassDef(AliGoofieGasResult, 1)           // AliGoofieGasResult structure
};
#endif
