//----------------------------------------------------------------------------
//  Header: AliGoofiePack.cpp, v 1.0                          2007/10/22    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>

#include "TProcessID.h"

#include "AliGoofieRawEvent.h"
#include "AliGoofiePack.h"

using namespace std;


ClassImp(AliGoofiePack)

   
TClonesArray *AliGoofiePack::fgCRawEvent = NULL;


AliGoofiePack::AliGoofiePack() : TObject()
{
   if(!fgCRawEvent) {
      fgCRawEvent = new TClonesArray("AliGoofieRawEvent", 2500);
   }
   fCRawEvent      = fgCRawEvent;
   fNumberOfEvents = 0;
}

AliGoofiePack::AliGoofiePack(const AliGoofiePack &_ref) : TObject()
{   
   fAcquisitionTime    = _ref.fAcquisitionTime;
   fGasAnalysisResults = _ref.fGasAnalysisResults;
   fNumberOfEvents     = _ref.fNumberOfEvents;

   TClonesArray &temp = *fCRawEvent;
   for(int i=0; i<fNumberOfEvents; ++i) {
      //TClonesArray &temp = *fCRawEvent;
      new(temp[i]) AliGoofieRawEvent(*_ref.GetCRawEvent(i));
   }
}
   
AliGoofiePack & AliGoofiePack::operator=(const AliGoofiePack &_ref)
{
   if(this != &_ref) {
      
      fAcquisitionTime    = _ref.fAcquisitionTime;
      fGasAnalysisResults = _ref.fGasAnalysisResults;
      fNumberOfEvents     = _ref.fNumberOfEvents;

      TClonesArray &temp = *fCRawEvent;
      for(int i=0; i<fNumberOfEvents; ++i) {
	 //TClonesArray &temp = *fCRawEvent;
	 new(temp[i]) AliGoofieRawEvent(*_ref.GetCRawEvent(i));
      }
   }
   return *this;
}

AliGoofiePack::~AliGoofiePack()
{
   if(fgCRawEvent) {
      fgCRawEvent->Delete();
      fgCRawEvent = NULL;        // ????
   }
   fCRawEvent = fgCRawEvent; 
}

void AliGoofiePack::Clear()
{
   // Clear TClonesArray's
   if(fCRawEvent)   fCRawEvent->Delete();
   fAcquisitionTime.Set();  
   fGasAnalysisResults.Clear();
   fNumberOfEvents = 0;
}
 

void AliGoofiePack::BuildEvent(const AliGoofieGasResult *_GasAnaResults, const std::deque<AliGoofieRawEvent> &fPack)
{
   //Save current Object count
   Int_t ObjectNumber = TProcessID::GetObjectCount();
   Clear();

   // Set gas analysis results
   fAcquisitionTime.Set();                   //   time when a pack has been stored 
   fGasAnalysisResults = *_GasAnaResults;    //   results from onLine application for a given pack

   AliGoofieRawEvent *event = NULL;                //   time spectra
   for(unsigned int pos=0; pos<fPack.size(); ++pos) {
      TClonesArray &tmpEvent = *fCRawEvent;
      event = new(tmpEvent[fNumberOfEvents++]) AliGoofieRawEvent(fPack.at(pos));
   }
   //Save reference to last Event in the collection of Events
   fLastEvent = event;
   
   TProcessID::SetObjectCount(ObjectNumber);
}

void AliGoofiePack::ChangeCRawEvent(AliGoofieRawEvent *_event, Int_t &_val)
{
   TClonesArray &tmp = *fCRawEvent;
   tmp[_val] = _event;
   fNumberOfEvents = fCRawEvent->GetEntries();
}
