//----------------------------------------------------------------------------
//  Header: AliGoofiePack.h, v 1.0                            2007/10/22    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef ALIGOOFIEPACK
#define ALIGOOFIEPACK

#include <deque>

#include <TObject.h>
#include <TClonesArray.h>
#include <TRef.h>
#include <TTimeStamp.h>

#include "AliGoofieGasResult.h"

class AliGoofieRawEvent;


class AliGoofiePack : public TObject
{
public:
   
   AliGoofiePack();
   AliGoofiePack(const AliGoofiePack &_ref);
   virtual ~AliGoofiePack();
   AliGoofiePack & operator=(const AliGoofiePack &_ref);

   virtual void Clear();

   void BuildEvent(const AliGoofieGasResult *_GasAnaResults , const std::deque<AliGoofieRawEvent> &fPack);

   void ChangeCRawEvent(AliGoofieRawEvent *track, Int_t &_val);

   AliGoofieRawEvent  *GetCRawEvent(Int_t &_val) const             {return (AliGoofieRawEvent*)fCRawEvent->At(_val);}
   AliGoofieGasResult *GetGasResults()                             {return &fGasAnalysisResults;}
   Int_t GetNumberOfEvents()                                       {return fNumberOfEvents;}
   TTimeStamp *GetTimeStamp()                                      {return &fAcquisitionTime;}
   
private:
   TTimeStamp               fAcquisitionTime;           //   time when a pack has been stored 
   AliGoofieGasResult       fGasAnalysisResults;        //   results from gas analysis application for a given pack
   Int_t                    fNumberOfEvents;            //   how many event used for pack construction
   TClonesArray            *fCRawEvent;                 //-> event array
   static TClonesArray     *fgCRawEvent;                //-> static event array
   TRef                     fLastEvent;                 //! 

   ClassDef(AliGoofiePack, 1)                           // AliGoofiePack structure
};
#endif
