//----------------------------------------------------------------------------
//  Body: AliGoofieRawEvent.cxx, v 1.0                        2007/10/22    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>

#include "AliGoofieRawEvent.h"

using namespace std;


ClassImp(AliGoofieRawEvent)
   

AliGoofieRawEvent::AliGoofieRawEvent() : TObject()
{
   // array size
   fLocalCopyOfNumberOfADCSampels = 0;
   // pointers to arrays
   fDataPickup = NULL;
   fDataNear   = NULL;
   fDataFar    = NULL;
   // set to zero
   Clear();
   
#ifdef DEBUG_ONLINE
   cout << "Debuge Info => AliGoofieRawEvent object created, status code = 0" << flush << endl;
#endif
}

AliGoofieRawEvent::AliGoofieRawEvent(const Int_t _size) : TObject()
{
   // pointers to arrays
   fDataPickup = NULL;
   fDataNear   = NULL;
   fDataFar    = NULL;
   // allocate memory
   CreateArrays(_size);
   // set to zero
   Clear();
   
#ifdef DEBUG_ONLINE
   cout << "Debuge Info => AliGoofieRawEvent(int) object created, status code = 0" << flush << endl;
#endif
}

AliGoofieRawEvent::AliGoofieRawEvent(const AliGoofieRawEvent &_data) : TObject()
{
   fLocalCopyOfNumberOfADCSampels = _data.fLocalCopyOfNumberOfADCSampels;
   
   fGoodEvent                     = _data.fGoodEvent;
   fEventTime                     = _data.fEventTime;
   fTriggerChannel                = _data.fTriggerChannel;
   // set to zero slowControl variable
   fSlowControl.sPress            = _data.fSlowControl.sPress;
   fSlowControl.sTemp1            = _data.fSlowControl.sTemp1;
   fSlowControl.sTemp2            = _data.fSlowControl.sTemp2;
   fSlowControl.sEfVoltage        = _data.fSlowControl.sEfVoltage;
   fSlowControl.sHVPickup         = _data.fSlowControl.sHVPickup;
   
   try {
      fDataPickup = new Int_t[fLocalCopyOfNumberOfADCSampels];
      fDataNear   = new Int_t[fLocalCopyOfNumberOfADCSampels];
      fDataFar    = new Int_t[fLocalCopyOfNumberOfADCSampels];
   } catch(...) {
      cerr << "Error in <AliGoofieRawEvent::AliGoofieRawEvent(const AliGoofieRawEvent &)>: Dynamic memory allocation failed " << endl;
   }

   if(_data.fDataPickup &&  _data.fDataNear && _data.fDataFar) {
      for(int i=0; i<(fLocalCopyOfNumberOfADCSampels); ++i) {
	 fDataPickup[i] = _data.fDataPickup[i];
	 fDataNear[i]   = _data.fDataNear[i];
	 fDataFar[i]    = _data.fDataFar[i];
      }
   } else {
      cerr << "Error in <AliGoofieRawEvent::AliGoofieRawEvent(const AliGoofieRawEvent &)>: Approaching to read memory segment which is out of program scope" << endl;
   }
   
#ifdef DEBUG_ONLINE
   cout << "Debuge Info => copy constructor of AliGoofieRawEvent used, status code = 0" << flush << endl;
#endif
}

AliGoofieRawEvent & AliGoofieRawEvent::operator=(const AliGoofieRawEvent &_data)
{
   if(this!=&_data) {
      fLocalCopyOfNumberOfADCSampels = _data.fLocalCopyOfNumberOfADCSampels;

      fGoodEvent                     = _data.fGoodEvent;
      fEventTime                     = _data.fEventTime;
      fTriggerChannel                = _data.fTriggerChannel;
      // set to zero slowControl variable
      fSlowControl.sPress            = _data.fSlowControl.sPress;
      fSlowControl.sTemp1            = _data.fSlowControl.sTemp1;
      fSlowControl.sTemp2            = _data.fSlowControl.sTemp2;
      fSlowControl.sEfVoltage        = _data.fSlowControl.sEfVoltage;
      fSlowControl.sHVPickup         = _data.fSlowControl.sHVPickup;
   
      if(_data.fDataPickup &&  _data.fDataNear && _data.fDataFar) {
	 if(!fDataPickup) {
	    fDataPickup = new Int_t[fLocalCopyOfNumberOfADCSampels];
	 }
	 if(!fDataNear) {
	    fDataNear   = new Int_t[fLocalCopyOfNumberOfADCSampels];
	 }
	 if(!fDataFar) {
	    fDataFar    = new Int_t[fLocalCopyOfNumberOfADCSampels];
	 }
	 for(int i=0; i<(fLocalCopyOfNumberOfADCSampels); ++i) {
	    fDataPickup[i] = _data.fDataPickup[i];
	    fDataNear[i]   = _data.fDataNear[i];
	    fDataFar[i]    = _data.fDataFar[i];
	 }
      } else {
	 cerr << "Error in <AliGoofieRawEvent::operator=(const AliGoofieRawEvent &)>: Approaching to read memory segment which is out of program scope" << endl;
      }
   }
#ifdef DEBUG_ONLINE
   cout << "Debuge Info => overloaded = operator of AliGoofieRawEvent used, status code = 0" << flush << endl;
#endif
   return *this;
}


AliGoofieRawEvent::~AliGoofieRawEvent()
{ 
   // delete 
   if(fDataPickup) {
      delete [] fDataPickup;
      fDataPickup = NULL;
   }
   if(fDataNear) {
      delete [] fDataNear;
      fDataNear = NULL;
   }
   if(fDataFar) {
      delete [] fDataFar;
      fDataFar = NULL;
   }
#ifdef DEBUG_ONLINE
   cout << "Debuge Info => AliGoofieRawEvent object destroyed, status code = 0" << flush << endl;
#endif
}


void AliGoofieRawEvent::Clear()
{
   // spectrum arreies
   for(int i=0; i<(fLocalCopyOfNumberOfADCSampels); ++i) {
      if(fDataPickup) {
	 fDataPickup[i] = 0;
      }
      if(fDataNear) {
	 fDataNear[i]   = 0;
      }
      if(fDataFar) {
	 fDataFar[i]    = 0;
      }
   }
   fGoodEvent              = 0;
   fEventTime              = 0;
   fTriggerChannel         = 0;
   // set to zero slowControl variable
   fSlowControl.sPress     = 0.0;
   fSlowControl.sTemp1     = 0.0;
   fSlowControl.sTemp2     = 0.0;
   fSlowControl.sEfVoltage = 0.0;
   fSlowControl.sHVPickup  = 0.0;
}


void AliGoofieRawEvent::CreateArrays(const Int_t _size)
{
   fLocalCopyOfNumberOfADCSampels = _size;
   try {
      if(!fDataPickup) {
	 fDataPickup = new Int_t[fLocalCopyOfNumberOfADCSampels];
      } else throw (string)"Pickup";
      if(!fDataNear) {
	 fDataNear   = new Int_t[fLocalCopyOfNumberOfADCSampels];
      } else throw (string)"Near";
      if(!fDataFar) {
	 fDataFar    = new Int_t[fLocalCopyOfNumberOfADCSampels];
      } else throw (string)"Far";
   } catch(string _code) {
      cerr << "Warrning in <AliGoofieRawEvent::CreateArrays(int _size)>: Pointer is already set to data buffor, call skiped" << flush << endl;
   } catch(...) {
      cerr << "Error in <AliGoofieRawEvent::CreateArrays(int _size)>: Dynamic memory allocation failed " << flush << endl;
   }
}

Int_t AliGoofieRawEvent::GetEntryInDataNear(Int_t i)  const
{
  if(i>=0 && i<fLocalCopyOfNumberOfADCSampels) {
    return fDataNear[i]; 
  } else {
    return -1;
  }
}
 
Int_t AliGoofieRawEvent::GetEntryInDataFar(Int_t i) const
{
  if(i>=0 && i<fLocalCopyOfNumberOfADCSampels) {
    return fDataFar[i];
  } else {
    return -1;
  }
}

Int_t AliGoofieRawEvent::GetEntryInDataPickup(Int_t i) const 
{
  if(i>=0 && i<fLocalCopyOfNumberOfADCSampels) {
    return fDataPickup[i];
  } else {
    return -1;
  }
}
