//----------------------------------------------------------------------------
//  Header: AliGoofieRawEvent.h, v 1.0                        2007/10/22    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef ALIGOOFIERAWEVENT
#define ALIGOOFIERAWEVENT

#include <TObject.h>

class AliGoofieRawEvent : public TObject
{
public:
  struct AliGoofieSlowControl {
      Float_t sPress;                    // pressure
      Float_t sTemp1;                    // temperature from sensor 1 inside GOOFIE
      Float_t sTemp2;                    // temperature from sensor 2 inside GOOFIE
      Float_t sEfVoltage;                // voltage of the last resistor inside GOOFIE
      Float_t sHVPickup;                 // voltage applied to the pickup detector
   };
   
   AliGoofieRawEvent();
   AliGoofieRawEvent(const Int_t _size);
   AliGoofieRawEvent(const AliGoofieRawEvent &_data);
   virtual ~AliGoofieRawEvent();
   AliGoofieRawEvent & operator= (const AliGoofieRawEvent &_data);
   
   virtual void Clear(); 

   Int_t    GetEntryInDataNear(Int_t i)    const;
   Int_t    GetEntryInDataFar(Int_t i)     const; 
   Int_t    GetEntryInDataPickup(Int_t i)  const;
   Int_t    GetEventTime()                 const   {return fEventTime;}        
   Float_t  GetHVPickup()                  const   {return fSlowControl.sHVPickup;}
   Float_t  GetEfVoltage()                 const   {return fSlowControl.sEfVoltage;}
   Float_t  GetPressure()                  const   {return fSlowControl.sPress;}
   Float_t  GetTemp1()                     const   {return fSlowControl.sTemp1;}
   Float_t  GetTemp2()                     const   {return fSlowControl.sTemp2;}
   Int_t    GetTriggerChannel()            const   {return fTriggerChannel;}

   Bool_t   GetEventAnalyzedStatus()               const   {return this->TestBit(BIT(14));}          // true-already analysed, false-no yet analyzed
   void     SetEventAnalyzedStatus(Bool_t _status)         {this->SetBit(BIT(14), _status);}         // set bit value

   void     SetArraySize(const Int_t _size)         {fLocalCopyOfNumberOfADCSampels = _size;}
   void     CreateArrays(const Int_t _size); 
   
public:
   //service members
   Int_t fLocalCopyOfNumberOfADCSampels; // copy of setup variable

   // event quality
   Int_t  fGoodEvent;                    // quality of event is based on ???? (not ready)
   // triger information
   Int_t  fEventTime;                    // time set via gettimeofday, synch with local clock only
   Int_t  fTriggerChannel;               // wich channel trigger Near/Far
   // envarioment conndition
   struct AliGoofieSlowControl fSlowControl;   // Temp1, Temp2, pressure, Field Voltage, HV Pickup
   // spectrum arreies
   Int_t *fDataPickup;                   //[fLocalCopyOfNumberOfADCSampels] Bufor for FACD counts, pick-up
   Int_t *fDataNear;                     //[fLocalCopyOfNumberOfADCSampels] Bufor for FACD counts, Near
   Int_t *fDataFar;                      //[fLocalCopyOfNumberOfADCSampels] Bufor for FACD counts, Far
    
   ClassDef(AliGoofieRawEvent, 1)        //AliGoofieRawEvent structure
};
#endif
