#ifndef __CINT__
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <unistd.h>
#include <TROOT.h>
#include <TFile.h>
#include <TTree.h>
#include <TStyle.h>
#include <TString.h>
#include <TH1.h>
#include <TF1.h>
#include <TH2.h>
#include <TStopwatch.h>
#include <TTimeStamp.h>
// include header file for goofie root data
#include "AliGoofieRawEvent.h"
#include "AliGoofieGasResult.h"
#include "AliGoofiePack.h"

using namespace std;
#endif

int loop(TString input_ff, TString output_ff);

#ifndef __CINT__
int main(int argc, char *argv[])
{
   TString FileInput,FileOutput;
   Int_t ok=-1;
   
   if(argv[1] != NULL) {
      FileInput = argv[1];
      if(argv[2] != NULL) {
	 FileOutput = argv[2];
	 ok = loop(FileInput, FileOutput);
      } 
   }
   if(ok != 0) {
      cout << "Usage: goofie_file [inputfile] [outputfile] " << endl;
      cout << "    where  inputfile  is a 'goofie' root file \n"
	   << "    where  outputfile is the path and the name of file where the result\n" 
	   << "                       of the application should be save\n" <<endl; 
   }
   return 0;
}
#endif


int loop(TString input_ff, TString output_ff)
{
   TStopwatch timer;               //Timer
   Int_t printoutFrequency = 1;  
   Int_t entryOk           = 0;

   int   TimeOffset        = 1197400000;

   //Create Output File with histograms
   TFile *f1= TFile::Open(output_ff, "RECREATE"); 
   if(f1->IsZombie()) {
      return -100;
   }
   
   int   fTimeHistValues[3] = {2000, 70000, 90000};
   // pack info
   TH1D *hTTimeStamp              = new TH1D("hTTimeStamp",     "Data aquisition time [sec]", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2]);
   TH1D *hNumberOfEvents          = new TH1D("hNumberOfEvents", "Number of events", 5000, 0.0, 5000);
   // Gas results
   TH2D *hGasResFieldCageHV       = new TH2D("hGasResFieldCageHV" ,"Field Voltage vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 1600, -15000, 1000);
   TH2D *hGasResPickupHV          = new TH2D("hGasResPickupHV", "Pickup Voltage vs. Time",  fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 1000, 700, 1700);     
   TH2D *hGasResPeakPosNear       = new TH2D("hGasResPeakPosNear", "Near peak position vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 512, 0, 512);     
   TH2D *hGasResPeakPosFar        = new TH2D("hGasResPeakPosFar", "Far peak position vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 512, 0, 512);     
   TH2D *hGasResPeakAreaNear      = new TH2D("hGasResPeakAreaNear", "Near area vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 1000, 0.0, 10000.0);     
   TH2D *hGasResPeakAreaFar       = new TH2D("hGasResPeakAreaFar", "Far area vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 1000, 0.0, 10000.0);     
   TH2D *hGasResTemp1             = new TH2D("hGasResTemp1","Temp1 vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 700, 0.0, 350.0);  
   TH2D *hGasResTemp2             = new TH2D("hGasResTemp2","Temp2 vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 700, 0.0, 350.0);      
   TH2D *hGasResPressure          = new TH2D("hGasResPressure","Pressure vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 400, 850.0, 1050.0);  
   TH2D *hGasResGainNear          = new TH2D("hGasResGainNear","Near Gain vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 1000, 0.0, 10000.0);
   TH2D *hGasResGainFar           = new TH2D("hGasResGainFar","Far Gain vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 1000, 0.0, 10000.0);     
   TH2D *hGasResFractionCO2       = new TH2D("hGasResFractionCO2", "CO2 vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 1500, 0.0, 15.0);     
   TH2D *hGasResFractionN2        = new TH2D("hGasResFractionN2", "N2 vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 1500, 0.0, 15.0);     
   TH2D *hGasResDriftVelocity     = new TH2D("hGasResDriftVelocity", "Drift velocity vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 3000, 1.0, 3.0);        
   TH2F *hGasResNumberOfDischarge = new TH2F("hGasResNumberOfDischarge", "Discharges vs. Time", fTimeHistValues[0], fTimeHistValues[1], fTimeHistValues[2], 1000, 0.0, 10000);
   // events
   TH1D *specPickup               = new TH1D("sperPickup","Pickup", 513, -0.5, 512.5);
   TH1D *specNear                 = new TH1D("specNear","Near", 513, -0.5, 512.5);   
   TH1D *specFar                  = new TH1D("specFar","Far", 513, -0.5, 512.5);
   
   TFile *hfile = NULL;
   TTree *tree  = NULL;
   //Open File  
   hfile = new TFile(input_ff);
   tree = (TTree*)hfile->Get("T");
   
   timer.Start();

   //Set address of CPack
   AliGoofiePack *pack = new AliGoofiePack();   //we create the event object once 
   tree->SetBranchAddress("Pack", &pack);
   tree->SetBranchStatus("*", 1);
   
   Int_t nevent = (Int_t) tree->GetEntries();

   for (Int_t ev=0; ev<nevent; ++ev) {
   // *************** main loop ***************
      if( ev>0 )
	 pack->Clear();
      
      entryOk = tree->GetEvent(ev);
      if(entryOk < 0) {
	 cout << "Error event number = " <<  ev << endl; 
	 continue;  
      }
      if(ev%printoutFrequency == 0) {
	 cout<<"analyzing event::"<< ev<<endl;
	 cout << "number of events per pack : " << pack->GetNumberOfEvents() << endl;
      }

      int localTime = pack->GetTimeStamp()->GetSec() - TimeOffset;

      cout << "local time = " << localTime << endl;
      // fill pack and gas results
      hTTimeStamp->Fill(localTime);
      hNumberOfEvents->Fill(pack->GetNumberOfEvents());
      hGasResFieldCageHV->Fill(localTime, pack->GetGasResults()->fFieldCageHV);    
      hGasResPickupHV->Fill(localTime, pack->GetGasResults()->fPickupHV);         
      hGasResPeakPosNear->Fill(localTime, pack->GetGasResults()->fPeakPosNear);        
      hGasResPeakPosFar->Fill(localTime, pack->GetGasResults()->fPeakPosFar);      
      hGasResPeakAreaNear->Fill(localTime, pack->GetGasResults()->fPeakAreaNear);       
      hGasResPeakAreaFar->Fill(localTime, pack->GetGasResults()->fPeakAreaFar);      
      hGasResTemp1->Fill(localTime, pack->GetGasResults()->fTemp1);      
      hGasResTemp2->Fill(localTime, pack->GetGasResults()->fTemp2);      
      hGasResPressure->Fill(localTime, pack->GetGasResults()->fPressure);      
      hGasResGainNear->Fill(localTime, pack->GetGasResults()->fGainNear);     
      hGasResGainFar->Fill(localTime, pack->GetGasResults()->fGainFar);     
      hGasResFractionCO2->Fill(localTime, pack->GetGasResults()->fFractionCO2);     
      hGasResFractionN2->Fill(localTime, pack->GetGasResults()->fFractionN2);     
      hGasResDriftVelocity->Fill(localTime, pack->GetGasResults()->fDriftVelocity);          
      hGasResNumberOfDischarge->Fill(localTime, pack->GetGasResults()->fNumberOfDischarge); 

      // loop over events
      for(int eventID=0; eventID<pack->GetNumberOfEvents(); ++eventID) {
	 AliGoofieRawEvent *event = pack->GetCRawEvent(eventID);

	 cout << "event " << eventID << ", time = " << event->fEventTime << endl; 

	 for(int bin=0; bin<event->fLocalCopyOfNumberOfADCSampels; ++bin) {
	    specFar->AddBinContent(bin+1, event->GetEntryInDataFar(bin));
	    specNear->AddBinContent(bin+1, event->GetEntryInDataNear(bin));  
	    specPickup->AddBinContent(bin+1, event->GetEntryInDataPickup(bin));
	 }
      }
      cout << "--------------- after loop ------------------- " << endl;
   }

   
   timer.Stop();
   Double_t rtime = timer.RealTime();
   Double_t ctime = timer.CpuTime();
   cout << "RealTime="<< rtime << " seconds, CpuTime=" <<ctime<< " seconds" << endl;
   cout<<"done with the loop"<<endl;    
   // Write ALL Histograms 
   gROOT->GetList()->Write();
   f1->Write();
   // Close File
   cout<<"after 1st histo"<<endl;
   cout<<"Going to close root HISTO file"<<endl;
   f1->Close();

   return 0;
}
