//----------------------------------------------------------------------------
//  Header: CDIMServer.h, v 1.0                               2007/07/18    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <list>
#include <string>
#include <dis.hxx>

#include "CGasAnaResults.h"

class CDIMServer : public DimServer
{
public:
   CDIMServer();
   virtual ~CDIMServer();

   void  SetDataPointer(const CGasAnaResults *_res);                // set pointer to analysis results
   int   CreateServer(std::string _serverName);                     // create server (open conetcion with DNS) 
   int   RunServer();                                               // update data point, FMS
   int   Recovery();                                                // recovery for error
   int   StopServer();                                              // stop dim server
   // action
   int   GetAction()          {return fkAction;}                    // get current action value
   void  ResetAction()        {fkAction=0;}                         // set action value to zero
   // states
   int   GetState()           {return fkState;}                     // get current state value
   void  SetState(int _val)   {fkState=_val;}                       // set state value to zero

public:
   static const int kActionStop;                                    // actions definition
   static const int kActionStart;
   static const int kActionGoOn;
   static const int kActionGoOff;
   static const int kActionRecover;
   static const int kStateOff;                                      // states definition
   static const int kStateOn;
   static const int kStateError;

private:

   int CreateServerNameSpace();               // create all data pionts and commands
   int UpdateValues();                        // update data points with values from CGasAnaResults.
   int UpdateStates();
   // virtual
   void commandHandler();                     // goofie dim command handler
   void SetToNullDataPoint();                 //

private:
   // DIM data points
   CGasAnaResults fDimDataPoints;             // dim server Data Points

   // DIM service data point
   DimService   *fsFieldCageHV;               // field voltage in monitor read through last resistor
   DimService   *fsPickupHV;                  // anod voltage on pickup detector
   DimService   *fsPeakPosNear;               // near peak position in pickup spectrum
   DimService   *fsPeakPosFar;                // far peak position in pickup spectrum
   DimService   *fsPeakAreaNear;              // near peak area in pickup spectrum
   DimService   *fsPeakAreaFar;               // far peak area in pickup spectrum
   DimService   *fsTemp1;                     // temperature from sensor 1
   DimService   *fsTemp2;                     // temperature from sensor 2
   DimService   *fsPressure;                  // pressure inside monitor
   DimService   *fsGainNear;                  // gain calculated using Near area
   DimService   *fsGainNearCorrected;         // gain calculated using Near area corected for P/T U variations
   DimService   *fsGainFar;                   // gain calculated using Far area
   DimService   *fsGainFarCorrected;          // gain calculated using Far area corected for P/T U variations
   DimService   *fsFractionCO2;               // CO_2 fraction in gas mixture
   DimService   *fsFractionN2;                // N_2 fraction in gas mixture
   DimService   *fsDriftVelocity;             // drift velocity of electron
   DimService   *fsDriftVelocityCorrected;    // drift velocity of electron corected for P/T U variations
   DimService   *fsCorrectionFactor;          // calculated from P/T U variations
   DimService   *fsNumberOfDischarge;         // how many discharges happend during this time window

   //list of dataPoints
   std::list<DimService*>           fListOfDataPoints;       // for internal handling
   std::list<DimService*>::iterator fitForListOfDataPoint;   // list iterator

   // DIM FMS
   // states
   DimService   *fskState;
   //value
   int           fkState;
   // actions
   DimCommand   *fckAction;
   // value
   int           fkAction;

   // exchange buffer
   const CGasAnaResults  *fGasAnalysisResults;

   // server name for recovery
   std::string            fserverName;
};
