//----------------------------------------------------------------------------
//  Header: COnLineAna.h, v 1.0                               2007/04/16    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef CONLINEANA
#define CONLINEANA

#include <pthread.h>
#include <deque>
#include <string>

class CService;                     // for exchanging pack between readout part
class COnLineExeption;              // for error\exeption handling
class CDataStructure;               // wrapper for data structure class
class CDIMServer;                   // dim server object
class AliGoofieGasAnalysis;         // gas analysis object
class CGasAnaResults;               // results buffer
class CRootFileHandler;             // storing raw data in root file
class AliGoofieRawEvent;            // data structure class


class COnLineAna 
{ 
 public:
   COnLineAna();
   COnLineAna(CService *service);
   virtual ~COnLineAna();
  
   static void *MakeAnalysis(void* arg);  // thread function
   pthread_t   *StartONA(const std::string _configFile);

   void GetAndAnalysePack()                                  throw(COnLineExeption);
   void InitializeSoftware(const std::string _configFile="") throw(COnLineExeption);
   void MoveResultsToCSDimServer()                           throw(COnLineExeption);
   void FillAndCopyRootFile()                                throw(COnLineExeption);
   
   std::string DumpVectorToString();

 public:
   //thread
   pthread_t                       tid_ona;
   
   // for wrapper class
   CService                       *fWrapper;
   
   // dim data exchange server
   CDIMServer                     *fDimServerHandler;

   //gas analysis
   AliGoofieGasAnalysis           *fGasAnalysis;
 
   // results of gas analysis - local bufor 
   CGasAnaResults                 *fGasAnalysisResults;

   // raw data handler
   CRootFileHandler               *fRootFileHandler;
   
   // error\exeption status
   COnLineExeption                *fExeptionStatus;
   
   // vector of RAW events --> stored additionally in files
   std::deque<CDataStructure>     *fPack;

   // vector of event used for analysis -- events could be used repeatedly
   std::deque<AliGoofieRawEvent>   fAnalysisPack;
   
 private:
   //service members
   int fLocalCopyOfEventsInPack;
   int fCurrentPackNumber;             // to know how many pack has been analysed 
};

#endif
