//----------------------------------------------------------------------------
//  Body: CReadOutBoards.cpp, v 1.0                           2007/03/14    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <cmath>
#include <string>
#include <time.h>

#include "CMacroDef.h"
#include "COnLineExeption.h"
#include "SConfigData.h"
#include "CDataStructure.h"
#include "CGoofieBoard.h"
#include "CReadNI.h"
#include "CService.h"
#include "CReadOutBoards.h"

using namespace std;


// TODO:
// 1 - finishing function pHandlerROB(void *arg)
// 2 - finishing CReadOutBoards::UpDateHardwareState()
// 3 - finishing CReadOutBoards::UpDateSoftwareState()
// 4 - finishing CReadOutBoards::SimulateHardware(CDataStructure *_eventStr)
//

// ----------- Global function ----------------------------------------------------
void fgCleanupHandlerROB(void *arg)
{
   //CReadOutBoards *fRB = reinterpret_cast<CReadOutBoards*>(arg);
   
   DEBUGINFO("Debuge Info => CReadOutBoards::fgCleanupHandlerROB =>  delete all objects used by the thread",0)
}

// ------------- START - static functions - START ---------------------------------
//thread function
void *CReadOutBoards::ReadoutEvents(void* arg)
{
   CReadOutBoards *fRB = reinterpret_cast<CReadOutBoards*>(arg);
   
   // stop data aquisition
   int laquisition = 1;
   
   // set cleanup method
   pthread_cleanup_push(fgCleanupHandlerROB, arg);
   // set cancelation state
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
   // wait while creation of thread is over.
   usleep(50000);
   
   try {   
     while(laquisition) {
	DEBUGINFO("------------------------------- NEXT EVENT --------------------------------",0)
	// read data from hardware
	fRB->FillEventIntoVector();
       
	// is a pack ready to copy?
	if(fRB->MovePackToCServiceAndChecksUpdates()) {
	   // cancelation point
	   pthread_testcancel();
	   // check whether aquisition is still require
	   // only after full pack was collected
	   laquisition = fRB->fpwrapper->GetRunROBStatus();
	}
	
#ifdef NO_HARDWARE_REQUIRED
	usleep(50000);
#endif
	
     }
   }
   catch(COnLineExeption *eVal) {
      // for debuging only
      DEBUGINFO("Debuge Info => CReadOutBoards::ReadoutEvents => Exeption !!!! ",0)
      fRB->fexeptionStatus->PrintExeptionError();
   }
   //call clean up method
   pthread_cleanup_pop(1);
   
   return NULL;
}
// ------------- STOP - static functions - STOP ---------------------------------


//---------constructor----------
CReadOutBoards::CReadOutBoards(CService *service)
{
   // set pointer for CService obj.
   fpwrapper = service;

   // error / exeption handling
   fexeptionStatus  = new COnLineExeption();

   // objects for reading hardware
   fRAWdata         = NULL;
   fSCdata          = NULL;
   
   // storing place for events
   fPack            = new  deque<CDataStructure>(SConfigData::fEventsInPack);
   fNumberEvents    = 0;

   // local copy to avoid access to the global variable
   // by two or more threads at the same time
   fLocalCopyOfEventsInPack = SConfigData::fEventsInPack;

   // configuartion files initialization state
   fInitializationState = false;
   
   DEBUGINFO("Debuge Info => CReadOutBoards object created",0)
}


void CReadOutBoards::InitializeHardware(const string _configFile) throw (COnLineExeption) 
{
   try {
#ifndef NO_HARDWARE_REQUIRED
      // create objects for reading hardware
      if(!fRAWdata) {
	 fRAWdata = new CGoofieBoard();
      }
      if(!fSCdata) {
	 fSCdata  = new CReadNI();
      }
      // GOOFIE board
      if(fRAWdata->InitializeCard(_configFile)) throw int(1);
      // NI board
      if(fSCdata->InitializeCard(_configFile)) throw int(2);
#else
      fRAWdata = NULL;
      fSCdata  = NULL;
      DEBUGINFO("Debuge Info => CReadOutBoards::InitializeHardware => Running in simulation hardware mode",err)
#endif
   } catch (int _err) {
      fexeptionStatus->SetExeptionError(_err, "<CReadOutBoards::InitializeHardware()>: Initialization of cards failed");
      throw fexeptionStatus;
   } catch (...) {
      fexeptionStatus->SetExeptionError(0, "<CReadOutBoards::InitializeHardware()>: Initialization of cards failed");
      throw fexeptionStatus;
   }
}

pthread_t *CReadOutBoards::StartROB(const std::string _configFile)
{
   unsigned int err = 0;
   try {
      // statring comunication with hardware and checking connections
      if(!fInitializationState) {
	 InitializeHardware(_configFile);  // throw COnLineExeption
	 fInitializationState = true;
      } else {
	 fInitializationState = false;
	 DEBUGINFO("Debuge Info => CReadOutBoards::StartROB => hardware already initialized",0)
      }
      pthread_attr_t  attr_rob;
      // setting thread attribute
      err = pthread_attr_init(&attr_rob);
      DEBUGINFO("Debuge Info => CReadOutBoards::StartROB => thread attribute initialization",err)
      if(err) throw err;

      // attribute ...
      pthread_attr_setdetachstate(&attr_rob, PTHREAD_CREATE_JOINABLE);
      // stack size
      size_t stacksize;
      err = pthread_attr_getstacksize (&attr_rob, &stacksize);
      DEBUGINFO("Debuge Info => CReadOutBoards::StartROB => getting stack size",err)
      if(err) throw err;
      stacksize = (size_t)4.0*stacksize;
      err = pthread_attr_setstacksize (&attr_rob, stacksize);
      DEBUGINFO("Debuge Info => CReadOutBoards::StartROB => setting new stack size",err)
      if(err) throw err;
      
      // create thread
      err = pthread_create(&tid_rob, &attr_rob, ReadoutEvents, (void*)this);
      DEBUGINFO("Debuge Info => CReadOutBoards::StartROB => thread created",err)
      if(err) throw err;
      
      // delete thread attribute obj
      err = pthread_attr_destroy(&attr_rob);
      DEBUGINFO("Debuge Info => CReadOutBoards::StartROB => thread attribute destroyed",err)
      if(err) throw err;
      
   } catch (unsigned int _err) {
      fexeptionStatus->SetExeptionError(_err, "<CReadOutBoards::StartROB()>: Initialization of thread failed");
      
   } catch (COnLineExeption _err) {
      _err.PrintExeptionError();
      // close object
      this->~CReadOutBoards();
   }
   return &tid_rob;
}

//---------destructor----------
CReadOutBoards::~CReadOutBoards() 
{
   // close ROB thread
   pthread_exit(NULL);
   
   if(fexeptionStatus) {
      delete fexeptionStatus;
      fexeptionStatus = NULL;
   }
   if(fRAWdata) {
      delete fRAWdata;
      fRAWdata = NULL;
   }
   if(fSCdata) {
      delete fSCdata;
      fSCdata = NULL;
   }
   if(fPack) {
      fPack->clear();
      delete fPack;
      fPack=NULL;
   }
   DEBUGINFO("Debuge Info => CReadOutBoards object destroyed",0)
}

void CReadOutBoards::FillEventIntoVector() throw(COnLineExeption)
{
   try {
      
      // get element from vector
      CDataStructure *lOneEvent = &fPack->at(fNumberEvents++);

      // set value to CDataStructure from hardware
#ifndef NO_HARDWARE_REQUIRED
      unsigned int err = fRAWdata->GetEvent(lOneEvent);
      if(err) throw err;
      // update NI information after ?? goofie's events
      err = fSCdata->GetSCData(lOneEvent, (!((fNumberEvents-1)%SConfigData::fNIUpdataRate)) );
      if(err) throw err;
#else
      DEBUGINFO("Debuge Info => CReadOutBoards::FillEventIntoVector => setting expected values",err)
      SimulateHardware(lOneEvent); 
#endif    
      DEBUGINFO("Debuge Info => CReadOutBoards::FillEventIntoVector => Event number   = ",fNumberEvents)
      DEBUGINFO("Debuge Info => CReadOutBoards::FillEventIntoVector => Trigger Channe = ",lOneEvent->fTriggerChannel)
      DEBUGINFO("Debuge Info => CReadOutBoards::FillEventIntoVector => NI temp. A     = ",lOneEvent->fSlowControl.sTempA)
      
   } catch(unsigned int _err) {
      // set error code
      fexeptionStatus->SetExeptionError(_err, "<CReadOutBoards::FillEventIntoVector()>: Filling of vector failed");
      throw fexeptionStatus;
      
   } catch(const std::exception &_err) {
      string errorInfo =  "<CReadOutBoards::FillEventIntoVector()>: "; errorInfo +=_err.what();
      fexeptionStatus->SetExeptionError((unsigned int)0x0002000, errorInfo.c_str());
      throw fexeptionStatus;  
   }
}

int CReadOutBoards::MovePackToCServiceAndChecksUpdates() throw(COnLineExeption)
{
   int ret=0;
   if(fNumberEvents>=fLocalCopyOfEventsInPack) {
      
      unsigned int err = fpwrapper->AddPackToQueue(fPack);
      if(err) {
	 fexeptionStatus->SetExeptionError(err, "<CReadOutBoards::MovePackToCService()>: Moving Pack to CService fail");
	 throw fexeptionStatus;
      }
      DEBUGINFO("Debuge Info => CReadOutBoards::MovePackToCServiceAndChecksUpdates => pack moved to CService",err)

      // clear pack
      CleanEventsInVector();
      DEBUGINFO("Debuge Info => CReadOutBoards::MovePackToCServiceAndChecksUpdates => all value in the pack reset to zero",0)

      // are there some requests from user side ?
      UpDateHardwareState();
      
      UpDateSoftwareState();
      ret = 1;
   }
   return ret;
}

void CReadOutBoards::CleanEventsInVector() throw(COnLineExeption)
{
   // clear counter
   fNumberEvents=0;
   
   //this part is time consuming, eventual one can removed it
   //for(int i=0; i<fLocalCopyOfEventsInPack; ++i) {
   //   (&fPack->at(i))->Clear();
   //}
}



// service members are executed only when pack is ready
void CReadOutBoards::UpDateHardwareState() throw(COnLineExeption)
{
   if(fpwrapper->ConfigureHardwareState()) {
      try {
	 

	 
      } catch (...) {
	 fexeptionStatus->SetExeptionError(0x0001100, "<CReadOutBoards::UpDateHardwareState()>: some hardware problems");
	 throw fexeptionStatus;
      }
      DEBUGINFO("Debuge Info => CReadOutBoards::UpDateHardwareState => all necessary registers has been modified", 0)
   }
}

void CReadOutBoards::UpDateSoftwareState() throw(COnLineExeption)
{
   if(fpwrapper->ConfigureSoftwareState()) {
      try {
	 //  update number of event store per one pack
	 if(fpwrapper->UpDateNumberOfEventInPack()) {
	    if(fPack) {
	       fPack->clear();
	       delete fPack;
	       fPack = NULL;
	    }
	    // get new value
	    fLocalCopyOfEventsInPack = fpwrapper->GetNumberOfEventsInPack();
	    // create new vector with proper size
	    fPack = new deque<CDataStructure>(fLocalCopyOfEventsInPack);

	    
	 }
      }  catch (...) {
	 fexeptionStatus->SetExeptionError(0x0001200, "<CReadOutBoards::UpDateSoftwareState()>: some software problems");
	 throw fexeptionStatus;
      }
      DEBUGINFO("Debuge Info => CReadOutBoards::UpDateSoftwareState() => all necessary values has been modified", 0)
   }
}



void CReadOutBoards::SimulateHardware(CDataStructure *_eventStr)
{
   

}
