//----------------------------------------------------------------------------
//  Header: CReadOutBoards.h, v 1.0                           2007/03/14    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <deque>
#include <pthread.h>

class CDataStructure;  // event data structure class 
class CReadNI;         // for getting information from NI board
class CGoofieBoard;    // for getting information from Goofie board
class CService;        // for exchanging pack between online part
class COnLineExeption; // for error\exeption handling

class CReadOutBoards 
{
   friend void fgCleanupHandlerROB(void *arg);
   
 public:
   // --- constructor \ destructor --
   CReadOutBoards(CService *service);
   virtual ~CReadOutBoards();

   void InitializeHardware(const std::string _configFile="") throw(COnLineExeption);
   void FillEventIntoVector()                                throw(COnLineExeption);
   void CleanEventsInVector()                                throw(COnLineExeption);
   int  MovePackToCServiceAndChecksUpdates()                 throw(COnLineExeption);
   void UpDateHardwareState()                                throw(COnLineExeption);
   void UpDateSoftwareState()                                throw(COnLineExeption);
   
   static void *ReadoutEvents(void* arg);                    // thread function
   pthread_t   *StartROB(const std::string _configFile="");  // start thread
   
public:
   pthread_t                    tid_rob;           // thread object
   CService                    *fpwrapper;         // for wrapper class
   COnLineExeption             *fexeptionStatus;   // error\exeption status
   std::deque<CDataStructure>  *fPack;             // vector of events
   int                          fNumberEvents;     // counter for incoming events from GOOFIE board
   CGoofieBoard                *fRAWdata;          // object for handling data from GOOFIE board (signals)
   CReadNI                     *fSCdata;           // object for handling data from NI board     (T/P/V etc..)

private:
   // test functions
   void SimulateHardware(CDataStructure *_eventStr);
   // service members
   int fLocalCopyOfEventsInPack;
   // config files initialization state
   bool fInitializationState;
   
};
