//----------------------------------------------------------------------------
//  Header: CRootFileHandler.h, v 1.0                         2007/12/07    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <deque>
#include <string>
#include "TTimeStamp.h"

class TFile;
class TTree;
class TBranch;
class TDirectory;

class CGasAnaResults;
class CDataStructure;
class AliGoofiePack;
class COnLineExeption;

class CRootFileHandler
{
public:
   CRootFileHandler();
   ~CRootFileHandler();

   void Initialize();
   void AddPackToTree(const CGasAnaResults *_GasAnaResults, const std::deque<CDataStructure> &_fPack);
   void CreateOutputFile();
   void SaveDataLocally();
   void SendDataToDAQ();
   void CopyPack(const std::deque<CDataStructure> &_fPack);
   // test
   void AddPackToTree();
   
private:
   TTimeStamp                       fCurrTime;               // get current system time via calling Set() function 
   TDirectory                      *fHandlerForCurrentDir;   // current directory before creating folder for tree
   TFile                           *fOutputRootFile;
   TTree                           *fPackTree;
   TBranch                         *fPackBranch; 
   AliGoofiePack                   *fPackObj;                // tree template

   std::string                      fCurrentFileName;        // current file name, template 

   bool                             fFileIsReady;            // check whether file is ready to be store

   int                              fCompresionLevel;        // root file properties
   int                              fBufferSize;
   int                              fSplittingValue;
   int                              fBranchStyle;
   
   std::deque<AliGoofieRawEvent>   *fPackLocalCopy;          // temporary local storage

   COnLineExeption *fexeptionStatus;                         // error / exeption handling
};
