//----------------------------------------------------------------------------
//  Body: CService.cpp, v 1.0                                 2007/03/14    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <unistd.h>
#include <errno.h>

#include "CMacroDef.h"
#include "COnLineExeption.h"
#include "CDataStructure.h"
#include "SConfigData.h"
#include "CService.h"

// TODO
// 1) Alert implementation
//


using namespace std;


CService::CService()
{
   fConfigureHardware            = 0;
   fConfigureSoftware            = 0;
   fUpDateNumberOfEventInPack    = 0;
   // default initialize local value from config class
   fCurrentNumbersOfEventsInPack = SConfigData::fEventsInPack;

   // create configuration object;
   fSoftwareConfigObj = new SConfigData();

   // error / exeption handling
   fexeptionStatus  = new COnLineExeption();
  
   // stl
   FIFOData = new queue< deque<CDataStructure> >;

   // create mutex and semaphor obj for data exchange
   CreateMutexSemaphoreObj();

   // set action states (> 0 => run)
   fRunROB = 1;
   fRunONA = 1;

   // set alert state/ bit state/
   // 32-bit word
   // if all 0 then application works OK.
   // 0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0-0
   //                                                               1  - hardware failed, stop aquisition thread
   //                                                             1    - software failed,
   //                                                           1      - analysis part failed
   //                                                         1        - service part faild 
   fAlertState = 0;
}

CService::~CService()
{
   if(FIFOData) {
      //FIFOData->clear();
      delete FIFOData;
      FIFOData = NULL;
   }
   if(fexeptionStatus) {
      delete fexeptionStatus;
      fexeptionStatus = NULL;
   }
   sem_destroy(&fsemaphore);
}

unsigned int CService::InitializeSetUp(const string _FName)
{
  unsigned int error = 0;
  // initialize from file
  error = fSoftwareConfigObj->InitializeFromFile(_FName);

  // set local copy 
  fCurrentNumbersOfEventsInPack = fSoftwareConfigObj->fEventsInPack;

  return error;
}

void CService::CreateMutexSemaphoreObj()
{
   unsigned int err = 0;

   // mutex part
   try {
      pthread_mutexattr_t  lmutexarrt;
      // setting mutex attribute
      err = pthread_mutexattr_init(&lmutexarrt);
      DEBUGINFO("Debuge Info => CService::CreateMutexSemaphoreObj => mutex attribute initialization",err)
      if(err) throw err;
      // attribute ...
            
      // initialize mutex
      err = pthread_mutex_init(&fmutex, &lmutexarrt);
      DEBUGINFO("Debuge Info => CService::CreateMutexSemaphoreObj => initialize mutex",err)
      if(err) throw err;
      
      err = pthread_mutexattr_destroy(&lmutexarrt);
      DEBUGINFO("Debuge Info => CService::CreateMutexSemaphoreObj => attribute mutex destroyed",err)
      if(err) throw err;
      
   } catch (unsigned int _err) {
      fexeptionStatus->SetExeptionError(_err, "<CService::CreateMutexSemaphoreObj()>: Initialization of mutex failed");
   }

   // semaphore part
   try {
      err = sem_init(&fsemaphore, 0, 0);
      DEBUGINFO("Debuge Info => CService::CreateMutexSemaphoreObj => semaphore initialized",err)
      if(err) throw err;
      
   } catch (unsigned int _err) {
      fexeptionStatus->SetExeptionError(_err, "<CService::CreateMutexSemaphoreObj()>: Initialization of semaphore failed");
   }
}


// this function should by call only from CReadOutBoards class
unsigned int CService::AddPackToQueue(deque<CDataStructure> *_Pack)
{
   unsigned int err = 0;

   try {
      // lock queue
      err = pthread_mutex_lock(&fmutex);
      DEBUGINFO("Debuge Info => CService::AddPackToQueue => locking thread mutex",err)
      if(err) throw err;
      // adding vector to queue
      FIFOData->push(*_Pack);

      // semofore on 
      err = sem_post(&fsemaphore);
      DEBUGINFO("Debuge Info => CService::AddPackToQueue => semaphore increase by one pack",err)
      if(err) throw err;

      // semaphore size only for tests
      int value = 0;
      sem_getvalue(&fsemaphore, &value);
      DEBUGINFO("Debuge Info => CService::AddPackToQueue => semaphore size",value)

      // unlock queue
      err = pthread_mutex_unlock(&fmutex);
      DEBUGINFO("Debuge Info => CService::AddPackToQueue => unlocking thread mutex",err)
      if(err) throw err;
      
   } catch (unsigned int _err) {
      DEBUGINFO("Debuge Info => CService::AddtPackToQueue => process failed",_err)
   }
   return err;
}


// this function should by call only from COnlineAna class
unsigned int CService::GetPackFromQueue(deque<CDataStructure> *_Pack)
{
   unsigned int err = 0;
   try {
      DEBUGINFO("Debuge Info => CService::GetPackFromQueue => waitting for semaphore",0)
      err = sem_wait(&fsemaphore);
      DEBUGINFO("Debuge Info => CService::GetPackFromQueue => semaphore decrease by one pack",err)
      if(err) throw err;

      // try to local queue
      err = pthread_mutex_lock(&fmutex);
      DEBUGINFO("Debuge Info => CService::GetPackFromQueue => locking thread mutex",err)

      // get data, using = operator
      (*_Pack) = FIFOData->front();
      // remove vector from queue
      FIFOData->pop();

      // unlock queue
      err = pthread_mutex_unlock(&fmutex);
      DEBUGINFO("Debuge Info => CService::GetPackFromQueue => unlocking thread mutex",err)
      if(err) throw err;
      
   } catch (unsigned int _err) {
      DEBUGINFO("Debuge Info => CService::GetPackFromQueue => moving data to COnlineAna class failed",_err)
   }
   return err;
}



// hardware and software UpData members
// not ready
int CService::GetNumberOfEventsInPack()
{
   return fCurrentNumbersOfEventsInPack;
}

// actions functions
int  CService::GetRunROBStatus()
{
   int retValue = 0;
   // lock
   unsigned int err = pthread_mutex_lock(&fmutex);
   DEBUGINFO("Debuge Info => CService::GetRunROBStatus => locking exeption mutex",err)
   // copy to local buffer
   retValue = fRunROB;
   // unlock 
   err = pthread_mutex_unlock(&fmutex);
   DEBUGINFO("Debuge Info => CService::GetRunROBStatus => unlocking thread mutex",err)
      
   return retValue;
}

int  CService::GetRunONAStatus()
{
   int retValue = 0;
   // lock
   unsigned int err = pthread_mutex_lock(&fmutex);
   DEBUGINFO("Debuge Info => CService::GetRunONAStatus => locking exeption mutex",err)
   // copy to local buffer
   retValue = fRunONA;
   // unlock 
   err = pthread_mutex_unlock(&fmutex);
   DEBUGINFO("Debuge Info => CService::GetRunONAStatus => unlocking thread mutex",err)
      
   return retValue;
}

void  CService::SetRunROBStatus(int _val)
{
   // lock
   unsigned int err = pthread_mutex_lock(&fmutex);
   DEBUGINFO("Debuge Info => CService::SetRunROBStatus => locking exeption mutex",err)
   // set value
   fRunROB = _val;
   // unlock 
   err = pthread_mutex_unlock(&fmutex);
   DEBUGINFO("Debuge Info => CService::GetRunROBStatus => unlocking thread mutex",err)
}

void  CService::SetRunONAStatus(int _val)
{
   // lock
   unsigned int err = pthread_mutex_lock(&fmutex);
   DEBUGINFO("Debuge Info => CService::SetRunONAStatus => locking exeption mutex",err)
   // set value
   fRunONA = _val;
   // unlock 
   err = pthread_mutex_unlock(&fmutex);
   DEBUGINFO("Debuge Info => CService::SetRunONAStatus => unlocking thread mutex",err)
}

// Alert States
int CService::GetAlertState()
{
   int retValue = 0;
   // lock
   unsigned int err = pthread_mutex_lock(&fmutex);
   DEBUGINFO("Debuge Info => CService::GetAlertState => locking exeption mutex",err)
   // copy to local buffer
   retValue = fAlertState;
   // unlock 
   err = pthread_mutex_unlock(&fmutex);
   DEBUGINFO("Debuge Info => CService::GetAlertState => unlocking thread mutex",err)
      
   return retValue; 
}

void CService::SetAlertState(int _val)
{
   // lock
   unsigned int err = pthread_mutex_lock(&fmutex);
   DEBUGINFO("Debuge Info => CService::SetAlertState => locking exeption mutex",err)
   // set value
   fAlertState = _val;
   // unlock 
   err = pthread_mutex_unlock(&fmutex);
   DEBUGINFO("Debuge Info => CService::SetAlertState => unlocking thread mutex",err)
}
