//----------------------------------------------------------------------------
//  Header: CService.h, v 1.0                                 2007/04/16    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef CSERVICE
#define CSERVICE

#include <pthread.h>
#include <semaphore.h>
#include <string>
#include <queue>
#include <deque>

class CDataStructure;  // event data structure class 
class COnLineExeption; // for error\exeption handling
class SConfigData;     // for configuration file handling

class CService
{
public:
   // constructors/destructors
   CService();
   virtual ~CService();
   // initialization from files
   unsigned int InitializeSetUp(const std::string _FName);

   // Pack exchange functions
   //   inserting Pack to queue 
   unsigned int AddPackToQueue(std::deque<CDataStructure> *_Pack);
   //   getting Pack from queue
   unsigned int GetPackFromQueue(std::deque<CDataStructure> *_Pack);
   
   // hardware part
   bool ConfigureHardwareState()     {return fConfigureHardware;}         // 
   // software part
   bool ConfigureSoftwareState()     {return fConfigureSoftware;}         // 
   bool UpDateNumberOfEventInPack()  {return fUpDateNumberOfEventInPack;} // 
   int  GetNumberOfEventsInPack();                      // for updating the pack size
   
   // create mutex/semaphore
   void CreateMutexSemaphoreObj();

   // actions
   int   GetRunROBStatus();                             // get status for running boards reading applications 
   int   GetRunONAStatus();                             // get status for running analysis application  
   void  SetRunROBStatus(int _val);                     // set status for running boards reading applications
   void  SetRunONAStatus(int _val);                     // set status for running analysis application  
   // states -- bit states see body file for more information 
   int   GetAlertState();                               // get aplication status
   void  SetAlertState(int _val);                       // set application status

public:
   // lock for threads
   pthread_mutex_t  fmutex;                              // lock for all common variables
   sem_t            fsemaphore;                          // flag for analysis part to start proccess
   
   // error\exeption status
   COnLineExeption *fexeptionStatus;                     // handling error mesages 

   // software configuration object                      // handling configuration/calibration values
   SConfigData     *fSoftwareConfigObj; 

   // std containers
   std::queue<std::deque<CDataStructure> > *FIFOData;    // main container for pack

   // stop/start actions
   int  fRunROB;                                         // controling behevior of hardware read part
   int  fRunONA;                                         // controling behevior of analysis/DIM part
   
   // Hardware & software configuration requests
   bool fConfigureHardware;                              // flag to load new hardward configuration NI/GOOFIE boards
   bool fConfigureSoftware;                              // flag to load new software configuration
   bool fUpDateNumberOfEventInPack;                      // flag ro load new number of of event in pack
   int  fCurrentNumbersOfEventsInPack;                   // local copy of Number of Event in Pack from SConfigData class
   int  fAlertState;                                     //    
};

#endif
