//----------------------------------------------------------------------------
//  Body: AliGoofieConfigData.cxx, v 1.0                      2008/09/17    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>

#include "AliGoofieConfigData.h"
#include "AliGoofieFileParser.h"

using namespace std;

   
double AliGoofieConfigData::fDriftLenght                  = 12.75;       // electron drift length (cm) (12.75=TRD, 20.4=TPC),  difference between alpha sources
double AliGoofieConfigData::fADCTimeBin                   = 0.05;        // Goofie ACD single time bin in usec.

double AliGoofieConfigData::fDriftCorrectionP0            = 0.0;         // Drift velocity correction parameter P0
double AliGoofieConfigData::fDriftCorrectionP1            = 1.0;         // Drift velocity correction parameter P1
double AliGoofieConfigData::fGainNearCorrectionP0         = 0.0;         // Gain correction parameter P0 for near peak
double AliGoofieConfigData::fGainNearCorrectionP1         = 1.0;         // Gain correction parameter P1 for near peak
double AliGoofieConfigData::fGainFarCorrectionP0          = 0.0;         // Gain correction parameter P0 for far peak
double AliGoofieConfigData::fGainFarCorrectionP1          = 1.0;         // Gain correction parameter P1 for far peak
double AliGoofieConfigData::fMeanValue_P_TU               = 1.0;         // Mean value of P/TU  --> for drift velocity
double AliGoofieConfigData::fMeanValue_P_T                = 1.0;         // Mean value of P/T   --> for gas gain
double AliGoofieConfigData::fMeanValue_DriftVel           = 1.0;         // Mean value of drift velocity  --> for gas composition calculation, simulation data normalized to TRD=, TPC=2.52383 (cm/us) 
double AliGoofieConfigData::fMeanValue_GainNear           = 1.0;         // Mean value of gas gain near   --> for gas composition calculation
double AliGoofieConfigData::fMeanValue_GainFar            = 1.0;         // Mean value of gas gain far    --> for gas composition calculation
// when modifying parameters below please see and update also information
// in AliGoofieGasCompositionAnalysis::CalculateFractionOfCO2AndN2()
double AliGoofieConfigData::fGasComFitToDvPlaneP0         = 1.0;         // Fit parameters for parametrization of simulated dependency of electron drift on gas composition
double AliGoofieConfigData::fGasComFitToDvPlaneP1         = 1.0;         // Fit equation --> DriftVelocity = GasComFitToDvPlaneP0*N2(%) + GasComFitToDvPlaneP1*CO2(%) + GasComFitToDvPlaneP2
double AliGoofieConfigData::fGasComFitToDvPlaneP2         = 0.0;         //
double AliGoofieConfigData::fGasComFitToGainPlaneP0       = 1.0;         // Fit parameters for parametrization of simulated dependency of gas gain on gas composition
double AliGoofieConfigData::fGasComFitToGainPlaneP1       = 1.0;         // Fit equation --> GasGain = GasComFitToGainPlaneP0*N2(%) + GasComFitToGainPlaneP1*CO2(%) + GasComFitToGainPlaneP2
double AliGoofieConfigData::fGasComFitToGainPlaneP2       = 0.0;         //
// event quality cut
double AliGoofieConfigData::fTriggerMinimumAmplitude      = 45.0;        // event quality value
double AliGoofieConfigData::fTriggerMaximumPosition       = 35.0;        // 35
double AliGoofieConfigData::fPickupMinimumWidth           = 8.0;         // 8
double AliGoofieConfigData::fPickupMinimumAmplitude       = 5.0;         // 5
double AliGoofieConfigData::fPickupMinimumOffset          = 50.0;        // 50
  
int    AliGoofieConfigData::fAnalysisMethodEvent          = 1;           // 0 - only pickup information used, 1 - all channels used
int    AliGoofieConfigData::fAnalysisMethod               = 1;           // 0 - integrated spectra, 1 - event by event from fit, 2 -
int    AliGoofieConfigData::fEventQualityCutOnOff         = 1;           // 0 - off, 1 - on
int    AliGoofieConfigData::fGainFromTriggersOrPickup     = 0;           // 0 - from Pickup, 1 - from triggers 
int    AliGoofieConfigData::fTemperatureScale             = 0;           // 0 - K, 1 - C
int    AliGoofieConfigData::fCalculateMeans               = 0;           // 0 - no, 1 - yes --> store and calculate mean values P/T, P/TU, Dv, Gain
int    AliGoofieConfigData::fEventNearPeakLow             = 20;          // Near peak window, low limit
int    AliGoofieConfigData::fEventNearPeakHigh            = 100;         // Near peak window, high limit
int    AliGoofieConfigData::fEventFarPeakLow              = 160;         // Far peak window, low limit
int    AliGoofieConfigData::fEventFarPeakHigh             = 240;         // Far peak window, high limit
double AliGoofieConfigData::fPeakIntegrationRange         = 100;         // Constant integration window, center at 30 % of the window


AliGoofieConfigData::AliGoofieConfigData()
{
   fInitializationState = false;
#ifdef DEBUG_ONLINE
   cout << "Debuge Info => AliGoofieConfigData object created" << endl;
#endif
}


AliGoofieConfigData::AliGoofieConfigData(const AliGoofieConfigData &_obj)
{
#ifdef DEBUG_ONLINE
   cout << "Debuge Info => copy constructor of AliGoofieConfigData used" << endl;
#endif
}


AliGoofieConfigData & AliGoofieConfigData::operator=(const AliGoofieConfigData &_obj)
{
  if(this!=&_obj) {
     *this=_obj;
#ifdef DEBUG_ONLINE
     cout << "Debuge Info => overloaded = operator of AliGoofieConfigData used" << endl;
#endif
  }
  return *this;
}


AliGoofieConfigData::~AliGoofieConfigData()
{
#ifdef DEBUG_ONLINE
   cout << "Debuge Info => AliGoofieConfigData object destroyed" << endl;
#endif
}


void AliGoofieConfigData::SetConfigurationValues(const std::string _configFile)
{
   if(!fInitializationState) {
      try {
	 // Numbers
	 AliGoofieFileParser<MapStringDbl_t> cfgNumber(_configFile);
	 // set values from file
	 fDriftLenght              =      cfgNumber.get_value("DriftLenght");
	 fADCTimeBin               =      cfgNumber.get_value("ADCTimeBin");

	 fDriftCorrectionP0        =      cfgNumber.get_value("DriftCorrectionP0");
	 fDriftCorrectionP1        =      cfgNumber.get_value("DriftCorrectionP1");
	 fGainNearCorrectionP0     =      cfgNumber.get_value("GainNearCorrectionP0");
	 fGainNearCorrectionP1     =      cfgNumber.get_value("GainNearCorrectionP1");
	 fGainFarCorrectionP0      =      cfgNumber.get_value("GainFarCorrectionP0");
	 fGainFarCorrectionP1      =      cfgNumber.get_value("GainFarCorrectionP1");
	 fMeanValue_P_TU           =      cfgNumber.get_value("MeanValue_P_TU");
	 fMeanValue_P_T            =      cfgNumber.get_value("MeanValue_P_T");
	 fMeanValue_DriftVel       =      cfgNumber.get_value("MeanValue_DriftVel");
	 fMeanValue_GainNear       =      cfgNumber.get_value("MeanValue_GainNear");
	 fMeanValue_GainFar        =      cfgNumber.get_value("MeanValue_GainFar");
	 fGasComFitToDvPlaneP0     =      cfgNumber.get_value("GasComFitToDvPlaneP0");
	 fGasComFitToDvPlaneP1     =      cfgNumber.get_value("GasComFitToDvPlaneP1");
	 fGasComFitToDvPlaneP2     =      cfgNumber.get_value("GasComFitToDvPlaneP2");
	 fGasComFitToGainPlaneP0   =      cfgNumber.get_value("GasComFitToGainPlaneP0");
	 fGasComFitToGainPlaneP1   =      cfgNumber.get_value("GasComFitToGainPlaneP1");
	 fGasComFitToGainPlaneP2   =      cfgNumber.get_value("GasComFitToGainPlaneP2");
	 fTriggerMinimumAmplitude  =      cfgNumber.get_value("TriggerMinimumAmplitude");
	 fTriggerMaximumPosition   =      cfgNumber.get_value("TriggerMaximumPosition");
	 fPickupMinimumWidth       =      cfgNumber.get_value("PickupMinimumWidth");
	 fPickupMinimumAmplitude   =      cfgNumber.get_value("PickupMinimumAmplitude");
	 fPickupMinimumOffset      =      cfgNumber.get_value("PickupMinimumOffset");

	 fAnalysisMethodEvent      = (int)cfgNumber.get_value("AnalysisMethodEvent");
         fAnalysisMethod           = (int)cfgNumber.get_value("AnalysisMethod");
	 fEventQualityCutOnOff     = (int)cfgNumber.get_value("EventQualityCutOnOff");
	 fGainFromTriggersOrPickup = (int)cfgNumber.get_value("GainFromTriggersOrPickup");
	 fTemperatureScale         = (int)cfgNumber.get_value("TemperatureScale");
	 fCalculateMeans           = (int)cfgNumber.get_value("CalculateMeans"); 
	 fEventNearPeakLow         = (int)cfgNumber.get_value("EventNearPeakLow");
	 fEventNearPeakHigh        = (int)cfgNumber.get_value("EventNearPeakHigh");
	 fEventFarPeakLow          = (int)cfgNumber.get_value("EventFarPeakLow");
	 fEventFarPeakHigh         = (int)cfgNumber.get_value("EventFarPeakHigh");
	 fPeakIntegrationRange     =      cfgNumber.get_value("PeakIntegrationRange");

	 cout << "Config File: " << endl;
	 cout << cfgNumber << endl;

	 fInitializationState      = true;
      } catch(exception &e) {
	 string errorInfo = "<SConfigData::InitializeFromFile()>:";
	 errorInfo +=  e.what();
	 // error info
	 cerr << errorInfo << flush << endl;

	 fInitializationState = false;
      }
   }
}
