//----------------------------------------------------------------------------
//  Header: AliGoofieConfigData.h, v 1.0                      2008/09/17    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef ALIGOOFIECONFIGDATA
#define ALIGOOFIECONFIGDATA

#include <string>

class AliGoofieConfigData
{
public:
   AliGoofieConfigData();
   AliGoofieConfigData(const AliGoofieConfigData &_obj);
   virtual ~AliGoofieConfigData();
   AliGoofieConfigData & operator= (const AliGoofieConfigData &_obj);

   void SetConfigurationValues(const std::string _configFile);          // Set configuration parameters obj.

   void DumpConfigurationValue();

public:
   bool          fInitializationState;            // initialization from file on/off
   
   static double fDriftLenght;                    // electron drift length (cm) (12.75=TRD, 20.4=TPC), difference between alpha sources
   static double fADCTimeBin;                     // Goofie ACD single time bin in usec.
 
   static double fDriftCorrectionP0;              // Drift velocity correction parameter P0
   static double fDriftCorrectionP1;              // Drift velocity correction parameter P1
   static double fGainNearCorrectionP0;           // Gain correction parameter P0 for near peak
   static double fGainNearCorrectionP1;           // Gain correction parameter P1 for near peak
   static double fGainFarCorrectionP0;            // Gain correction parameter P0 for far peak
   static double fGainFarCorrectionP1;            // Gain correction parameter P1 for far peak
   static double fMeanValue_P_TU;                 // Mean value of P/TU  --> for drift velocity calculation
   static double fMeanValue_P_T;                  // Mean value of P/T   --> for gas gain calculation
   static double fMeanValue_DriftVel;             // Mean value of drift velocity --> for gas composition calculation, simulation data normalized to TRD=, TPC=2.52383 (cm/us) 
   static double fMeanValue_GainNear;             // Mean value of gas gain near  --> for gas composition calculation, 
   static double fMeanValue_GainFar;              // Mean value of gas gain far   --> for gas composition calculation
   static double fGasComFitToDvPlaneP0;           // Fit parameters for parametrization of simulated dependency of electron drift on gas composition
   static double fGasComFitToDvPlaneP1;           // Fit equation --> DriftVelocity = GasComFitToDvPlaneP0*N2(%) + GasComFitToDvPlaneP1*CO2(%) + GasComFitToDvPlaneP2
   static double fGasComFitToDvPlaneP2;           //
   static double fGasComFitToGainPlaneP0;         // Fit parameters for parametrization of simulated dependency of gas gain on gas composition
   static double fGasComFitToGainPlaneP1;         // Fit equation --> GasGain = GasComFitToGainPlaneP0*N2(%) + GasComFitToGainPlaneP1*CO2(%) + GasComFitToGainPlaneP2
   static double fGasComFitToGainPlaneP2;         //
                                                     
   static double fTriggerMinimumAmplitude;        // event quality value 5
   static double fTriggerMaximumPosition;         // 35
   static double fPickupMinimumWidth;             // 8
   static double fPickupMinimumAmplitude;         // 5
   static double fPickupMinimumOffset;            // 50  
      
   static int    fAnalysisMethodEvent;            // 0 - only pickup information used, 1 - all channels used, 2 - all channels no fit information
   static int    fAnalysisMethod;                 // 0 - integrated spectra, 1 - event by event from fit, 2 -
   static int    fEventQualityCutOnOff;           // 0 - off, 1 - on
   static int    fGainFromTriggersOrPickup;       // 0 - from Pickup, 1 - from triggers 
   static int    fTemperatureScale;               // 0 - K, 1 - C, gas analysis
   static int    fCalculateMeans;                 // 0 - no, 1 - yes --> store and calculate mean values P/T, P/TU, Dv, Gain
   static int    fEventNearPeakLow;               // Near peak window, low limit
   static int    fEventNearPeakHigh;              // Near peak window, high limit
   static int    fEventFarPeakLow;                // Far peak window, low limit
   static int    fEventFarPeakHigh;               // Far peak window, high limit
   static double fPeakIntegrationRange;           // Constant integration window, center at 30 % of the window
};
#endif
