//----------------------------------------------------------------------------
//  Body: AliGoofieDriftVelocityAnalysis.cxx, v 1.0             2009/01/30  //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2009                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <string>
#include <cmath>

#include "TH1.h"
#include "TH2.h"

#include "AliGoofieConfigData.h"
#include "AliGoofieGasResult.h"
#include "AliGoofieFitFunction.h"

#include "AliGoofieDriftVelocityAnalysis.h"

using namespace std;

AliGoofieDriftVelocityAnalysis::AliGoofieDriftVelocityAnalysis()
{
#ifdef DEBUG_ONLINE    
   cout << "Debuge Info => AliGoofieDriftVelocityAnalysis object created" << endl;
#endif
   fhDriftVelocityVsGasDensity = new TH2D("DriftVelocityVsGasDensity","DriftVelocityVsGasDensity", 1000, 0.5, 1.5, 1000, 0.0, 10.0);
   // Gas Analysis prt
   fGasAnalysisResults = NULL;
}

AliGoofieDriftVelocityAnalysis::~AliGoofieDriftVelocityAnalysis()
{
   if(fhDriftVelocityVsGasDensity) {
      // fhDriftVelocityVsGasDensity->Delete(); some problems ???
      fhDriftVelocityVsGasDensity = NULL;
   }
   // clear containers
   fP_TU_Value.clear();
   fDriftVelocity_Value.clear();
}

AliGoofieDriftVelocityAnalysis::AliGoofieDriftVelocityAnalysis(const AliGoofieDriftVelocityAnalysis &_obj)
{
   if(this!=&_obj) {
      if(_obj.fhDriftVelocityVsGasDensity) {
	 fhDriftVelocityVsGasDensity = new TH2D(*_obj.fhDriftVelocityVsGasDensity);
      } else {
	 fhDriftVelocityVsGasDensity = NULL;
      }
      fNearPeak = _obj.fNearPeak;
      fFarPeak  = _obj.fFarPeak;
      //
      fGasAnalysisResults = _obj.fGasAnalysisResults;
      // copy containers
      fP_TU_Value.clear();
      std::copy(_obj.fP_TU_Value.begin (), _obj.fP_TU_Value.end (), std::back_inserter(fP_TU_Value));

      fDriftVelocity_Value.clear();
      std::copy(_obj.fDriftVelocity_Value.begin (), _obj.fDriftVelocity_Value.end (), std::back_inserter(fDriftVelocity_Value));
   }
}

AliGoofieDriftVelocityAnalysis & AliGoofieDriftVelocityAnalysis::operator=(const AliGoofieDriftVelocityAnalysis &_obj)
{
   if(this!=&_obj) {
      if(_obj.fhDriftVelocityVsGasDensity) {
	 fhDriftVelocityVsGasDensity = new TH2D(*_obj.fhDriftVelocityVsGasDensity);
      } else {
	 fhDriftVelocityVsGasDensity = NULL;
      }
      fNearPeak = _obj.fNearPeak;
      fFarPeak  = _obj.fFarPeak;
      //
      fGasAnalysisResults = _obj.fGasAnalysisResults;
      // copy containers
      fP_TU_Value.clear();
      std::copy(_obj.fP_TU_Value.begin (), _obj.fP_TU_Value.end (), std::back_inserter(fP_TU_Value));

      fDriftVelocity_Value.clear();
      std::copy(_obj.fDriftVelocity_Value.begin (), _obj.fDriftVelocity_Value.end (), std::back_inserter(fDriftVelocity_Value));
   }
   return *this;
}

Bool_t AliGoofieDriftVelocityAnalysis::CalculateDriftVelocity(TH1D *_hSpectrumTimeAll,
							      TH1D *_hSpectrumTime,
							      AliGoofieFitFunction *_fFitSpectrum)  
{
   Bool_t retVal = false;

   //get pack number
   string packNo = _hSpectrumTime->GetName();
   int index = packNo.find("_");
   if(index > 0) {
      packNo.erase(0, index);
   }
   // get peaks information
   // find peaks
   if(AliGoofieConfigData::fAnalysisMethod == 0) {
      fNearPeak.FindMaximumPeak(_hSpectrumTimeAll, &fFarPeak);
   } else if(AliGoofieConfigData::fAnalysisMethod == 1) {
      fNearPeak.FindMaximumPeak(_hSpectrumTime, &fFarPeak);
   }
   // check who is first
   if(!(fNearPeak.fPosition>AliGoofieConfigData::fEventNearPeakLow && 
	fNearPeak.fPosition<AliGoofieConfigData::fEventNearPeakHigh)) {
     AliGoofiePeakProperties temp = fNearPeak;
     fNearPeak = fFarPeak;
     fFarPeak = temp;    
   }
   // set names
   fNearPeak.fName = "NearPeak"; fNearPeak.fName += packNo; 
   fFarPeak.fName  = "FarPeak";  fFarPeak.fName  += packNo;
   
   try {
      if(AliGoofieConfigData::fAnalysisMethod == 0) {
	 retVal = _fFitSpectrum->MakeFit(_hSpectrumTimeAll, &fNearPeak);
	 if(!retVal) throw int(-100);
	 retVal = _fFitSpectrum->MakeFit(_hSpectrumTimeAll, &fFarPeak);
	 if(!retVal) throw int(-101);
      } else if(AliGoofieConfigData::fAnalysisMethod == 1) {
	 retVal = _fFitSpectrum->MakeGaussFit(_hSpectrumTime, &fNearPeak);
	 if(!retVal) throw int(-102);
	 retVal = _fFitSpectrum->MakeGaussFit(_hSpectrumTime, &fFarPeak);
	 if(!retVal) throw int(-103);
      }
 #ifdef DEBUG_ONLINE
   cout << "======================= Fit Results Near ==========================" << endl;
   cout << "Peak name   = " << fNearPeak.fName << endl;
   cout << "Peak pos    = " << fNearPeak.fPosition  << " +- " << fNearPeak.fErrPosition << endl;
   cout << "Amplitude   = " << fNearPeak.fAmplitude << " +- " << fNearPeak.fErrAmplitude << endl;
   cout << "Width       = " << fNearPeak.fWidth << " +- " << fNearPeak.fErrWidth << endl;
   cout << "Offset      = " << fNearPeak.fOffset << " +- " << fNearPeak.fErrOffset << endl;
   cout << "Area        = " << fNearPeak.fArea << endl;    
   cout << "chi2        = " << fNearPeak.fChi2 << endl;
   cout << "gradient    = " << fNearPeak.fGradient << endl;
   cout << "Multi peaks = " << fNearPeak.fMultiPeakSpectrum << endl;
   cout << "======================= Fit Results Far ===============================" << endl;
   cout << "Peak name   = " << fFarPeak.fName << endl;
   cout << "Peak pos    = " << fFarPeak.fPosition  << " +- " << fFarPeak.fErrPosition << endl;
   cout << "Amplitude   = " << fFarPeak.fAmplitude << " +- " << fFarPeak.fErrAmplitude << endl;
   cout << "Width       = " << fFarPeak.fWidth << " +- " << fFarPeak.fErrWidth << endl;
   cout << "Offset      = " << fFarPeak.fOffset << " +- " << fFarPeak.fErrOffset << endl;
   cout << "Area        = " << fFarPeak.fArea << endl;    
   cout << "chi2        = " << fFarPeak.fChi2 << endl;
   cout << "gradient    = " << fFarPeak.fGradient << endl;
   cout << "Multi peaks = " << fFarPeak.fMultiPeakSpectrum << endl;
   cout << "===================================================================" << endl;
  #endif

   // check whether peak position is positive
   if(fNearPeak.fPosition<=0.0) {retVal = false; throw int(-104);}
   if(fFarPeak.fPosition<=0.0)  {retVal = false; throw int(-105);}

   // set peaks position
   fGasAnalysisResults->fPeakPosNear = fNearPeak.fPosition;           // near peak position in pickup spectrum
   fGasAnalysisResults->fPeakPosFar  = fFarPeak.fPosition;            // far peak position in pickup spectrum

   // compare fit amplitude, in case that they differ more them 90% we neglect such data.
   if((fNearPeak.fAmplitude*0.1) > fFarPeak.fAmplitude)     {retVal = false; throw int(-106);}
   if((fFarPeak.fAmplitude*0.1)  > fNearPeak.fAmplitude)    {retVal = false; throw int(-107);}

   // calculate difference deltaT
   double timeDifference =  fabs(fFarPeak.fPosition-fNearPeak.fPosition)*AliGoofieConfigData::fADCTimeBin;
   if(timeDifference>0) {
      fGasAnalysisResults->fDriftVelocity =  AliGoofieConfigData::fDriftLenght/timeDifference;
   } else {
      fGasAnalysisResults->fDriftVelocity =  0.0;
      retVal = false;
      throw int(-108);
   }
   // Fill P_TU Correlation
   if(fGasAnalysisResults->fTemp1 == 0) throw int(-109);        // in general such situation should newer happen, but if does, one can try to randomize this case with +- 0.0000000001.
   if(fGasAnalysisResults->fFieldCageHV == 0) throw int(-110);  // if there is zero, then HV power supply failed

   double P_TU   = fGasAnalysisResults->fPressure/fabs(fGasAnalysisResults->fTemp1*fGasAnalysisResults->fFieldCageHV); // temperature from one sensor only required some study T1=NEAR
   double N_P_TU = (1.0/AliGoofieConfigData::fMeanValue_P_TU)*P_TU;

   // correct drift velocity
   fGasAnalysisResults->fCorrectionFactor       = AliGoofieConfigData::fDriftCorrectionP1*(1.0-N_P_TU);                         // calculated from P/T U variations
   fGasAnalysisResults->fDriftVelocityCorrected = fGasAnalysisResults->fDriftVelocity + fGasAnalysisResults->fCorrectionFactor; // drift velocity of electron corected for P/T U variations

   // store correction factor if required
   if(AliGoofieConfigData::fCalculateMeans) {
      fP_TU_Value.push_back(P_TU);
      fDriftVelocity_Value.push_back(fGasAnalysisResults->fDriftVelocityCorrected);
   }
   // fill drift velocity vs. gas gain corrected for HV Efield fluctuation correlation
   fhDriftVelocityVsGasDensity->Fill(N_P_TU, fGasAnalysisResults->fDriftVelocity);

   } catch(int _errState) {
      cerr << "Error form <AliGoofieDriftVelocityAnalysis::CalculateDriftVelocity>: Drift Velocity calculation error = " << _errState << flush << endl;
   }
   return retVal;
}


Double_t AliGoofieDriftVelocityAnalysis::GetMeanP_TU()
{
   double retVal = 0.0;
   for(unsigned int pos = 0; pos<fP_TU_Value.size(); ++pos) {
      retVal += fP_TU_Value[pos];
   }
   if(fP_TU_Value.size()>1) {
      retVal /= fP_TU_Value.size();
   }
   return retVal;
}

Double_t  AliGoofieDriftVelocityAnalysis::GetMeanDriftVelocity()
{
   double retVal = 0.0;
   for(unsigned int pos = 0; pos<fDriftVelocity_Value.size(); ++pos) {
      retVal += fDriftVelocity_Value[pos];
   }
   if(fDriftVelocity_Value.size()>1) {
      retVal /= fDriftVelocity_Value.size();
   }
   return retVal;
}

TF1 *AliGoofieDriftVelocityAnalysis::MakeNearFitFunction(AliGoofieFitFunction *_fFitSpectrum)
{
   TF1 *retPrt = NULL;
   if(AliGoofieConfigData::fAnalysisMethod == 0) {
      retPrt = _fFitSpectrum->GetFitResults(&fNearPeak);
   } else if(AliGoofieConfigData::fAnalysisMethod == 1) {
      retPrt = _fFitSpectrum->GetGaussFitResults(&fNearPeak);
   }
   return retPrt;
}

TF1 *AliGoofieDriftVelocityAnalysis::MakeFarFitFunction(AliGoofieFitFunction *_fFitSpectrum)
{
   TF1 *retPrt = NULL;
   if(AliGoofieConfigData::fAnalysisMethod == 0) {
      retPrt = _fFitSpectrum->GetFitResults(&fFarPeak);
   } else if(AliGoofieConfigData::fAnalysisMethod == 1) {
      retPrt = _fFitSpectrum->GetGaussFitResults(&fFarPeak);
   }
   return retPrt;
}
