//----------------------------------------------------------------------------
//  Header: AliGoofieDriftVelocityAnalysis.h, v 1.0             2009/01/30  //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2009                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef ALIGOOFIEDRIFTVELOCITYANALYSIS
#define ALIGOOFIEDRIFTVELOCITYANALYSIS

#include <deque>

#include "AliGoofiePeakProperties.h"

class TH1D;
class TH2D;
class TF1;
class AliGoofieGasResult;
class AliGoofieFitFunction;

class AliGoofieDriftVelocityAnalysis
{
public:
   AliGoofieDriftVelocityAnalysis();
   AliGoofieDriftVelocityAnalysis(const AliGoofieDriftVelocityAnalysis &_obj);
   AliGoofieDriftVelocityAnalysis & operator= (const AliGoofieDriftVelocityAnalysis &_obj);
   ~AliGoofieDriftVelocityAnalysis();

   Bool_t    CalculateDriftVelocity(TH1D *_hSpectrumTimeAll,
				    TH1D *_hSpectrumTime,
				    AliGoofieFitFunction *_fFitSpectrum);  

   void      SetGasResultsPrt(AliGoofieGasResult *_prtGasAnalysis)       {fGasAnalysisResults = _prtGasAnalysis;}
   TH2D     *GetPrtToDriftVelocityVsGasDensity()                         {return fhDriftVelocityVsGasDensity;}
   TF1      *MakeNearFitFunction(AliGoofieFitFunction *_fFitSpectrum);
   TF1      *MakeFarFitFunction(AliGoofieFitFunction *_fFitSpectrum);
   Double_t  GetMeanP_TU();
   Double_t  GetMeanDriftVelocity();
   
public:
   AliGoofiePeakProperties      fNearPeak;
   AliGoofiePeakProperties      fFarPeak;

   TH2D                        *fhDriftVelocityVsGasDensity;
   AliGoofieGasResult          *fGasAnalysisResults;

   std::deque<double>           fP_TU_Value;
   std::deque<double>           fDriftVelocity_Value;
};
#endif
