//----------------------------------------------------------------------------
//  Body: AliGoofieEventInfo.cxx, v 1.0                       2008/08/02    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>

#include "TH1.h"
#include "TF1.h"
#include "TList.h"

#include "AliGoofieConfigData.h"
#include "AliGoofiePeakProperties.h"

#include "AliGoofieEventInfo.h"

using namespace std;

ClassImp(AliGoofieEventInfo)

AliGoofieEventInfo::AliGoofieEventInfo() : AliGoofieRawEvent(512) //fNumberOfADCSampels
{
#ifdef DEBUG_ONLINE
   cout << "Info from <AliGoofieEventInfo()> object created" << endl;
#endif
   fAnalysisMethod       = 0;
   fPeakTriggeredBy      = TG_NOISE;  // where peak located in the spectrum, 1-Far, 2-Near, 0-Noise
}

AliGoofieEventInfo::AliGoofieEventInfo(const AliGoofieEventInfo &_obj) : AliGoofieRawEvent(_obj)
{
#ifdef DEBUG_ONLINE
   cout << "Info from <AliGoofieEventInfo>: copy constructor called" << endl;
#endif
   if(this!=&_obj) {
      AliGoofieRawEvent &refData = *this;
      refData = _obj;

      fAnalysisMethod      = _obj.fAnalysisMethod;
      fPeakTriggeredBy     = _obj.fPeakTriggeredBy;      // where peak located in the spectrum, 1-Near, 2-Far, 0-Noise

      // pickup properties
      fPickupPeakInfo      = _obj.fPickupPeakInfo;
      // triggers properties
      fTriggerNearPeakInfo = _obj.fTriggerNearPeakInfo;
      fTriggerFarPeakInfo  = _obj.fTriggerFarPeakInfo;
   }
}

AliGoofieEventInfo & AliGoofieEventInfo::operator=(const AliGoofieEventInfo &_obj)
{
#ifdef DEBUG_ONLINE
   cout << "Info from <AliGoofieEventInfo>: overloaded = operator used" << endl;
#endif
   if(this!=&_obj) {
      AliGoofieRawEvent &refData = *this;
      refData = _obj;

      fAnalysisMethod      = _obj.fAnalysisMethod;
      fPeakTriggeredBy     = _obj.fPeakTriggeredBy;      // where peak located in the spectrum, 1-Near, 2-Far, 0-Noise

      // pickup properties
      fPickupPeakInfo      = _obj.fPickupPeakInfo;
      // triggers properties
      fTriggerNearPeakInfo = _obj.fTriggerNearPeakInfo;
      fTriggerFarPeakInfo  = _obj.fTriggerFarPeakInfo;
   }
   return *this;
}

AliGoofieEventInfo::~AliGoofieEventInfo()
{
#ifdef DEBUG_ONLINE
   cout << "Info from <~AliGoofieEventInfo()>: object destroyed" << endl;
#endif
}

Bool_t AliGoofieEventInfo::AnalyzeEvent(const AliGoofieRawEvent &_event,
					AliGoofieFitFunction *_fFitSpectrum,
					Int_t  _packNumber,
					Int_t  _eventNumber,
					TList *_toDelete)
{
   bool retVal = false;

   // copy AliGoofieRawEvent part
   AliGoofieRawEvent &refData = *this;
   refData = _event;

   // create peak histograms
   static bool laddToDelete = true;
   TString hName = "Pickup_"; hName += _packNumber; hName += "_"; hName += _eventNumber;
   fPickupPeakInfo.fName = hName.Data();
   static TH1D *lPickupSpectrum      = new TH1D(hName.Data(), hName.Data(), _event.fLocalCopyOfNumberOfADCSampels, 0.0, _event.fLocalCopyOfNumberOfADCSampels);
   hName = "Near_"; hName += _packNumber; hName += "_"; hName += _eventNumber;
   fTriggerNearPeakInfo.fName = hName.Data();
   static TH1D *lTriggerNearSpectrum = new TH1D(hName.Data(), hName.Data(), _event.fLocalCopyOfNumberOfADCSampels, 0.0, _event.fLocalCopyOfNumberOfADCSampels);
   hName = "Far_"; hName += _packNumber; hName += "_"; hName += _eventNumber;
   fTriggerFarPeakInfo.fName = hName.Data();
   static TH1D *lTriggerFarSpectrum  = new TH1D(hName.Data(), hName.Data(), _event.fLocalCopyOfNumberOfADCSampels, 0.0, _event.fLocalCopyOfNumberOfADCSampels);
   if(laddToDelete) {
      if(_toDelete) {
	 _toDelete->Add(lPickupSpectrum);
	 _toDelete->Add(lTriggerNearSpectrum);
	 _toDelete->Add(lTriggerFarSpectrum);
      }
      laddToDelete = false;
   }
   // get spectra to TH1
   for(int bin=0; bin<_event.fLocalCopyOfNumberOfADCSampels; ++bin) {
      lPickupSpectrum->SetBinContent(bin+1, _event.GetEntryInDataPickup(bin));
      if(fAnalysisMethod==1) {
	 lTriggerNearSpectrum->SetBinContent(bin+1, _event.GetEntryInDataNear(bin));
	 lTriggerFarSpectrum->SetBinContent(bin+1, _event.GetEntryInDataFar(bin));
      }
   }
   // Fit Spectra
   retVal = fPickupPeakInfo.FitSpectrum(lPickupSpectrum, _fFitSpectrum);
   // Fit trigger
   if(fAnalysisMethod==1) {
      fTriggerNearPeakInfo.FitSpectrum(lTriggerNearSpectrum, _fFitSpectrum);
      fTriggerFarPeakInfo.FitSpectrum(lTriggerFarSpectrum, _fFitSpectrum);
   }
   //cout << "Limits Near = " << AliGoofieConfigData::fEventNearPeakLow << "-" << AliGoofieConfigData::fEventNearPeakHigh << endl;
   //cout << "Limits Far  = " << AliGoofieConfigData::fEventFarPeakLow << "-" << AliGoofieConfigData::fEventFarPeakHigh << endl;

   //Set trigger channel
   if(fPickupPeakInfo.fPosition>AliGoofieConfigData::fEventNearPeakLow &&
      fPickupPeakInfo.fPosition<AliGoofieConfigData::fEventNearPeakHigh) {
      fPeakTriggeredBy = TG_NEAR;  // near trigger
   } else if(fPickupPeakInfo.fPosition>AliGoofieConfigData::fEventFarPeakLow &&
	     fPickupPeakInfo.fPosition<AliGoofieConfigData::fEventFarPeakHigh) {
      fPeakTriggeredBy = TG_FAR;  // far trigger
   } else {
      fPeakTriggeredBy = TG_NOISE;  // noise
   }
   return retVal;
}


TH1D *AliGoofieEventInfo::MakePickupSpectrum()
{
   TString name = fPickupPeakInfo.fName.c_str(); name += "_ev";
   TH1D *lPickupSpectrum = new TH1D(name,
				    name,
				    fLocalCopyOfNumberOfADCSampels,
				    0.0,
				    fLocalCopyOfNumberOfADCSampels);

   for(int bin=0; bin<fLocalCopyOfNumberOfADCSampels; ++bin) {
      lPickupSpectrum->SetBinContent(bin+1, GetEntryInDataPickup(bin));
   }
   return lPickupSpectrum;
}

TF1  *AliGoofieEventInfo::MakePickupFitFunction(AliGoofieFitFunction *_fFitSpectrum)
{
   return _fFitSpectrum->GetFitResults(&fPickupPeakInfo);
}

TH1D *AliGoofieEventInfo::MakeNearSpectrum()
{
   TString name = fTriggerNearPeakInfo.fName.c_str(); name += "_ev";
   TH1D *lTriggerNearSpectrum = new TH1D(name,
					 name,
					 fLocalCopyOfNumberOfADCSampels,
					 0.0,
					 fLocalCopyOfNumberOfADCSampels);

   for(int bin=0; bin<fLocalCopyOfNumberOfADCSampels; ++bin) {
      lTriggerNearSpectrum->SetBinContent(bin+1, GetEntryInDataNear(bin));
   }
   return lTriggerNearSpectrum;
}

TF1  *AliGoofieEventInfo::MakeNearFitFunction(AliGoofieFitFunction *_fFitSpectrum)
{
   return _fFitSpectrum->GetFitResults(&fTriggerNearPeakInfo);
}

TH1D *AliGoofieEventInfo::MakeFarSpectrum()
{
   TString name = fTriggerFarPeakInfo.fName.c_str(); name += "_ev";
   TH1D *lTriggerFarSpectrum = new TH1D(name,
					name,
					fLocalCopyOfNumberOfADCSampels,
					0.0,
					fLocalCopyOfNumberOfADCSampels);

   for(int bin=0; bin<fLocalCopyOfNumberOfADCSampels; ++bin) {
      lTriggerFarSpectrum->SetBinContent(bin+1, GetEntryInDataFar(bin));
   }
   return lTriggerFarSpectrum;
}

TF1  *AliGoofieEventInfo::MakeFarFitFunction(AliGoofieFitFunction *_fFitSpectrum)
{
   return _fFitSpectrum->GetFitResults(&fTriggerFarPeakInfo);
}
