//----------------------------------------------------------------------------
//  Header: AliGoofieEventInfo.h, v 1.0                       2008/08/02    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef ALIGOOFIEEVENTINFO
#define ALIGOOFIEEVENTINFO

#include <TObject.h>
#include "AliGoofieRawEvent.h"
#include "AliGoofieFitFunction.h"
#include "AliGoofiePeakProperties.h"

// define Trigger flags
#define TG_NEAR  2
#define TG_FAR   1
#define TG_NOISE 0

class TH1D;
class TList;
class AliGoofieFitFunction;

class AliGoofieEventInfo : public AliGoofieRawEvent
{
public:
   AliGoofieEventInfo();
   AliGoofieEventInfo(const AliGoofieEventInfo &_obj);
   virtual ~AliGoofieEventInfo();
   AliGoofieEventInfo & operator= (const AliGoofieEventInfo &_obj);

   Bool_t                 AnalyzeEvent(const AliGoofieRawEvent &_event,
				       AliGoofieFitFunction *_fFitSpectrumInt,
				       Int_t _packNumber,
				       Int_t _eventNumber,
				       TList *_toDelete=NULL);                   // return false when pickup fit will be wrong

   AliGoofieEventInfo    & GetCurrentEvent()                         {return *this;}

   void SetAnalysisMethod(const Int_t _AnalysisMethod)               {fAnalysisMethod = _AnalysisMethod;}

   TH1D *MakePickupSpectrum();
   TF1  *MakePickupFitFunction(AliGoofieFitFunction *_fFitSpectrum);
   TH1D *MakeNearSpectrum();
   TF1  *MakeNearFitFunction(AliGoofieFitFunction *_fFitSpectrum);
   TH1D *MakeFarSpectrum();
   TF1  *MakeFarFitFunction(AliGoofieFitFunction *_fFitSpectrum);

public:

   Int_t        fAnalysisMethod;           // 0 - analysis only based on pickup, 1 - all infomation used
   Int_t        fPeakTriggeredBy;          // where peak located in the spectrum, 1-Far, 2-Near, 0-Noise

   // pickup properties
   AliGoofiePeakProperties   fPickupPeakInfo;
   // triggers properties
   AliGoofiePeakProperties   fTriggerNearPeakInfo;
   AliGoofiePeakProperties   fTriggerFarPeakInfo;

   ClassDef(AliGoofieEventInfo, 1)        //AliGoofieEventInfo structure
};
#endif
