//----------------------------------------------------------------------------
//  Header: AliGoofieGasAnalysis.cpp, v 1.0                   2008/08/01    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------


#include <iostream>

#include <TROOT.h>

#include "AliGoofieRawEvent.h"
#include "AliGoofieConfigData.h"
#include "AliGoofieGasResult.h"
#include "AliGoofieEventInfo.h"
#include "AliGoofieFitFunction.h"
#include "AliGoofiePairAnalysis.h"

#include "AliGoofieGasAnalysis.h"

using namespace std;


AliGoofieGasAnalysis::AliGoofieGasAnalysis()
{
#ifdef DEBUG_ONLINE
   cout << "Info from <AliGoofieGasAnalysis::AliGoofieGasAnalysis()>: constructor called" << flush << endl;
#endif
   fGasAnalysisConfiguration = new AliGoofieConfigData();
   fGasAnalysisResults       = new AliGoofieGasResult();
   fEventProperty            = new AliGoofieEventInfo();
   fFitSpectrum              = new AliGoofieFitFunction();
   fPairAnalysis             = new AliGoofiePairAnalysis(fGasAnalysisResults);

   fDelFlagOfGasAnalysis     = true;
   fPackNo                   = 0;

   gROOT->SetBatch(true);
}

AliGoofieGasAnalysis::AliGoofieGasAnalysis(AliGoofieGasResult *_pbuf)
{
#ifdef DEBUG_ONLINE
   cout << "Info from <AliGoofieGasAnalysis::AliGoofieGasAnalysis(GasResPrt)>: constructor called" << flush << endl;
#endif
   // configuration information
   fGasAnalysisConfiguration = new AliGoofieConfigData();
   // gas results pointer/object
   if(_pbuf==NULL) {
      fGasAnalysisResults   = new AliGoofieGasResult();
      fDelFlagOfGasAnalysis = true;
   } else {
      fGasAnalysisResults   = _pbuf;
      fDelFlagOfGasAnalysis = false;
   }
   // Event analysis obj.
   fEventProperty            = new AliGoofieEventInfo();
   fFitSpectrum              = new AliGoofieFitFunction();
   // Pair analysis obj.
   fPairAnalysis             = new AliGoofiePairAnalysis(fGasAnalysisResults);
   fPackNo                   = 0;

   gROOT->SetBatch(true);
}


AliGoofieGasAnalysis::~AliGoofieGasAnalysis()
{
   if(fGasAnalysisConfiguration) {
      delete fGasAnalysisConfiguration;
      fGasAnalysisConfiguration = NULL;
   }
   if(fDelFlagOfGasAnalysis) {
      if(fGasAnalysisResults) {
	 delete fGasAnalysisResults;
	 fGasAnalysisResults = NULL;
      }
      fDelFlagOfGasAnalysis = false;
   }
   if(fEventProperty) {
      delete fEventProperty;
      fEventProperty = NULL;
   }
   if(fFitSpectrum) {
      delete fFitSpectrum;
      fFitSpectrum = NULL;
   }
   if(fPairAnalysis) {
      delete fPairAnalysis;
      fPairAnalysis = NULL;
   }
}


void AliGoofieGasAnalysis::PrintResults(Bool_t _TrueFalse)
{
   fPairAnalysis->PrintResults(_TrueFalse);
}


void AliGoofieGasAnalysis::SetConfiguration(const std::string _configFile)
{
   fGasAnalysisConfiguration->SetConfigurationValues(_configFile);
}


void AliGoofieGasAnalysis::AnalyzePack(const std::deque<AliGoofieRawEvent> &_pack)
{
   // number of analazed pack
    ++fPackNo;
   // Event analysis
   fEventProperty->SetAnalysisMethod(AliGoofieConfigData::fAnalysisMethodEvent);
   deque<AliGoofieRawEvent>::const_iterator pos;
   // event counter
   int  counter = 0;
   for(pos = _pack.begin(); pos != _pack.end(); ++pos) {
      // get event properties
      fEventProperty->AnalyzeEvent(*pos, fFitSpectrum, fPackNo, ++counter);
#ifdef DEBUG_ONLINE  
      cout << "<AliGoofieGasAnalysis::AnalyzePack>: Add analyzed event to deque" << flush << endl;
#endif
      fEventInfo.push_back(fEventProperty->GetCurrentEvent());                 // Event results
   }
   // Pair analysis
   fPairAnalysis->SetAnalysisMethod(AliGoofieConfigData::fAnalysisMethod);
   fPairAnalysis->MakeAnalysis(fEventInfo, fPackNo, fFitSpectrum);             // Pack results
}


void AliGoofieGasAnalysis::ClearEventList()
{
   // clear event list
   fEventInfo.clear();
}
