//----------------------------------------------------------------------------
//  Header: AliGoofieGasAnalysis.h, v 1.0                     2008/08/01    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef ALIGOOFIEGASANALYSIS
#define ALIGOOFIEGASANALYSIS

#include <deque>
#include <string>


class AliGoofieRawEvent;
class AliGoofieConfigData;
class AliGoofieGasResult;
class AliGoofieEventInfo;
class AliGoofieFitFunction;
class AliGoofiePairAnalysis;


class AliGoofieGasAnalysis
{
public:

   AliGoofieGasAnalysis();
   AliGoofieGasAnalysis(AliGoofieGasResult *_pbuf);
   virtual ~AliGoofieGasAnalysis();
   void SetConfiguration(const std::string _configFile);                           // Set configuration parameters obj.
   void AnalyzePack(const std::deque<AliGoofieRawEvent> &_pack);                   // Start analysis chain
   void PrintResults(Bool_t _TrueFalse);
   void ClearEventList();                                                          // Must be call after AnalyzePack function was called

   AliGoofieGasResult             *GetGasAnaResults()                {return fGasAnalysisResults;}  // Get current gas analysis results
   AliGoofieEventInfo             *GetEventInfoPtr()                 {return fEventProperty;}       // Get current event
   AliGoofieEventInfo             *GetEventFromPack(UInt_t _EventNo) {if(_EventNo<=fEventInfo.size()) return &fEventInfo[_EventNo]; else return NULL;}
   AliGoofieFitFunction           *GetFitFunctionPrt()               {return fFitSpectrum;}
   AliGoofiePairAnalysis          *GetPairInfoPtr()                  {return fPairAnalysis;}
   int                             GetEventListSize()                {return fEventInfo.size();}

private:

   AliGoofieConfigData            *fGasAnalysisConfiguration;                       // Gas analysis configuration file
   AliGoofieGasResult             *fGasAnalysisResults;                             // Keep current gas analysis results
   AliGoofieEventInfo             *fEventProperty;                                  // Current event analysis
   AliGoofieFitFunction           *fFitSpectrum;                                    // 1d fit function
   AliGoofiePairAnalysis          *fPairAnalysis;                                   // Current pair analysis
   Bool_t                          fDelFlagOfGasAnalysis;                           // flag for destructor
   std::deque<AliGoofieEventInfo>  fEventInfo;                                      // List of event results
   Int_t                           fPackNo;                                         // current pack noumber
};
#endif
