//----------------------------------------------------------------------------
//  Header: AliGoofieGasCompositionAnalysis.cxx, v 1.0          2009/01/30  //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2009                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <cmath>

#include "TH1.h"
#include "TH2.h"

#include "AliGoofieConfigData.h"
#include "AliGoofieGasResult.h"

#include "AliGoofieGasCompositionAnalysis.h"

using namespace std;

AliGoofieGasCompositionAnalysis::AliGoofieGasCompositionAnalysis()
{
#ifdef DEBUG_ONLINE
   cout << "Debuge Info => AliGoofieGasCompositionAnalysis object created" << endl;
#endif
   fhGainNearVsGasDensity = new TH2D("fhGainNearVsGasDensity","Gain Near Vs Gas Density", 1000, 0.5, 1.5, 1500, 0.0, 15000.0);
   fhGainFarVsGasDensity  = new TH2D("fhGainFarVsGasDensity","Gain Far Vs Gas Density", 1000, 0.5, 1.5, 1500, 0.0, 15000.0);
   // Gas Analysis prt
   fGasAnalysisResults = NULL;
}


AliGoofieGasCompositionAnalysis::~AliGoofieGasCompositionAnalysis()
{
   if(fhGainNearVsGasDensity) {
      // fhGainNearVsGasDensity->Delete(); some problems ???
      fhGainNearVsGasDensity = NULL;
   }
   if(fhGainFarVsGasDensity) {
      // fhGainFarVsGasDensity->Delete(); some problems ???
      fhGainFarVsGasDensity = NULL;
   }
   // vector
   fP_T_Value.clear();
   fGasGainNear_Value.clear();
   fGasGainFar_Value.clear();
}


AliGoofieGasCompositionAnalysis::AliGoofieGasCompositionAnalysis(const AliGoofieGasCompositionAnalysis &_obj)
{
   if(this!=&_obj) {
      if(_obj.fhGainNearVsGasDensity) {
	 fhGainNearVsGasDensity = new TH2D(*_obj.fhGainNearVsGasDensity);
      } else {
	 fhGainNearVsGasDensity = NULL;
      }
      if(_obj.fhGainFarVsGasDensity) {
	 fhGainFarVsGasDensity = new TH2D(*_obj.fhGainFarVsGasDensity);
      } else {
	 fhGainFarVsGasDensity = NULL;
      }
      //
      fGasAnalysisResults = _obj.fGasAnalysisResults;
      // copy mean values
      fP_T_Value.clear();
      std::copy(_obj.fP_T_Value.begin (), _obj.fP_T_Value.end (), std::back_inserter(fP_T_Value));

      fGasGainNear_Value.clear();
      std::copy(_obj.fGasGainNear_Value.begin (), _obj.fGasGainNear_Value.end (), std::back_inserter(fGasGainNear_Value));

      fGasGainFar_Value.clear();
      std::copy(_obj.fGasGainFar_Value.begin (), _obj.fGasGainFar_Value.end (), std::back_inserter(fGasGainFar_Value));
   }
}


AliGoofieGasCompositionAnalysis & AliGoofieGasCompositionAnalysis::operator=(const AliGoofieGasCompositionAnalysis &_obj)
{
   if(this!=&_obj) {
      if(_obj.fhGainNearVsGasDensity) {
	 fhGainNearVsGasDensity = new TH2D(*_obj.fhGainNearVsGasDensity);
      } else {
	 fhGainNearVsGasDensity = NULL;
      }
      if(_obj.fhGainFarVsGasDensity) {
	 fhGainFarVsGasDensity = new TH2D(*_obj.fhGainFarVsGasDensity);
      } else {
	 fhGainFarVsGasDensity = NULL;
      }
      //
      fGasAnalysisResults = _obj.fGasAnalysisResults;
      // copy mean values
      fP_T_Value.clear();
      std::copy(_obj.fP_T_Value.begin (), _obj.fP_T_Value.end (), std::back_inserter(fP_T_Value));

      fGasGainNear_Value.clear();
      std::copy(_obj.fGasGainNear_Value.begin (), _obj.fGasGainNear_Value.end (), std::back_inserter(fGasGainNear_Value));

      fGasGainFar_Value.clear();
      std::copy(_obj.fGasGainFar_Value.begin (), _obj.fGasGainFar_Value.end (), std::back_inserter(fGasGainFar_Value));
   }
   return *this;
}


Bool_t AliGoofieGasCompositionAnalysis::CalculateGasComposition(TH1D *_hAmpNear,  TH1D *_hAmpFar,
								TH1D *_hAreaNear, TH1D *_hAreaFar)
{
   Bool_t retVal = true;
   try {
      // get peaks information
      fGasAnalysisResults->fPeakAreaNear = _hAreaNear->GetMean();      // near peak area in pickup spectrum
      fGasAnalysisResults->fPeakAreaFar  = _hAreaFar->GetMean();       // far peak area in pickup spectrum

      // calculate P_T Correlation  --> Temperature from T1 = NEAR -- require some study
      if(fGasAnalysisResults->fTemp1==0) throw (int)-100; // in general such situation should newer happen, but if does one can try to randomize this case with +- 0.0000000001.

      double P_T   = fGasAnalysisResults->fPressure/fabs(fGasAnalysisResults->fTemp1);
      double N_P_T = (1.0/AliGoofieConfigData::fMeanValue_P_T)*P_T;

      // get gas gain value
      fGasAnalysisResults->fGainNear            = GetGainFromPeak(_hAreaNear);                                                             // gain calculated using Near area
      fGasAnalysisResults->fGainNearCorrected   = fGasAnalysisResults->fGainNear + AliGoofieConfigData::fGainNearCorrectionP1*(1.0-N_P_T); // gain calculated using Near area corected for P/T U variations
      fGasAnalysisResults->fGainFar             = GetGainFromPeak(_hAreaFar);                                                              // gain calculated using Far area
      fGasAnalysisResults->fGainFarCorrected    = fGasAnalysisResults->fGainFar + AliGoofieConfigData::fGainFarCorrectionP1*(1.0-N_P_T);   // gain calculated using Far area corected for P/T U variations
      // fill plots for correlation between gas gain and gas density
      fhGainNearVsGasDensity->Fill(N_P_T, fGasAnalysisResults->fGainNear);
      fhGainFarVsGasDensity->Fill(N_P_T, fGasAnalysisResults->fGainFar);
      // store correction factor and gas gain values if required
      if(AliGoofieConfigData::fCalculateMeans) {
	 fP_T_Value.push_back(P_T);
	 fGasGainNear_Value.push_back(fGasAnalysisResults->fGainNearCorrected);
	 fGasGainFar_Value.push_back(fGasAnalysisResults->fGainFarCorrected);
      }
      // calculate gas composition
      if(!CalculateFractionOfCO2AndN2()) throw (int)-102;

   } catch (int _errState) {
      cerr << "Error form <AliGoofieGasCompositionAnalysis::CalculateGasComposition>: Gas Gain calculation error = " << _errState << flush << endl;
      retVal = false;
   }
   return retVal;
}


Double_t AliGoofieGasCompositionAnalysis::GetGainFromPeak(TH1D *_hPeak) throw(int)
{
   double gain            = 0.0;
   double MaxBinValue     = 0.0;
   int    StartBinAt10Pec = 0;
   int    MaxBin          = 0;
   int    FallingEadge    = 0;

   for(int bin = _hPeak->GetNbinsX(); bin > 0; --bin) {
      // Maximum bin
      if(MaxBinValue < _hPeak->GetBinContent(bin)) {
	 // check whether it is not a single bin fluctuation
	 if(_hPeak->GetBinContent(bin) < _hPeak->GetBinContent(bin-1)) {
	    // set current maximum bin
	    MaxBinValue = _hPeak->GetBinContent(bin);
	    MaxBin = bin;
	    if(StartBinAt10Pec==0) {
	       StartBinAt10Pec = bin;
	    }
	 }
      }
      // take first maximum seen from top X position
      if(MaxBinValue >  _hPeak->GetBinContent(bin)) {
	 ++FallingEadge;
	 if(FallingEadge==5) {
	    break;
	 }
      }
   }
   // knowing first maximum find end of distribution
   for(int bin = StartBinAt10Pec; bin > 0; --bin) {
      // start of peak eadge
      if( (MaxBinValue*0.05) < _hPeak->GetBinContent(bin)) {
	 StartBinAt10Pec = bin;
	 break;
      }
   }
   // calculate mean area for range [MaxBin, StartBinAt10Pec]
   double weight       = 0.0;
   double sumVal       = 0.0;
   double sumWeight    = 0.0;
   for(int binXpos=MaxBin; binXpos<StartBinAt10Pec; ++binXpos) {
      // cout << "bin cont : " << _hPeak->GetBinContent(binXpos) << " center : " << _hPeak->GetBinCenter(binXpos) << endl;
      weight     = _hPeak->GetBinContent(binXpos);
      sumVal    += _hPeak->GetBinCenter(binXpos)*weight;
      sumWeight += weight;
   }
   if(sumWeight) {
      gain = sumVal/sumWeight;
      //errResMeanPt = 1.0/sqrt(sumWeight);
   }
   if(gain==0) throw (int)-101;
   return gain;
}


Double_t AliGoofieGasCompositionAnalysis::GetMeanP_T()
{
   double retVal = 0.0;
   for(unsigned int pos = 0; pos<fP_T_Value.size(); ++pos) {
      retVal += fP_T_Value[pos];
   }
   if(fP_T_Value.size()>1) {
      retVal /= fP_T_Value.size();
   }
   return retVal;
}


Double_t  AliGoofieGasCompositionAnalysis::GetMeanGasGainNear()
{
   double retVal = 0.0;
   for(unsigned int pos = 0; pos<fGasGainNear_Value.size(); ++pos) {
      retVal += fGasGainNear_Value[pos];
   }
   if(fGasGainNear_Value.size()>1) {
      retVal /= fGasGainNear_Value.size();
   }
   return retVal;
}


Double_t  AliGoofieGasCompositionAnalysis::GetMeanGasGainFar()
{
   double retVal = 0.0;
   for(unsigned int pos = 0; pos<fGasGainFar_Value.size(); ++pos) {
      retVal += fGasGainFar_Value[pos];
   }
   if(fGasGainFar_Value.size()>1) {
      retVal /= fGasGainFar_Value.size();
   }
   return retVal;
}


Bool_t AliGoofieGasCompositionAnalysis::CalculateFractionOfCO2AndN2()
{
   // ------------------ Fit parameter for ALICE GOOFIE TPC gas mixture ----------------------------
   // Fit Parameter obtained for normailzed drift velocity and gain simulated data
   // Simulation at 760 Torr and 20 C at 400V/cm for Ne-CO2-N2
   // mean drift velocity = 2.52383 [cm/us]
   // mean gain           = 595 [a.u]

   // ------------------ Fit parameter for ALICE GOOFIE TRD gas mixture ----------------------------
   // Fit Parameter obtained for normailzed drift velocity and gain simulated data
   // Simulation at ??? Torr and ?? C at 700V/cm for Xe-CO2-N2.
   // mean drift velocity =  [cm/us]
   // mean gain           =  [a.u]

   // fit function
   // drift_velocity = GasComFitToDvPlaneP0*N2 + GasComFitToDvPlaneP1*CO2 + GasComFitToDvPlaneP2
   // gas_gain       = GasComFitToGainPlaneP0*N2 + GasComFitToGainPlaneP1*CO2 + GasComFitToGainPlaneP2


   // get normalized drift velocity;
   double NormalizedDriftVelocity = fGasAnalysisResults->fDriftVelocityCorrected/AliGoofieConfigData::fMeanValue_DriftVel;
   // at this stage I don't known what will be better near of far or both.
   double NormalizedGasGain       = fGasAnalysisResults->fGainNearCorrected/AliGoofieConfigData::fMeanValue_GainNear;
   //double NormalizedGasGain       = fGasAnalysisResults->fGainFarCorrected/AliGoofieConfigData::fMeanValue_GainFar;

   double denominator = ((AliGoofieConfigData::fGasComFitToDvPlaneP0 * AliGoofieConfigData::fGasComFitToGainPlaneP1)
			 - (AliGoofieConfigData::fGasComFitToDvPlaneP1 * AliGoofieConfigData::fGasComFitToGainPlaneP0));
   if(denominator) {
      // N_2 fraction in gas mixture
      fGasAnalysisResults->fFractionN2   = ( ((NormalizedDriftVelocity - AliGoofieConfigData::fGasComFitToDvPlaneP2)*AliGoofieConfigData::fGasComFitToGainPlaneP1)
					     - ((AliGoofieConfigData::fGasComFitToDvPlaneP1*(NormalizedGasGain - AliGoofieConfigData::fGasComFitToGainPlaneP2))) ) / denominator;
      // CO_2 fraction in gas mixture
      fGasAnalysisResults->fFractionCO2  = ( ((NormalizedGasGain - AliGoofieConfigData::fGasComFitToGainPlaneP2)*AliGoofieConfigData::fGasComFitToDvPlaneP0)
					     - ((AliGoofieConfigData::fGasComFitToGainPlaneP0*(NormalizedDriftVelocity - AliGoofieConfigData::fGasComFitToDvPlaneP2))) ) / denominator;
      return true;
   } else {
      fGasAnalysisResults->fFractionN2   = 0.0;
      fGasAnalysisResults->fFractionCO2  = 0.0;
      return false;
   }
}
