//----------------------------------------------------------------------------
//  Body: AliGoofiePairAnalysis.cxx, v 1.0                    2008/10/01    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>

#include "TH1.h"

#include "AliGoofieConfigData.h"
#include "AliGoofieGasResult.h"
#include "AliGoofieFitFunction.h"

#include "AliGoofiePairAnalysis.h"

using namespace std;


AliGoofiePairAnalysis::AliGoofiePairAnalysis()
{
   fAnalysisMethod         = 0;

   fPairAnalysisResults    = NULL;
   fPairAnalysisLocal      = false;

   fTimePickupSpectrum     = NULL;
   fTimePickupSpectrumAll  = NULL;
   fAreaSpectrumTByNear    = NULL;
   fAreaSpectrumTByFar     = NULL;
   fAmplSpectrumTByNear    = NULL;
   fAmplSpectrumTByFar     = NULL;

#ifdef DEBUG_ONLINE
   cout << "Debuge Info => AliGoofiePairAnalysis object created" << endl;
#endif
}

AliGoofiePairAnalysis::AliGoofiePairAnalysis(AliGoofieGasResult *_pairAnalysisResults)
{
   fAnalysisMethod         = 0;
   fPairAnalysisResults    = _pairAnalysisResults;
   fPairAnalysisLocal      = false;

   fTimePickupSpectrum     = NULL;
   fTimePickupSpectrumAll  = NULL;
   fAreaSpectrumTByNear    = NULL;
   fAreaSpectrumTByFar     = NULL;
   fAmplSpectrumTByNear    = NULL;
   fAmplSpectrumTByFar     = NULL;

#ifdef DEBUG_ONLINE 
   cout << "Debuge Info => AliGoofiePairAnalysis(AliGoofieGasResult *) object created" << endl;
#endif
}

AliGoofiePairAnalysis::AliGoofiePairAnalysis(const AliGoofiePairAnalysis &_obj)
{
   if(this!=&_obj) {
      fAnalysisMethod = 0;

      if(_obj.fPairAnalysisLocal==true) {
	 fPairAnalysisResults = new AliGoofieGasResult(*_obj.fPairAnalysisResults);
      } else {
	 fPairAnalysisResults = _obj.fPairAnalysisResults;
      }
      fPairAnalysisLocal = _obj.fPairAnalysisLocal;

      fDriftVelocityCalculation   = _obj.fDriftVelocityCalculation;     // Get Drift Velocity
      fGasCompositionCalculation  = _obj.fGasCompositionCalculation;    // Gain & Gas Composition Calculation

      if(_obj.fTimePickupSpectrum) {
	 fTimePickupSpectrum = new TH1D(*_obj.fTimePickupSpectrum);
      } else {
	 fTimePickupSpectrum = NULL;
      }
      if(_obj.fTimePickupSpectrumAll) {
	 fTimePickupSpectrumAll = new TH1D(*_obj.fTimePickupSpectrumAll);
      } else {
	 fTimePickupSpectrumAll = NULL;
      }
      if(_obj.fAreaSpectrumTByNear) {
	 fAreaSpectrumTByNear = new TH1D(*_obj.fAreaSpectrumTByNear);
      } else {
	 fAreaSpectrumTByNear = NULL;
      }
      if(fAreaSpectrumTByFar) {
	fAreaSpectrumTByFar = new TH1D(*_obj.fAreaSpectrumTByFar);
      } else {
	 fAreaSpectrumTByFar = NULL;
      }
      if(fAmplSpectrumTByNear) {
	 fAmplSpectrumTByNear = new TH1D(*_obj.fAmplSpectrumTByNear);
      } else {
	 fAmplSpectrumTByNear = NULL;
      }
      if(fAmplSpectrumTByFar) {
	 fAmplSpectrumTByFar = new TH1D(*_obj.fAmplSpectrumTByFar);
      } else {
	 fAmplSpectrumTByFar = NULL;
      }
#ifdef DEBUG_ONLINE
      cout << "Debuge Info => copy constructor of AliGoofiePairAnalysis used" << endl;
#endif
   }
}

AliGoofiePairAnalysis & AliGoofiePairAnalysis::operator=(const AliGoofiePairAnalysis &_obj)
{
  if(this!=&_obj) {

     fAnalysisMethod = _obj.fAnalysisMethod;

     if(_obj.fPairAnalysisLocal==true) {
	if(fPairAnalysisResults) {
	   delete fPairAnalysisResults;
	}
	fPairAnalysisResults = new AliGoofieGasResult(*_obj.fPairAnalysisResults);
     } else {
	fPairAnalysisResults = _obj.fPairAnalysisResults;
     }
     fPairAnalysisLocal = _obj.fPairAnalysisLocal;

     fDriftVelocityCalculation   = _obj.fDriftVelocityCalculation;     // Get Drift Velocity
     fGasCompositionCalculation  = _obj.fGasCompositionCalculation;    // Gain & Gas Composition Calculation

     if(_obj.fTimePickupSpectrum) {
	if(fTimePickupSpectrum) {
	   fTimePickupSpectrum->Delete();
	}
	fTimePickupSpectrum = new TH1D(*_obj.fTimePickupSpectrum);
     } else {
	fTimePickupSpectrum = NULL;
     }
     if(_obj.fTimePickupSpectrumAll) {
	if(fTimePickupSpectrumAll) {
	   fTimePickupSpectrumAll->Delete();
	}
	fTimePickupSpectrumAll = new TH1D(*_obj.fTimePickupSpectrumAll);
     } else {
	fTimePickupSpectrumAll = NULL;
     }
     if(_obj.fAreaSpectrumTByNear) {
	if(fAreaSpectrumTByNear) {
	   fAreaSpectrumTByNear->Delete();
	}
	fAreaSpectrumTByNear = new TH1D(*_obj.fAreaSpectrumTByNear);
     } else {
	fAreaSpectrumTByNear = NULL;
     }
     if(fAreaSpectrumTByFar) {
	if(fAreaSpectrumTByFar) {
	   fAreaSpectrumTByFar->Delete();
	}
	fAreaSpectrumTByFar = new TH1D(*_obj.fAreaSpectrumTByFar);
     } else {
	fAreaSpectrumTByFar = NULL;
     }
     if(fAmplSpectrumTByNear) {
	if(fAmplSpectrumTByNear) {
	   fAmplSpectrumTByNear->Delete();
	}
	fAmplSpectrumTByNear = new TH1D(*_obj.fAmplSpectrumTByNear);
     } else {
	fAmplSpectrumTByNear = NULL;
     }
     if(fAmplSpectrumTByFar) {
	if(fAmplSpectrumTByFar) {
	   delete fAmplSpectrumTByFar;
	}
	fAmplSpectrumTByFar = new TH1D(*_obj.fAmplSpectrumTByFar);
     } else {
	fAmplSpectrumTByFar = NULL;
     }
#ifdef DEBUG_ONLINE
     cout << "Debuge Info => overloaded = operator of AliGoofiePairAnalysis used" << endl;
#endif
  }
  return *this;
}

AliGoofiePairAnalysis::~AliGoofiePairAnalysis()
{
   if(fPairAnalysisLocal) {
      if(fPairAnalysisResults) {
	 delete fPairAnalysisResults;
	 fPairAnalysisResults = NULL;
      }
      fPairAnalysisLocal = false;
   }
   if(fTimePickupSpectrum) {
      fTimePickupSpectrum->Delete();
      fTimePickupSpectrum = NULL;
   }
   if(fTimePickupSpectrumAll) {
      fTimePickupSpectrumAll->Delete();
      fTimePickupSpectrumAll = NULL;
   }
   if(fAreaSpectrumTByNear) {
      fAreaSpectrumTByNear->Delete();
      fAreaSpectrumTByNear = NULL;
   }
   if(fAreaSpectrumTByFar) {
      fAreaSpectrumTByFar->Delete();
      fAreaSpectrumTByFar = NULL;
   }
   if(fAmplSpectrumTByNear) {
      fAmplSpectrumTByNear->Delete();
      fAmplSpectrumTByNear = NULL;
   }
   if(fAmplSpectrumTByFar) {
      fAmplSpectrumTByFar->Delete();
      fAmplSpectrumTByFar = NULL;
   }
#ifdef DEBUG_ONLINE
  cout << "Debuge Info => AliGoofiePairAnalysis object destroyed" << endl;
#endif
}


void AliGoofiePairAnalysis::CreateHistograms(Int_t _packNumber, Int_t _ADCBins)
{
   TString hName = "TimeSpectrum_"; hName += _packNumber;
   if(!fTimePickupSpectrum) {
      fTimePickupSpectrum = new TH1D(hName.Data(), hName.Data(), _ADCBins*2, 0.0, _ADCBins);
   } else {
      fTimePickupSpectrum->Reset();
      fTimePickupSpectrum->SetName(hName.Data());
      fTimePickupSpectrum->SetTitle(hName.Data());
   }

   hName = "TimeSpectrumAll_"; hName += _packNumber;
   if(!fTimePickupSpectrumAll) {
      fTimePickupSpectrumAll = new TH1D(hName.Data(), hName.Data(), _ADCBins, 0.0, _ADCBins);
   } else {
      fTimePickupSpectrumAll->Reset();
      fTimePickupSpectrumAll->SetName(hName.Data());
      fTimePickupSpectrumAll->SetTitle(hName.Data());
   }

   hName = "AreaNearSpectrum_"; hName += _packNumber;
   if(!fAreaSpectrumTByNear) {
       fAreaSpectrumTByNear = new TH1D(hName.Data(), hName.Data(), 250, 0.0, 10000.0);
   } else {
      fAreaSpectrumTByNear->Reset();
      fAreaSpectrumTByNear->SetName(hName.Data());
      fAreaSpectrumTByNear->SetTitle(hName.Data());
   }

   hName = "AreaFarSpectrum_"; hName += _packNumber;
   if(!fAreaSpectrumTByFar) {
      fAreaSpectrumTByFar = new TH1D(hName.Data(), hName.Data(), 250, 0.0, 10000.0);
   } else {
      fAreaSpectrumTByFar->Reset();
      fAreaSpectrumTByFar->SetName(hName.Data());
      fAreaSpectrumTByFar->SetTitle(hName.Data());
   }

   hName = "AmplitudeNearSpectrum_"; hName += _packNumber;
   if(!fAmplSpectrumTByNear) {
      fAmplSpectrumTByNear = new TH1D(hName.Data(), hName.Data(), 306, 0.0, 1024.0);
   } else {
      fAmplSpectrumTByNear->Reset();
      fAmplSpectrumTByNear->SetName(hName.Data());
      fAmplSpectrumTByNear->SetTitle(hName.Data());
   }

   hName = "AmplitudeFarSpectrum_"; hName += _packNumber;
   if(!fAmplSpectrumTByFar) {
      fAmplSpectrumTByFar = new TH1D(hName.Data(), hName.Data(), 306, 0.0, 1024.0); 
   } else {
      fAmplSpectrumTByFar->Reset();
      fAmplSpectrumTByFar->SetName(hName.Data());
      fAmplSpectrumTByFar->SetTitle(hName.Data());
   }
}


void AliGoofiePairAnalysis::SetGoofieGasResult(AliGoofieGasResult  *_PairAnalysisResults)
{
   if(_PairAnalysisResults) {
      fPairAnalysisResults = _PairAnalysisResults;
      fPairAnalysisLocal   = false;
   }
}


void AliGoofiePairAnalysis::MakeGoofieGasResult()
{
   if(!fPairAnalysisResults) {
      fPairAnalysisResults = new AliGoofieGasResult();
      fPairAnalysisLocal   = true;
   }
}


void AliGoofiePairAnalysis::PrintResults(Bool_t _TrueFalse)
{
   if(_TrueFalse) {
      fPairAnalysisResults->DumpValues(std::cout);
   }
}


Bool_t AliGoofiePairAnalysis::SetSpectraForAnalysis()
{
   // only good event  if event quality cuts required
   if(fiterEvent->fPeakTriggeredBy>0 || !AliGoofieConfigData::fEventQualityCutOnOff) {
      // integrated spectrum
      for(int bin=0; bin<fiterEvent->fLocalCopyOfNumberOfADCSampels; ++bin) {
	 fTimePickupSpectrumAll->SetBinContent(bin+1, fTimePickupSpectrumAll->GetBinContent(bin+1) + fiterEvent->GetEntryInDataPickup(bin));
      }
      // only MPV from event fit
      fTimePickupSpectrum->Fill(fiterEvent->fPickupPeakInfo.fPosition);
      // events triggered by Near counter  
      if(fiterEvent->fPeakTriggeredBy==2) {

	 if(AliGoofieConfigData::fGainFromTriggersOrPickup == 0) {
	    fAreaSpectrumTByNear->Fill(fiterEvent->fPickupPeakInfo.fArea);
	    fAmplSpectrumTByNear->Fill(fiterEvent->fPickupPeakInfo.fAmplitude);
	 } else {
	    fAreaSpectrumTByNear->Fill(fiterEvent->fTriggerNearPeakInfo.fArea);
	    fAmplSpectrumTByNear->Fill(fiterEvent->fTriggerNearPeakInfo.fAmplitude);
 	 }
      }
      // events triggered by Far counter
      if(fiterEvent->fPeakTriggeredBy==1) {
	 if(AliGoofieConfigData::fGainFromTriggersOrPickup == 0) {
	    fAreaSpectrumTByFar->Fill(fiterEvent->fPickupPeakInfo.fArea);
	    fAmplSpectrumTByFar->Fill(fiterEvent->fPickupPeakInfo.fAmplitude);
	 } else {
	    fAreaSpectrumTByFar->Fill(fiterEvent->fTriggerFarPeakInfo.fArea);
	    fAmplSpectrumTByFar->Fill(fiterEvent->fTriggerFarPeakInfo.fAmplitude);
	 }
      }
   } else return false;

   return true;
}


Bool_t AliGoofiePairAnalysis::SetEnvironmentVariableForAnalysis(Bool_t _eventQuality, Bool_t _lastEvent, Int_t _NoOfEvent)
{
   // set T, P, HV
   if(_eventQuality) {
      fPairAnalysisResults->fFieldCageHV += fiterEvent->GetEfVoltage();
      fPairAnalysisResults->fPickupHV    += fiterEvent->GetHVPickup();
      fPairAnalysisResults->fTemp1       += fiterEvent->GetTemp1();
      fPairAnalysisResults->fTemp2       += fiterEvent->GetTemp2();
      fPairAnalysisResults->fPressure    += fiterEvent->GetPressure();
   }
   if(_NoOfEvent>1) {
      if(_lastEvent) {
	 fPairAnalysisResults->fFieldCageHV /= (double)_NoOfEvent;
	 fPairAnalysisResults->fPickupHV    /= (double)_NoOfEvent;
	 fPairAnalysisResults->fTemp1       /= (double)_NoOfEvent;  // in K default
	 fPairAnalysisResults->fTemp2       /= (double)_NoOfEvent;  // in K default
	 fPairAnalysisResults->fPressure    /= (double)_NoOfEvent;
	 // convert to requested units
	 if(AliGoofieConfigData::fTemperatureScale) {
	    fPairAnalysisResults->fTemp1       -= 273.16;           // in C
	    fPairAnalysisResults->fTemp2       -= 273.16;           // in C
	 }
      } else return false;
   }
   return true;
}


Bool_t AliGoofiePairAnalysis::CheckEventQuality()
{
   // global settings from AliGoofieConfigData class             --> start 2500 events
   if(!AliGoofieConfigData::fEventQualityCutOnOff)  return true;
   // event quality selection
   // check from distributions
   // ----------------Trigger quality --------------------------
   if(fiterEvent->fPeakTriggeredBy==1) {
      if(fiterEvent->fTriggerFarPeakInfo.fAmplitude < AliGoofieConfigData::fTriggerMinimumAmplitude)       return false;
      if(fiterEvent->fTriggerFarPeakInfo.fPosition  > AliGoofieConfigData::fTriggerMaximumPosition)        return false;
   }
   if(fiterEvent->fPeakTriggeredBy==2) {
      if(fiterEvent->fTriggerNearPeakInfo.fAmplitude < AliGoofieConfigData::fTriggerMinimumAmplitude+5.0)  return false;
      if(fiterEvent->fTriggerNearPeakInfo.fPosition  > AliGoofieConfigData::fTriggerMaximumPosition)       return false;
   }
   //                                                            --> ~ -52 events
   // --------------- Pickup quality --------------------------- --> 
   // Error peak position
   //if(fiterEvent->fPickupPeakInfo.fErrPosition <= 0.025)             return false;
   if(fiterEvent->fPickupPeakInfo.fErrPosition > 1.5)               return false;
   //                                                            --> ~ -15 events
   // peak widht
   if(fiterEvent->fPickupPeakInfo.fWidth > 40.0)                    return false;
   if(fiterEvent->fPickupPeakInfo.fWidth < AliGoofieConfigData::fPickupMinimumWidth)                        return false;
   //                                                            --> ~ -77 events
   // amplitude must be larger then ...
   if(fiterEvent->fPickupPeakInfo.fAmplitude <= AliGoofieConfigData::fPickupMinimumAmplitude)               return false;
   //                                                            --> ~ -15 events
   // offset must be positive and larger then 
   if(fiterEvent->fPickupPeakInfo.fOffset <= AliGoofieConfigData::fPickupMinimumOffset)                     return false;
   //                                                            --> ~ -1 events
   // only one pulse per spectrum
   //if(fiterEvent->fPickupPeakInfo.fMultiPeakSpectrum > 1)           return flase;
   //Double_t fChi2;                    // fitting quality
   // ----------------------------------------------------------
   //Double_t fGradient;                // falling edge slope
   return true;
}

Bool_t AliGoofiePairAnalysis::MakeAnalysis(const std::deque<AliGoofieEventInfo> &_eventList,
					   Int_t _currentPack,
					   AliGoofieFitFunction *_fFitSpectrumInt)
{
   Bool_t retVal = false;

   fPairAnalysisResults->Clear();
   {
      // necessary to create ADC spectra containers 
      Int_t lNumberOfADCs = _eventList[0].fLocalCopyOfNumberOfADCSampels;
      // create pack analysis histgrams
      CreateHistograms(_currentPack, lNumberOfADCs);
   }
   // get information from events
   unsigned int lNumberOfAllEvents  = 0;
   fNumberOfUsedEvents = 0;
   for(fiterEvent = _eventList.begin(); fiterEvent != _eventList.end(); ++fiterEvent) {
      // all events
      ++lNumberOfAllEvents;
      //Check Event Quality
      bool eventQuality = CheckEventQuality();
      if(eventQuality) {
	 // counter of used events -- necessary to calculate average values
	 ++fNumberOfUsedEvents;
	 // fill spectra for gas analysis
	 SetSpectraForAnalysis();
      }
      // fill average T, P, HV values -- data only from used (eventQuality == true) events
      SetEnvironmentVariableForAnalysis(eventQuality, (lNumberOfAllEvents==_eventList.size()), fNumberOfUsedEvents);
   }
   // get information from pack
   // calculate drift velocity + corrected drift velocity
   fDriftVelocityCalculation.SetGasResultsPrt(fPairAnalysisResults);
   retVal = fDriftVelocityCalculation.CalculateDriftVelocity(fTimePickupSpectrumAll, fTimePickupSpectrum, _fFitSpectrumInt);
   if(retVal) {
      // calculate gas gain + corrected gas gain + gas composition
      fGasCompositionCalculation.SetGasResultsPrt(fPairAnalysisResults);
      retVal = fGasCompositionCalculation.CalculateGasComposition(fAmplSpectrumTByNear, fAmplSpectrumTByFar,
								  fAreaSpectrumTByNear, fAreaSpectrumTByFar);
   }
   return retVal;
}
