//----------------------------------------------------------------------------
//  Header: AliGoofiePairAnalysis.h, v 1.0                      2008/10/01  //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef ALIGOOFIEPAIRANALYSIS
#define ALIGOOFIEPAIRANALYSIS

#include <deque>

#include "AliGoofieEventInfo.h"
#include "AliGoofieDriftVelocityAnalysis.h"
#include "AliGoofieGasCompositionAnalysis.h"

class AliGoofieGasResult;
class AliGoofieFitFunction;

class AliGoofiePairAnalysis
{
public:
   AliGoofiePairAnalysis();
   AliGoofiePairAnalysis(AliGoofieGasResult *_pairAnalysisResults);
   AliGoofiePairAnalysis(const AliGoofiePairAnalysis &_obj);
   virtual ~AliGoofiePairAnalysis();
   AliGoofiePairAnalysis & operator= (const AliGoofiePairAnalysis &_obj);

   void   SetAnalysisMethod(const Int_t _AnalysisMethod)         {fAnalysisMethod = _AnalysisMethod;}
   void   CreateHistograms(Int_t _packNumber, Int_t _ADCBins);
   Bool_t SetSpectraForAnalysis();                                                                       // get results from peaks
   Bool_t SetEnvironmentVariableForAnalysis(Bool_t _eventQuality, Bool_t _lastEvent, Int_t _NoOfEvent);  // calculate average value
   Bool_t CheckEventQuality();                                                                           // event quality flag

   Bool_t MakeAnalysis(const std::deque<AliGoofieEventInfo> &_eventList,
		       Int_t _currentPack,
		       AliGoofieFitFunction *_fFitSpectrumInt);            // make pair analysis

   void   PrintResults(Bool_t _TrueFalse);                                                               // dump current results
   Int_t  GetCurrentNumberOfUsedEvents()                                {return fNumberOfUsedEvents;}

   AliGoofieDriftVelocityAnalysis  *GetPrtToDriftVelocityAnalysis()     {return &fDriftVelocityCalculation;}
   AliGoofieGasCompositionAnalysis *GetPrtToGasCompositionAnalysis()    {return &fGasCompositionCalculation;}
   // service function
   void   SetGoofieGasResult(AliGoofieGasResult  *_PairAnalysisResults);
   void   MakeGoofieGasResult();

public:
   Int_t                            fAnalysisMethod;               // 0 = integrated spectra, 1 = event by event
   AliGoofieGasResult              *fPairAnalysisResults;          // store all gas analysis results
   AliGoofieDriftVelocityAnalysis   fDriftVelocityCalculation;     // Get Drift Velocity
   AliGoofieGasCompositionAnalysis  fGasCompositionCalculation;    // Gain & Gas Composition Calculation
   TH1D                            *fTimePickupSpectrum;           // used to calculate deift velocity
   TH1D                            *fTimePickupSpectrumAll;        // used to calculate deift velocity integrated spectrum
   TH1D                            *fAreaSpectrumTByNear;          // used to calculate gain from peak area (near)
   TH1D                            *fAreaSpectrumTByFar;           // used to calculate gain from peak area (far)
   TH1D                            *fAmplSpectrumTByNear;          // used to calculate gain from peak amplitude (near)
   TH1D                            *fAmplSpectrumTByFar;           // used to calculate gain from peak amplitude (far)

private:
   Bool_t                           fPairAnalysisLocal;            // true if created localy
   Int_t                            fNumberOfUsedEvents;           // current number of events used for analysis

   std::deque<AliGoofieEventInfo>::const_iterator    fiterEvent;   // deque iterator
};
#endif
