//----------------------------------------------------------------------------
//  Header: AliGoofiePeakProperties.cpp, v 1.0                2008/08/02    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------
#include <iostream>

#include "TF1.h"
#include "TH1D.h"

#include "AliGoofieFitFunction.h"
#include "AliGoofiePeakProperties.h"

using namespace std;

TSpectrum *AliGoofiePeakProperties::fFindCandidates = NULL;

AliGoofiePeakProperties::AliGoofiePeakProperties()
{
   fName.clear();                           // peak name
   fPosition          = 0.0;                // maximum peak postion
   fErrPosition       = 0.0;                // err maximum peak postion
   fWidth             = 0.0;                // at half peak position (usualy peak is asymetric)
   fErrWidth          = 0.0;                // err at half peak position (usualy peak is asymetric)
   fAmplitude         = 0.0;                // peak hight (offset+peak higth = total peak hight)
   fErrAmplitude      = 0.0;                // err peak hight (offset+peak higth = total peak hight)
   fOffset            = 0.0;                // baseline
   fErrOffset         = 0.0;                // err baseline
   fArea              = 0.0;                // 95 % of total peak
   fChi2              = 0.0;                // fitting quality
   fGradient          = 0.0;                // falling edge slope
   fMultiPeakSpectrum = 0;
   fPeakPosPrt        = NULL;
   fPeakValPrt        = NULL;
}

AliGoofiePeakProperties::~AliGoofiePeakProperties()
{
   fPeakPosPrt = NULL;
   fPeakValPrt = NULL; 
}

AliGoofiePeakProperties::AliGoofiePeakProperties(const AliGoofiePeakProperties &_obj)
{
   if(this != &_obj) {
      fName.assign(_obj.fName);                        // peak name
      fPosition          = _obj.fPosition;             // maximum peak postion
      fErrPosition       = _obj.fErrPosition;          // err maximum peak postion
      fWidth             = _obj.fWidth;                // at half peak position (usualy peak is asymetric)
      fErrWidth          = _obj.fErrWidth;             // err at half peak position (usualy peak is asymetric)
      fAmplitude         = _obj.fAmplitude;            // peak hight (offset+peak higth = total peak hight)
      fErrAmplitude      = _obj.fErrAmplitude;         // err peak hight (offset+peak higth = total peak hight)
      fOffset            = _obj.fOffset;               // baseline
      fErrOffset         = _obj.fErrOffset;            // err baseline
      fArea              = _obj.fArea;                 // 95 % of total peak
      fChi2              = _obj.fChi2;                 // fitting quality
      fGradient          = _obj.fGradient;             // falling enge slop
      fMultiPeakSpectrum = _obj.fMultiPeakSpectrum;    //
      fPeakPosPrt        = NULL;
      fPeakValPrt        = NULL;
   }
}

AliGoofiePeakProperties & AliGoofiePeakProperties::operator= (const AliGoofiePeakProperties &_obj)
{
   if(this != &_obj) {
      fName.assign(_obj.fName);                        // peak name
      fPosition          = _obj.fPosition;             // maximum peak postion
      fErrPosition       = _obj.fErrPosition;          // err maximum peak postion
      fWidth             = _obj.fWidth;                // at half peak position (usualy peak is asymetric)
      fErrWidth          = _obj.fErrWidth;             // err at half peak position (usualy peak is asymetric)
      fAmplitude         = _obj.fAmplitude;            // peak hight (offset+peak higth = total peak hight)
      fErrAmplitude      = _obj.fErrAmplitude;         // err peak hight (offset+peak higth = total peak hight)
      fOffset            = _obj.fOffset;               // baseline
      fErrOffset         = _obj.fErrOffset;            // err baseline
      fArea              = _obj.fArea;                 // 95 % of total peak
      fChi2              = _obj.fChi2;                 // fitting quality
      fGradient          = _obj.fGradient;             // falling enge slop
      fMultiPeakSpectrum = _obj.fMultiPeakSpectrum;    //
      fPeakPosPrt        = NULL;
      fPeakValPrt        = NULL;
   }
   return *this;
}

void AliGoofiePeakProperties::Clear()
{
   fPosition          = 0.0;                // maximum peak postion
   fErrPosition       = 0.0;                // err maximum peak postion
   fWidth             = 0.0;                // at half peak position (usualy peak is asymetric)
   fErrWidth          = 0.0;                // err at half peak position (usualy peak is asymetric)
   fAmplitude         = 0.0;                // peak hight (offset+peak higth = total peak hight)
   fErrAmplitude      = 0.0;                // err peak hight (offset+peak higth = total peak hight)
   fOffset            = 0.0;                // baseline
   fErrOffset         = 0.0;                // err baseline
   fArea              = 0.0;                // 95 % of total peak
   fChi2              = 0.0;                // fitting quality
   fGradient          = 0.0;                // falling edge slope
   fMultiPeakSpectrum = 0;
}

Bool_t AliGoofiePeakProperties::FitSpectrum(TH1D *_spectrum, AliGoofieFitFunction *_fFitSpectrum)
{
   Bool_t retValue = false;
   if(FindMaximumPeak(_spectrum)) {
      // calculate slope
      CalculateGradien(_spectrum);
      // fit spectrum
      retValue = _fFitSpectrum->MakeFit(_spectrum, this);
#ifdef DEBUG_ONLINE
      cout << "======================= Fit Results ===============================" << endl;
      cout << "Peak name   = " << fName << endl;
      cout << "Peak pos    = " << fPosition  << " +- " << fErrPosition << endl;
      cout << "Amplitude   = " << fAmplitude << " +- " << fErrAmplitude << endl;
      cout << "Width       = " << fWidth << " +- " << fErrWidth << endl;
      cout << "Offset      = " << fOffset << " +- " << fErrOffset << endl;
      cout << "Area        = " << fArea << endl;    
      cout << "chi2        = " << fChi2 << endl;
      cout << "gradient    = " << fGradient << endl;
      cout << "Multi peaks = " << fMultiPeakSpectrum << endl;
      cout << "===================================================================" << endl;
#endif
   }
   return retValue;
}


Bool_t AliGoofiePeakProperties::FindMaximumPeak(TH1D *_spectrum, AliGoofiePeakProperties *_p2)
{
   this->Clear();

   if(!fFindCandidates) {
      fFindCandidates = new TSpectrum(100);
   }
   int nPeaks = fFindCandidates->Search(_spectrum, 3, "newgoff", 0.2);
   if(nPeaks==0) {
#ifdef FIT_ERROR
      cerr << "Error in < AliGoofiePeakProperties::FindMaximumPeak>: any peak was found" << flush << endl;       
#endif
      return false;
   }
   fPeakPosPrt = fFindCandidates->GetPositionX();
   fPeakValPrt = fFindCandidates->GetPositionY();

   // estimate background
   int N=0;
   fOffset = 0.0;
   for(int bin=1; bin<_spectrum->GetNbinsX(); ++bin) {
      for(int a=0; a<nPeaks; ++a) {
	 if((bin<_spectrum->FindBin(fPeakPosPrt[a])*0.92) || (bin>_spectrum->FindBin(fPeakPosPrt[a])*1.2)) {
	    fOffset += _spectrum->GetBinContent(bin);
	    ++N;
	 }
      }
   }
   fOffset /= N;
   // find maximum
   for(int a=0; a<nPeaks; ++a) {
      if(fPeakValPrt[a] > fOffset*1.25) {
	 ++fMultiPeakSpectrum;
	 if(fAmplitude<(fPeakValPrt[a]-fOffset)) {
	    fPosition  = fPeakPosPrt[a];
	    fAmplitude = fPeakValPrt[a]-fOffset;
	 }
      }
   }
   // check whether any peak is above offset
   if(!fMultiPeakSpectrum) {
#ifdef FIT_ERROR
      cerr << "Error in <AliGoofiePeakProperties::FindMaximumPeak>: in " << fName << " all peaks (" << nPeaks << ") within noise level" << flush << endl;       
#endif
      return false;
   }
   // find width, def. as pos(mvp) - FirstFromLeftBinPos(offset*1.25)
   for(int bin=_spectrum->FindBin(fPosition); bin>1; --bin) {
      ++fWidth;
      if(_spectrum->GetBinContent(bin)<(fOffset*1.25)) break;
   }
   // ---------------------------------------------------------------
   //                        Find second peak                      //
   if(_p2) {
      _p2->Clear();
      _p2->fOffset = fOffset;
      _p2->fWidth  = fWidth;
      // find second maximum
      for(int a=0; a<nPeaks; ++a) {
	 if(fPeakValPrt[a] > fOffset*1.25) {
	    ++_p2->fMultiPeakSpectrum;
	    if((_p2->fAmplitude<(fPeakValPrt[a]-_p2->fOffset)) && (int(fPeakValPrt[a]-_p2->fOffset)<int(fAmplitude))) {
	       _p2->fPosition  = fPeakPosPrt[a];
	       _p2->fAmplitude = fPeakValPrt[a]-_p2->fOffset;
	    }
	 }
      }
   }
#ifdef DEBUG_ONLINE
   cout << "======================= INPUT FIT PARAMETER =======================" << endl;
   cout << "For       : " << fName << endl;
   cout << "Peak pos  = " << fPosition  << endl;
   cout << "Amplitude = " << fAmplitude << endl;
   cout << "Width     = " << fWidth << endl;
   cout << "Offset    = " << fOffset << endl;
   cout << "===================================================================" << endl;
#endif
   return true;
}

void AliGoofiePeakProperties::CalculateGradien(const TH1D *_spectrum)
{
   fGradient = 0.0;
}
