//----------------------------------------------------------------------------
//  Header: AliGoofiePeakProperties.h, v 1.0                  2008/08/02    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef ALIGOOFIEPEAKPROPERTIES
#define ALIGOOFIEPEAKPROPERTIES

#include <string>
#include <deque>

#include <TObject.h>
#include <TSpectrum.h>


class AliGoofieFitFunction;
class TF1;
class TH1D;

class AliGoofiePeakProperties
{
public:
   AliGoofiePeakProperties();
   ~AliGoofiePeakProperties();
   AliGoofiePeakProperties(const AliGoofiePeakProperties &_obj);
   AliGoofiePeakProperties & operator= (const AliGoofiePeakProperties &_obj);
   void Clear();

   Bool_t  FitSpectrum(TH1D *_spectrum, AliGoofieFitFunction *_fFitSpectrum);         // fitting and updating peak parameter 
   void    CalculateGradien(const TH1D *_spectrum);                                   // falling edge slope n.i.
   Bool_t  FindMaximumPeak(TH1D *_spectrum, AliGoofiePeakProperties *_p2 = NULL);     // temporaly overwrite peak parameter
                                                                                      // values to use them as start values for fit
   static void DeleteTSpectrumObj() {if(fFindCandidates) {delete fFindCandidates; fFindCandidates=NULL;}}

public:
   std::string fName;                    // peak name
   Double_t    fPosition;                // maximum peak postion
   Double_t    fErrPosition;             // err maximum peak postion
   Double_t    fWidth;                   // at half peak position (usualy peak is asymetric)
   Double_t    fErrWidth;                // err at half peak position (usualy peak is asymetric)
   Double_t    fAmplitude;               // peak hight (offset+peak higth = total peak hight)
   Double_t    fErrAmplitude;            // err peak hight (offset+peak higth = total peak hight)
   Double_t    fOffset;                  // baseline
   Double_t    fErrOffset;               // err baseline
   Double_t    fArea;                    // ~95 % of total peak - via Intergare function, integration range in setup
   Double_t    fChi2;                    // fitting quality
   Double_t    fGradient;                // falling edge slope
   Int_t       fMultiPeakSpectrum;       // number of peaks found in the specrum

private:
   Float_t              *fPeakPosPrt;
   Float_t              *fPeakValPrt;
   static TSpectrum     *fFindCandidates;
};
#endif
