//----------------------------------------------------------------------------
//  Body: CGoofieBoard.cpp,v 1.0                                2006/01/20  //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2006                                 //
//                                                                          //
//  Definition of class CGoofieBoard                                        //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

//standard
#include <iostream>
#include <list>
#include <vector>
#include <time.h>

// goofie board
#include "CGRegVal.h"
#include "CGUnpack.h"

#include "CMacroDef.h"
#include "COnLineExeption.h"
#include "CDataStructure.h"
#include "CGoofieBoard.h"


// TODO 
// 
// 1-find a event quality definition

using namespace std;

CGoofieBoard::CGoofieBoard()
{
   funpackStat             = 1;             // 1 - data is uncompressed, 0 - RAW (just for tests)
   ftriggerMode            = 1;             // 1 - signal trigger, 0 - software trigger (just for tests)
   fswappedTriggerChannels = false;
   unpackObj               = new CGUnpack();
   fexeptionStatus         = new COnLineExeption();
}

CGoofieBoard::~CGoofieBoard()
{
   if(unpackObj) {
      delete unpackObj;
      unpackObj = NULL;
   }
   if(fexeptionStatus) {
     delete fexeptionStatus;
     fexeptionStatus = NULL;
   }
}
 
unsigned int CGoofieBoard::InitializeCard(const string _configFile)
{
   unsigned int err       = 0;
   fconfigurationFileName = _configFile;
   
   if(unpackObj->IsInit()==0) {
      // set value from configuration file 
      unpackObj->GetRegPointer()->SetValueFromFile(_configFile);
      // write all register values into the GOOFIE board
      err = unpackObj->InitCVal();
//#ifdef DEBUG_ONLINE
      // GOOFIE Register information
      cout << *unpackObj->GetRegPointer() << endl; 
//#endif      
      // initialize read out modes -- info from constructor
      unpackObj->SetUnpacStatus(funpackStat);
      unpackObj->SetTriggerMode(ftriggerMode);      

      // get register values / test connection with the board 
      CGRegVal regRet = unpackObj->GetRegistersValues();
      err = unpackObj->GetErrorCode();
      if(!err) {
	DEBUGINFO("Debuge Info => CGoofieBoard::InitializeCard => initialization of the GOOFIE board is successful",err)
      }
      // get value from configuration file 
      fswappedTriggerChannels = regRet.fIsChannelACalledNear;
  
      cout << "test regRet.fIsChannelACalledNear = " << regRet.fIsChannelACalledNear << "  " << fswappedTriggerChannels << endl;
 
   }
   if(err!=0) {
      fexeptionStatus->SetExeptionError(err, "<CGoofieBoard::InitializeCard>:: Initialization of the GOOFIE board failed");
      unpackObj->ClearErrCode();
   }
   return err;
}

unsigned int CGoofieBoard::SetReadMode(bool _updata)
{
   unsigned int err = 0;
   if(_updata) {
      try {
	 if(unpackObj->GetErrorCode()!=0)
	    throw (int)-1;
	 // name of configuration file is set in InitializeCard or SetConfigurationFileName
	 unpackObj->GetRegPointer()->SetValueFromFile(fconfigurationFileName.c_str());
	 // update
	 err = !(unpackObj->InitCVal());
	 if(err!=0)
	    throw (int)-2;	 
	    
      } catch (int _err) {
	 switch(_err) {
	 case -1:
	    fexeptionStatus->SetExeptionError(_err, "<CGoofieBoard::SetReadMode>:: Error of unpacker object");
	    break;
	 case -2:
	    fexeptionStatus->SetExeptionError(_err, "<CGoofieBoard::SetReadMode>:: Initialization of GOOFIE board failed");
	    break;
	 }
	 unpackObj->ClearErrCode();
      }
   }
   return err;
}


unsigned int CGoofieBoard::GetEvent(CDataStructure *_eventStr)
{
   if(unpackObj->GetErrorCode()==0) {
     // list iterator
     list<vector<unsigned int> >::iterator iteChA;

     // get or wait for next event
     do {
       // read channels
       iteChA = unpackObj->ReadAllChannels(0).begin();
       //
       usleep(50); //wait 50us. // temporary           
     } while (unpackObj->EventReadyToRead()!=1);

     list<vector<unsigned int> >::iterator iteChB = iteChA;
     ++iteChB;
     list<vector<unsigned int> >::iterator iteChC = iteChB;
     ++iteChC;
	 
     // event quality
     _eventStr->fGoodEvent       = 1;                  // find a quality definition XX
     // triger information
     _eventStr->fEventTime       = time(NULL);         //second since January 1, 1970
     _eventStr->fTriggerChannel  = unpackObj->WhichChTrigged();  
     // fill spectrum arreies
     for(int i=0; i<(_eventStr->fLocalCopyOfNumberOfADCSampels); ++i) {
       _eventStr->fDataPickup[i] = (*iteChC).at(i);
       if(fswappedTriggerChannels) {
	 _eventStr->fDataNear[i]   = (*iteChA).at(i);
	 _eventStr->fDataFar[i]    = (*iteChB).at(i);
       } else {
	 _eventStr->fDataNear[i]   = (*iteChB).at(i);
	 _eventStr->fDataFar[i]    = (*iteChA).at(i);
       }
       //Test XXX
       //cout <<  _eventStr->fDataPickup[i] << "\t" <<  _eventStr->fDataNear[i] << "\t" << _eventStr->fDataFar[i] << endl;
     }
   } else
     return 1;
   
   return 0;
}
