//----------------------------------------------------------------------------
//  Header: CReadNI.h,v 1.0                                     2006/01/20  //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2006                                 //
//                                                                          //
//  Declaration of class CReadNI                                            //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef CREADNI_H
#define CREADNI_H

#include <map>
#include <string>
#include <deque>
#include <NIDAQmxBase.h>

class SConfigData;
class CDataStructure;
class COnLineExeption;

class CReadNI
{
public:
   // --- constructor--
   CReadNI();
   //---- destructor--
   virtual ~CReadNI();

   // initialization
   unsigned int InitializeCard(const std::string _configFile="");
   // read configuration values from file
   void         SetNICalValueFromFile(const std::string _filename);

   // reset channel states
   void         ResetUsedChannel();

   // read data
   double       ReadChannel(unsigned int _chNr);    // read channel number xx
   void         ReadDataForGOOFIE();                // read those channels which are relevant for GOOFIE

   // applay calibrations parameters 
   void         Calibrate();
   bool         GetCalibrationStatus()                {return fCalibrationOnOff;}
   void         SetCalibrationStatus(bool _cs)        {fCalibrationOnOff=_cs;}

   //get latest read value
   double       GetChannelValue(unsigned int _chNr)   {return fChannelValue[_chNr];}

   // provide slow control information to the event bilder
   unsigned int GetSCData(CDataStructure *_eventStr, bool _update=TRUE);

   // test function
   void         SaveDataInFile(std::ofstream *fout);       // dump values to file
   void         ShowAcq();                                 // dump values on screan
   std::string  GetChannelsValues();                       // dump values to string

private:
   // channel to string mapping
   std::map<unsigned int, std::string>     fPhysicalChannelName;      // like AI0, AI5 etc.
   std::map<unsigned int, std::string>     fAssignedChannelName;      // like voltage A, voltage B, etc.
   // read values
   std::map<unsigned int, double>          fChannelValue;             // current value from channel AI#
   // channels used
   std::map<unsigned int, bool>            fChannelUsed;              // which channel has benn read
                                                                      // function ResetUsedChannel() will clear all states in this container
   // appaly calibrations
   bool                                    fCalibrationOnOff;
   // calibration values
   std::deque<std::pair<double, double> >  fCalibrationValues;        // first is P0, second is P1, used equation is y=P0+P1*x,
                                                                      // oredr in deque is channel 0,1,2, etc..
   //types defined in NIDAQmxBase
   int32       ferror;                                                // NI error code
   TaskHandle  fTaskHandle;                                           // NI task handler
   float64     fReadArray[10];                                        // buffer for 10 sampels
   // common reading parameters
   int32       fTerminalConfig;                                       //
   float64     fMinRangeVolts;                                        //
   float64     fMaxRangeVolts;                                        //
   int32       fNumSampsPerChan;                                      //
   float64     fTimeout;                                              //
   bool32      fFillMode;                                             //
   uInt64      fMaxArraySize;                                         //
   int32       fReadArraySize;                                        //

   // error\exeption status
   COnLineExeption *fexeptionStatus;   // handling error mesages
};
#endif
