/************************************************************************/
/*									*/
/*  This is the header file for the FEC library				*/
/*									*/
/*  09. Jun. 05  L. Musa & H. Santos  created				*/
/*                                                                      */
/*  Last update: 07. Oct. 05                                            */
/*									*/
/************************************************************************/

#include <sys/types.h>
#include <tools/get_input.h>
#include <tools/rcc_time_stamp.h>
#include <altro/altro.h>
#include <u2f/u2f.h>

/*************/
/* Constants */
/*************/

#define MAX_NR_OF_FECS 16

#define K1_ADDR    0x0
#define K2_ADDR    0x1
#define K3_ADDR    0x2
#define L1_ADDR    0x3
#define L2_ADDR    0x4 
#define L3_ADDR    0x5
#define VFPED_ADDR 0x6 
#define PMDTA_ADDR 0x7
#define ADEVL_ADDR 0x11
#define ZSTHR_ADDR 0x8
#define BCTHR_ADDR 0x9
#define TRCFG_ADDR 0xA
#define DPCFG_ADDR 0xB
#define BFNPT_ADDR 0xC
#define ERSTR_ADDR 0x10
#define TRCNT_ADDR 0x12
#define PMADD_ADDR 0xD
  
/*********/ 
/* Macros*/
/*********/ 
#define CHECK_RET(ecode)\
  if (ecode)\
  {\
    rcc_error_print(stdout, ecode);\
    return ecode;\
  }\

#define FECISOPEN(handle) {if(!fec_is_open[handle]) return(FEC_NOTOPEN);} 


/*************/
/*error codes*/
/*************/
enum
{
  FEC_SUCCESS = 0,  
  FEC_U2F_FAIL = (P_ID_FEC << 8) + 1,
  FEC_NOTOPEN,
  FEC_NO_CODE,
  FEC_FILE,
  FEC_PARSE,
  FEC_ERROR_FAIL
};


/***************/
/*error strings*/
/***************/
#define FEC_SUCCESS_STR      "Function successfully executed"
#define FEC_U2F_FAIL_STR     "Error from U2F library"
#define FEC_NOTOPEN_STR      "The library has not been opened yet"
#define FEC_NO_CODE_STR      "Unknown error"
#define FEC_FILE_STR         "Failed to open configuration file"
#define FEC_PARSE_STR        "Parsing error. Check format of configuration file"

/**************/
/* Structures */
/**************/
typedef struct  
{
  u_int k1[16];
  u_int k2[16];
  u_int k3[16];
  u_int l1[16];
  u_int l2[16];
  u_int l3[16];
  u_int vfped[16];
  u_int pmdta[16];
  u_int adevl[16];
  u_int zsthr;
  u_int bcthr;
  u_int trcfg;
  u_int dpcfg;
  u_int bfnpt;
  u_int erstr;
  u_int trcnt;
  u_int pmadd;
} altro_regs_t;

/***********************************/ 
/*Official functions of the library*/
/***********************************/ 
u_int FEC_Open(char *node, int *handle);
u_int FEC_Close(int handle);
u_int FEC_Write_ALTRO_Reg(int handle, u_int ch_addr, u_int reg_addr, u_int reg_data);
u_int FEC_Initialize_ALTRO(int handle, u_int fec_addr, u_int altro_addr, altro_regs_t *init_data); 
u_int FEC_Write_Full_ALTRO_Reg(int handle,u_int fec_addr, u_int altro_addr, u_int reg_addr, u_int reg_data[]); 
u_int FEC_Read_ALTRO_Reg(int handle, u_int ch_addr, u_int reg_addr, u_int *reg_data);
u_int FEC_Write_ALTRO_PMEM(int handle, u_int bcast, u_int ch_addr, u_int nwords, char *fname);
u_int FEC_Read_ALTRO_PMEM(int handle, u_int ch_addr, u_int nwords, char fname[]); 
u_int FEC_Power_Reset(int handle, u_int fecs_addr);
u_int FEC_All_Reset(int handle);
u_int FEC_All_Status(int handle, int *fec_status);
u_int FEC_BC_Reg_Write(int handle, u_int fec_addr, u_int reg_addr, u_int data);
u_int FEC_BC_Reg_Read(int handle, u_int fec_addr, u_int reg_addr, u_int *reg_data);
u_int FEC_BC_Mon(int handle, u_int fec_addr, float *temp, float *ana_v,  float *ana_i, float *dig_v, float *dig_i);
u_int FEC_FM_Write(int handle, u_int fec_addr, u_int nwords, char fname[]);
u_int FEC_FM_Read(int handle, u_int fec_addr, u_int nwords, char fname[]);
u_int FEC_Broadcast_BC(int handle,u_int reg_addr, u_int reg_data);
u_int FEC_Broadcast_AL(int handle,u_int reg_addr, u_int reg_data); 
u_int FEC_Event_Readout(int handle, u_int fec_addr, u_int trg_mode, u_int rdo_mode, u_int nevents, u_int nsamples, u_int n_before_trg, char fname[]); 
u_int FEC_Config_Read(char *name, altro_regs_t *data);
u_int FEC_Initialize_Partition(int handle, char *name);
u_int FEC_Data_Decode(char *text, u_int num, u_int *data);


/********************/ 
/*Internal functions*/
/********************/ 
u_int FEC_err_get(err_pack err, err_str pid, err_str code);

