//----------------------------------------------------------------------------
//   Body: CGDataFormat.cpp,v 1.0                                2005/02/24 //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2004                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

//standard
#include <iostream>

// my
#include "CGDataFormat.h"

// TODO:
// change to STL containers
//

using namespace std;

DFormat::DFormat() 
{
  trailer     = new DTrailer();
  nofSamples  = 0;
  for(int i=0; i<2048; ++i) {
    dataRAW[i] = NULL;
  }
}
 
void DFormat::unpack_rawData(UInt_t *arr, const UInt_t &wr)
{
  DSample    *tempdataRAW   = NULL;
  int         IDtra         = 0;
  int         nstrData      = 1;
  UInt_t      sampcount     = 0; 
  
  try {
    //delete arries
    clear();
    // check last word of trailer
    if(arr[(wr-1)]!=0x2aa) {
      throw -100;
    }
    //get data
    for(int i=(wr-1); i>=0; --i) {
#ifdef DEBUG_GOOFIE_UNPACK
  cout << "i : " << i << "  Word " << showbase << hex << arr[i] << dec << endl;
#endif
      IDtra = (i>int(wr-5));
  
      switch(IDtra) {
      case 1:
	if(i==int(wr-1) || i==int(wr-2)) {
	  trailer->endM[(wr-i)-1] = arr[i];
	} else if(i==int(wr-3)) {
	  trailer->lWords = arr[i];
	} else if(i==int(wr-4)) {
	  trailer->hadd = arr[i];
	}
	nstrData  = 1;
	break;
      case 0:
	if(arr[i]==0x2aa && nstrData) {
#ifdef DEBUG_GOOFIE_UNPACK
	  cout << "empty :)))" << endl;
#endif
	  continue;
	} else {
	  // sample unpack
 	  if(nstrData) {
 	    //create array for the next sample
 	    dataRAW[nofSamples] = new DSample();
	    tempdataRAW         = dataRAW[nofSamples];
 	    // get sample
 	    sampcount = arr[i];
 	    tempdataRAW->nSample = (sampcount-2);
 	    tempdataRAW->createASample();
 	    nstrData = 0;
	    --sampcount;  // from n to 0;
 	  } else {
 	    --sampcount;
 	    if(sampcount == tempdataRAW->nSample) {
 	      tempdataRAW->timeOFlastSample = arr[i];
 	    } else {
 	      tempdataRAW->Sample[sampcount] = arr[i];
 	      if(sampcount==0) {
 		nstrData = 1;
 		++nofSamples;
	      }
#ifdef DEBUG_GOOFIE_UNPACK
 	      cout << "    data : " << sampcount << "   " << hex << tempdataRAW->Sample[sampcount] << dec << flush << endl;
#endif
	    }
	  }
	} 
	break;
      }
    }
  } catch (signed int a) {
    cerr << "Error in <DFormat::unpack_rawData> : Code : " << a << flush << endl;
  }
#ifdef DEBUG_GOOFIE_UNPACK
  cout  << trailer->endM[0] << "  " << trailer->endM[1] << "  " << trailer->lWords << "  " << trailer->hadd << endl;
  cout << showbase << hex << tempdataRAW->nSample+2 << dec << endl;
  cout << showbase << hex << tempdataRAW->timeOFlastSample << dec << endl;
  for(int i=int(tempdataRAW->nSample); i>0; --i)
    cout  << (i-1) << "  " << showbase << hex << tempdataRAW->Sample[(i-1)] << dec << endl;
#endif
}

void DFormat::fill_arr(UInt_t *arr, UInt_t &wr)
{
  int      tindex      = 0;
  int      i           = 0;
  DSample *tempdataRAW = NULL;

  for(int s=nofSamples; s>0; --s) {
    tempdataRAW = dataRAW[(s-1)];
    tindex = (tempdataRAW->timeOFlastSample-tempdataRAW->nSample);
    if((tindex+tempdataRAW->nSample) >= 1024) {
	cerr << "Error in <DFormat::fill_arr>: data length is larger then input array" << flush << endl; 
	continue;
    }
#ifdef DEBUG_GOOFIE_UNPACK
    cout << "Info from <DFormat::fill_arr>: for s = " << s << " data pointer is = " << tempdataRAW << flush << endl;    
    cout << "time index is = " << tindex << " local time = " << tempdataRAW->timeOFlastSample 
         << " local no. of samples = " << tempdataRAW->nSample << flush << endl;
#endif         
    for(i=0; i<int(tempdataRAW->nSample); ++i) {
       arr[i+tindex] = tempdataRAW->Sample[i];
    }
  }
  wr = (i+tindex);
}

void DFormat::clear()
{
  for(int i=0; i<nofSamples; ++i) {
    if(dataRAW[i]) {
      delete dataRAW[i];
      dataRAW[i] = NULL;
    }
  }
  trailer->clear();
  nofSamples = 0;
}

DTrailer::DTrailer() 
{
  clear();
}

void DTrailer::clear()
{
  hadd    = 0;
  lWords  = 0;
  endM[0] = 0;
  endM[1] = 0;
}

DSample::DSample() 
{
  Sample           = NULL;
  timeOFlastSample = 0;
  nSample          = 0;
}

DSample::~DSample() 
{
  if(Sample) {
    delete [] Sample;
    Sample = NULL;
  }
}

void DSample::createASample()
{
  Sample = new UInt_t[nSample];
}

void DSample::clear()
{
  if(Sample) {
    delete [] Sample;
    Sample = NULL;
  }
  timeOFlastSample = 0;
  nSample          = 0;
}
