//----------------------------------------------------------------------------
//  Body: CGRegVal.cpp,v 1.0                                 2004/11/16     //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2004                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <fstream>
#include <sstream>
#include <cstring>
#include <cstdlib>

#include <TString.h>

#include "CGRegVal.h"

using namespace std;

ClassImp(CGRegVal)

// use decimal notation

CGRegVal::CGRegVal() : TObject()
{
  DefValue();
}

void CGRegVal::DefValue()
{
  fZSCTL = 0;
  fZSTHD = 0;
  fZSPED = 0;
  fACTCH = 0; 
  fNSEVT = 490;
  fPRTRG = 9;
  fBSLCT = 0;
  fTMASK = 3;
  fTATHD = 200;
  fTBTHD = 200;
  fIsChannelACalledNear = 1; 
}

void CGRegVal::Clear()
{
  fZSCTL = 0;
  fZSTHD = 0;
  fZSPED = 0;
  fACTCH = 0; 
  fNSEVT = 0;
  fPRTRG = 0;
  fBSLCT = 0;
  fTMASK = 0;
  fTATHD = 0;
  fTBTHD = 0;
  fIsChannelACalledNear = 0;
}

void CGRegVal::SetValueFromFile(std::string _fname)
{
  // the names in the configuration file must be unique
  //
  //
  static char     *fRegName[] = {"ZSCTL", "ZSTHD", "ZSPED", "ACTCH", "NSEVT", "PRTRG", "BSLCT", "TMASK", "TATHD", "TBTHD", "TAISN"};
  static UShort_t *varr[]     = {&fZSCTL, &fZSTHD, &fZSPED, &fACTCH, &fNSEVT, &fPRTRG, &fBSLCT, &fTMASK, &fTATHD, &fTBTHD, &fIsChannelACalledNear};

  ifstream fin(_fname.c_str(),  ios::in);
  if (fin.is_open())
  {
    TString parse;
    TString temp;
    
    while(1) {
      parse.ReadLine(fin);
      if(fin.eof())
	break;

      // remove comments
      int pos = parse.Index("//");
      if(pos>=0 && pos<5) continue;

      // local copy
      temp = parse;
      temp.Remove(5);
    
      for(int i=0; i<11; ++i) {
	if(!temp.CompareTo(fRegName[i])) {
	  parse.Remove(0,(parse.Index("=")+1));
          int secondComment = parse.Index("//");
	  if(secondComment>0) {
	     parse.Remove(secondComment);
	  }
	  *varr[i]=atoi(parse.Data());
	  break;
	}
      }
    }
    fin.close();
  } else {
    cerr << "Warning in <CGRegVal::SetValueFromFile>: an input file doesn't found, default register value" << flush << endl; 
    DefValue();
  }
}

UShort_t CGRegVal::HexToDec(UShort_t &vdec) const
{
   UShort_t retVal  = vdec;
//   ostringstream s;
//   s << vdec;
//   ios_base& (*h)(ios_base&) = hex;
//   istringstream is(s.str());
//   is >> h >>  retVal;
  return retVal;
}

ostream & operator<<(ostream &stream_out, CGRegVal &data_in)
{
  // input data in HEX 
  stream_out << "Register | Value " << endl;
  stream_out << "=========|=======" << endl;
  stream_out << "ZSCTL    | " << data_in.HexToDec(data_in.fZSCTL) << endl;
  stream_out << "ZSTHD    | " << data_in.HexToDec(data_in.fZSTHD) << endl;
  stream_out << "ZSPED    | " << data_in.HexToDec(data_in.fZSPED) << endl;
  stream_out << "ACTCH    | " << data_in.HexToDec(data_in.fACTCH) << endl;
  stream_out << "NSEVT    | " << data_in.HexToDec(data_in.fNSEVT) << endl;
  stream_out << "PRTRG    | " << data_in.HexToDec(data_in.fPRTRG) << endl;
  stream_out << "BSLCT    | " << data_in.HexToDec(data_in.fBSLCT) << endl;
  stream_out << "TMASK    | " << data_in.HexToDec(data_in.fTMASK) << endl;
  stream_out << "TATHD    | " << data_in.HexToDec(data_in.fTATHD) << endl;
  stream_out << "TBTHD    | " << data_in.HexToDec(data_in.fTBTHD) << endl;
  return stream_out;
}
