//----------------------------------------------------------------------------
//   Body: CGUnpack.cpp,v 1.0                                   2004/11/04  //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2004                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

//Remove these constants. Just added by M. Joos to make the compiler happy
#define O_CHA 1
#define O_CHB 2
#define O_CHC 3


//standard
#include <unistd.h>
#include <iostream>
#include <sstream>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <sys/types.h>

#include <TH1.h>

// for reading data from usb
#include "get_input.h"
#include "rcc_error.h"
//#include "rcu_lib.h"
#include "goofie.h"

// my
#include "CGDataFormat.h"
#include "CGUnpack.h"
#include "CGRegVal.h"

using namespace std;

ClassImp(CGUnpack)

CGUnpack::CGUnpack() : TObject()
{
  data_r     = 0;
  errCode    = 0;
  fBinit     = 0;
  funpacTrue = 0; 
  fLASTBIN   = 0;
  dataIn     = NULL;

  fregInfo = new CGRegVal();
  CreateDataArray ();
}

CGUnpack::~CGUnpack()
{
  if(fregInfo) {
    delete fregInfo;
    fregInfo = NULL;
  }
  DeleteArray();
}

void CGUnpack::CreateDataArray()
{
   // raw data input
   if(!dataIn) {
      dataIn = new DFormat();
   }
   // local buffor
   vector<unsigned int> fspectrum(1024);
   flocalBuffer.insert(flocalBuffer.begin(), 3, fspectrum);
}

void CGUnpack::DeleteArray()
{
  dataIn->clear();
  if(!dataIn) {
    delete dataIn;
    dataIn = NULL;
  }
  // clear list
  flocalBuffer.clear();
}

UShort_t CGUnpack::DecToHex(UShort_t &vdec) const
{
  UShort_t retVal = vdec;
//   ios_base& (*h)(ios_base&) = hex;
//   ostringstream s;
//   s << h << vdec;
//   ios_base& (*d)(ios_base&) = dec;
//   istringstream is(s.str());
//   is >> d >>  retVal;
  return retVal;
}

int CGUnpack::InitCVal() 
{
#ifdef DEBUG_GOOFIE
  traceLevel = 10000;
  packageId  = 0;
#endif

  try {
    if(fBinit==0) {
      // open connection
      OpenConnection();
    }
    usleep(200);
    // zero suppression control
    errCode = Goofie_CSR_Write(O_ZSCTL, DecToHex(fregInfo->fZSCTL));
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -2;
    }
    usleep(200);
    errCode = Goofie_CSR_Write(O_ZSTHD, DecToHex(fregInfo->fZSTHD));
    if (errCode) {
          rcc_error_print(stdout, errCode);
      throw -3;
    }
    usleep(200);
    errCode = Goofie_CSR_Write(O_ZSPED, DecToHex(fregInfo->fZSPED));
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -4;
    }
    usleep(200);
    // numbers of channels (def :: 500)
    errCode = Goofie_CSR_Write(O_NSEVT, DecToHex(fregInfo->fNSEVT));
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -5;
    }
    usleep(200);
    //pre/post trigger
    errCode = Goofie_CSR_Write(O_PRTRG, DecToHex(fregInfo->fPRTRG));
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -6;
    }
    usleep(200);
    // bslct  (0=ADC-FPD, 3=MEM-FPD, 5=ACD-MEM)
    errCode = Goofie_CSR_Write(O_BSLCT, DecToHex(fregInfo->fBSLCT));
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -7;
    } 
    usleep(200);
    // TRIGGER
    // tmask
    errCode = Goofie_CSR_Write(O_TMASK, DecToHex(fregInfo->fTMASK));
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -8;
    } 
    usleep(200);
    // tathd  - treshold for chA
    errCode = Goofie_CSR_Write(O_TATHD, DecToHex(fregInfo->fTATHD));
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -9;
    } 
    usleep(200);
    // tbthd  - treshold for chB
    errCode = Goofie_CSR_Write(O_TBTHD, DecToHex(fregInfo->fTBTHD));
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -10;
    }
  } catch (signed int a) {
    cerr << "Error in <CGUnpack::InitCVal> : " << showbase << hex << errCode << "  , throw : " << dec << a << flush << endl;
  }
  return errCode;
}

void CGUnpack::InitBSCMode(const UInt_t &mode) 
{ 
  static UInt_t foldMode = 10000;

  if(foldMode!=mode) {
     // mode   (0=ACD-FPD, 1=ADC-MEM, 2=MEM-FPD, 3=ACD-FPD
     // bslct  (0=ADC-FPD, 3=MEM-FPD, 5=ACD-MEM)
    if(mode==0) {
      fregInfo->fBSLCT = 0;
      foldMode=mode;
    } else if(mode==1) {
      fregInfo->fBSLCT = 5;
      foldMode=mode;
    } else if(mode==2) {
      fregInfo->fBSLCT = 3;
      foldMode=mode;
    } else if(mode==3) {
      fregInfo->fBSLCT = 0;  //important
      foldMode=mode;
    } else {
      cerr << "Error in <CGUnpack::InitBSCMode>: this mode wasn't implemented" << flush << endl;
      return;
    }
    errCode = Goofie_CSR_Write(O_BSLCT, DecToHex(fregInfo->fBSLCT));
    if (errCode) {
      rcc_error_print(stdout, errCode);
    }
  }
}

void CGUnpack::OpenConnection()   throw(signed int)
{ 
  if(fBinit==0) {
    // open connection
    errCode = Goofie_Open(DEFAULT_NODE);
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -1;
    }
  }
#ifdef DEBUG_GOOFIE
    cout << " Debug Info :: -- opening connection -- " << endl;
#endif

  fBinit=1;
}

void CGUnpack::CloseConnection() 
{
  if(IsInit()!=0) {
    errCode = Goofie_Close();
    if (errCode) {
      rcc_error_print(stdout, errCode);
    }
    //reset flag
    fBinit=0;
#ifdef DEBUG_GOOFIE
    cout << " Debug Info :: -- closing connection -- " << endl;
#endif
  }
}

int CGUnpack::EventComming() 
{
  errCode = Goofie_CSR_Read(O_TRGEV, &data_r);
#ifdef DEBUG_GOOFIE
  cout << " Debug Info :: Get Trigger Info : " << showbase << hex << data_r << dec << endl;
#endif
  if (errCode) {
    rcc_error_print(stdout, errCode);
    return -1;
  } 
  return data_r;
}

int CGUnpack::WhichChTrigged()
{
  // 1 - ch A, 2 - ch B, 3 - ch A and B
  return (triggerBITS & 0x3);
}

void CGUnpack::ResetTrigger() 
{
  //reset trigger flags
  errCode = Goofie_Exec_Command(I_TWRST);
#ifdef DEBUG_GOOFIE 
  cout << " Debug Info :: Reset Trigger  : " << showbase << hex << errCode << dec << endl;
#endif
  if (errCode) {
    rcc_error_print(stdout, errCode);
  }
}

void CGUnpack::ResetSoftwareTrigger() 
{
  //reset trigger flags
  errCode = Goofie_Exec_Command(I_SWRST);
#ifdef DEBUG_GOOFIE
  cout << " Debug Info :: Reset Software Trigger  : " << showbase << hex << errCode << dec << endl;
#endif  
  if (errCode) {
    rcc_error_print(stdout, errCode);
  }
}

void CGUnpack::SetSoftwareTrigger() 
{
  data_r = 1;
  errCode = Goofie_Exec_Command(I_SWTRG);
#ifdef DEBUG_GOOFIE
  cout << " Debug Info :: Set Software Trigger  : " << showbase << hex << errCode << dec <<endl;
#endif
  if (errCode) {
    rcc_error_print(stdout, errCode);
  }
}

int CGUnpack::ReadDataLength(UInt_t &chanel)
{
  switch(chanel) {

  case 1:
    errCode = Goofie_EVLEN_Read(O_CHA, &data_r);
    break;
  case 2:
    errCode = Goofie_EVLEN_Read(O_CHB, &data_r);
    break;
  case 3:
    errCode = Goofie_EVLEN_Read(O_CHC, &data_r);
    break;

  default:
    errCode = Goofie_EVLEN_Read(O_CHA, &data_r);
  }
  if (errCode) {
    rcc_error_print(stdout, errCode);
  }
  int ret = (int)data_r;
#ifdef DEBUG_GOOFIE
  cout << " Debug Info :: Data Length : " << showbase << hex << ret << dec << endl;
#endif
  return ret;
}

int CGUnpack::ChannelNR(UInt_t &chanel)
{
  int ch_n = 0;
  switch(chanel) {
  case 1:
    ch_n = O_CHA;
    break;
  case 2:
    ch_n = O_CHB;
    break;
  case 3:
    ch_n = O_CHC;
    break;
  default:
    ch_n = O_CHA;
  }
  return ch_n;
} 

VectorList & CGUnpack::GetBufferList()
{
   //return first position from list
   return flocalBuffer;
}

VectorList & CGUnpack::ReadAllChannels(UInt_t _caseMode)
{
   // TODO
   // 1) add algorithm, in case buffer size is too small

   // set trigger mode
   bool (CGUnpack::*pfun)(UInt_t &) = NULL;
   if(ftriggerMode) {
      pfun = &CGUnpack::ReadDataWithSignalTrigger;
   } else {
      pfun = &CGUnpack::ReadDataWithSoftwareTrigger;
      SetSoftwareTrigger();
   }
   // check for trigger state
   triggerBITS = EventComming();
   // check whether acquisition occurred and event is ready to be read-out.
   if(EventReadyToRead()) {
     // check mode of reading
     if(_caseMode!=3) _caseMode = 0;
     // set iterator at the begining
     fitlocBuff = flocalBuffer.begin();

#ifdef DEBUG_GOOFIE
     cout << "---------------------------- Start ReadALL Channels ------------------------ " << endl; 
     cout << "case mode = " <<  _caseMode << ", trigger mode = " << ftriggerMode << ", unpack mode = " << funpacTrue <<  flush << endl;
     cout << " --------------- List size = " << flocalBuffer.size() << flush << endl;
     cout << " --------------- Local array size = " << S_DMEM << flush << endl;
#endif

     //read from chanells
     for(u_int ch_n=1; ch_n<=3; ++ch_n) {
       switch(_caseMode) {
       case 0:
	 (this->*pfun)(ch_n);
	 break;
       case 3:
	 ReadPedestMemory(ch_n);
	 break;
       default:
	 cerr << "Error in <CGUnpack::ReadAllChannels> case " << _caseMode << " is not implemented " << flush << endl;
       }
#ifdef DEBUG_GOOFIE         
       // cout << " ------------- data readed from goofie board  -- size = " << fitlocBuff->size() << " --  address = " << &(*fitlocBuff) << flush << endl;
       cout << " ------------- data readed from goofie board  -- --  address = " <<   &(*fitlocBuff) << flush << endl;
#endif       
       // check buffor size
       if(fitlocBuff->size() <= fLASTBIN) {
	 cerr << "Error in <CGUnpack::ReadAllChannels> buffor is too small " << flush << endl;
       } else {
	 // fill buffor
	 for(u_int i=0; i<fLASTBIN; ++i) {
	   fitlocBuff->at(i) = unpmemdata[i];
	 }
       }
       ++fitlocBuff;
     }
   }
   // reset trigger status
   ResetTrigger();
#ifdef DEBUG_GOOFIE  
   cout << "ReadAllChannels : list addres  = " << &flocalBuffer << endl;
   cout << "---------------------------- Stop ReadALL Channels ------------------------ " << endl;
#endif
   return flocalBuffer;
}


void CGUnpack::FillHistograms(TH1 *h[], UInt_t _caseMode) 
{
   int channel = -1;
   fitlocBuff = ReadAllChannels(_caseMode).begin();
#ifdef DEBUG_GOOFIE
   cout << "Info from <CGUnpack::Fillhists : iterator addres at begining       = " << &fitlocBuff << endl;
#endif
   // loop over channels
   do {
     ++channel;
     // loop over bins 
     for(UInt_t i=1; i<=fLASTBIN; ++i) {
       if(!h[channel]) {
	 cout << "Error in <CGUnpack::FillHists>  Pointer is NULL " << flush << endl;
       } else {
	 h[channel]->SetBinContent(i, fitlocBuff->at(i-1));
       }
     }
   } while((++fitlocBuff) != flocalBuffer.end());
}

bool CGUnpack::ReadDataWithSoftwareTrigger(UInt_t &chanel)
{
  bool flagStatus = false;
   //clear arraies
  for(int i=0; i<(S_DMEM*8); ++i) {
    memdata[i]    = 0;
    unpmemdata[i] = 0;
  }
  if((triggerBITS >> 0x2) == 3) {
    ReadDataMemory(chanel);
    flagStatus = true;
  } 
  return flagStatus;
}
  
bool CGUnpack::ReadDataWithSignalTrigger(UInt_t &chanel)
{ 
  bool flagStatus = false;
  //clear arraies
  for(int i=0; i<(S_DMEM*8); ++i) {
    memdata[i]    = 0;
    unpmemdata[i] = 0;
  }   
  if(((triggerBITS >> 0x2) == 3)) { // event ready to be read out
    // which chanel
    if(WhichChTrigged()) {
      ReadDataMemory(chanel);
      flagStatus = true;
    } 
  }
  return flagStatus;
}

void CGUnpack::ReadDataMemory(UInt_t &chanel)
{
#ifdef DEBUG_GOOFIE
  cout << " Debug Info :: ------ Read data from chanel: " << chanel << " -------" << endl;
#endif
  unsigned int lchann   = ChannelNR(chanel);
  UInt_t rsize    = ReadDataLength(chanel);
  fLASTBIN        = 0;

  // read data from board
  errCode = Goofie_DMEM_Read(lchann, rsize, &fLASTBIN, memdata);
  if (errCode) {
    rcc_error_print(stdout, errCode);
    return;
  }
#ifdef DEBUG_GOOFIE  
  cout << fLASTBIN << " Debug Info :: words received. Dumping data" << endl;
  for (UInt_t loop = 0; loop < fLASTBIN; loop++) {
    cout << "Word " << loop << " = " << showbase << hex << memdata[loop] << dec << endl;;
  }
#endif
  //unpack
  if(funpacTrue) {
    dataIn->unpack_rawData(memdata, fLASTBIN);
    dataIn->fill_arr(unpmemdata, fLASTBIN);
  } else {
    for(int i=0; i<(8*S_DMEM); i++)
      unpmemdata[i] = memdata[i];
    fLASTBIN = S_DMEM*4;
  }
}

void CGUnpack::ReadPedestMemory(UInt_t &chanel)
{
  
  cout << "read Pedestal Memory " << flush << endl;

  UInt_t offset   = 0;
  UInt_t lchann   = ChannelNR(chanel);
  //UInt_t rsize    = S_DMEM*4; 
  //clear arraies
  u_short testPmem[1025];
  
  for(int i=0; i<(S_DMEM*8); i++) {
    memdata[i]    = 0;
    unpmemdata[i] = 0;
    testPmem[i]   = 0;
  }
  // read pedestal memory from board
  errCode = Goofie_PMEM_Read(lchann, offset, testPmem);
  if (errCode) {
    rcc_error_print(stdout, errCode);
    return;
  }
  // test
  for(int i=0; i<(S_DMEM*8); i++) {
    unpmemdata[i] = testPmem[i];
  }
  fLASTBIN = S_DMEM*4;

#ifdef DEBUG_GOOFIE  
  cout << fLASTBIN << " Debug Info :: words received. Dumping data" << endl;
  for (UInt_t loop = 0; loop < fLASTBIN; loop++) {
    cout << "Word " << loop << " = " << showbase << hex << unpmemdata[loop] << dec << endl;;
  }
#endif
}

void CGUnpack::Write(UInt_t &chanel)
{
  UInt_t offset = 0;
  UInt_t lchann = ChannelNR(chanel);
  // UInt_t rsize  = (S_DMEM*4)-1; 
  cout << "ch : " << offset << endl; 

  if(fBinit==0) {
    InitCVal();
  }
  // set Baseline Correction to ADC-FPD
  fregInfo->fBSLCT = 0;
  errCode = Goofie_CSR_Write(O_BSLCT, DecToHex(fregInfo->fBSLCT));
  if (errCode) {
    rcc_error_print(stdout, errCode);
  }

  //get data from file  !!!!! it is not ready
  u_short testMem[1025];
  for(int i=0; i<(S_DMEM*8); ++i) {
    memdata[i] = 0xF;
    testMem[i] = 0XF;
  }

  errCode = Goofie_PMEM_Write(lchann, offset, testMem[0]);
  if (errCode) {
    rcc_error_print(stdout, errCode);
    return;
  }
}

CGRegVal CGUnpack::GetRegistersValues()
{
  if(fBinit==0)
    InitCVal();
  
  CGRegVal rtemp;
  rtemp.Clear();

  try {
    errCode = Goofie_CSR_Read(O_ZSCTL, &rtemp.fZSCTL);
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -1;
    } 
    usleep(200);
    errCode = Goofie_CSR_Read(O_ZSTHD, &rtemp.fZSTHD);
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -2;
    }
    usleep(200);
    errCode = Goofie_CSR_Read(O_ZSPED, &rtemp.fZSPED);
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -3;
    }
//     usleep(200);
//     errCode = U2F_Reg_Read(O_ACTCH, &rtemp.fACTCH);
//     if (errCode) {
//       rcc_error_print(stdout, errCode);
//       throw -4;
//     } 
   usleep(200);
    errCode = Goofie_CSR_Read(O_NSEVT, &rtemp.fNSEVT);
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -5;
    }
    usleep(200);
    errCode = Goofie_CSR_Read(O_PRTRG, &rtemp.fPRTRG);
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -6;
    }
    usleep(200);
    errCode = Goofie_CSR_Read(O_BSLCT, &rtemp.fBSLCT);
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -7;
    }
    usleep(200);
    errCode = Goofie_CSR_Read(O_TMASK, &rtemp.fTMASK);
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -8;
    }
    usleep(200);
    errCode = Goofie_CSR_Read(O_TATHD, &rtemp.fTATHD);
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -9;
    }
    usleep(200);
    errCode = Goofie_CSR_Read(O_TBTHD, &rtemp.fTBTHD);
    if (errCode) {
      rcc_error_print(stdout, errCode);
      throw -9;
    }
  } catch (signed int a) {
    cerr << "Error in <CGUnpack::PrintRegisters> : " << showbase << hex << errCode << "  , throw : " << dec << a << flush << endl;
  } 
  return rtemp;
}


CGRegVal CGUnpack::PrintRegisters()
{
  CGRegVal ret = GetRegistersValues();
  cout << ret << endl;
  return ret;
}
