//----------------------------------------------------------------------------
//   Header: CGUnpack.h,v 1.0                                   2004/11/04  //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2004                                 //
//                                                                          //
//  Declaration of class CGUnpack                                           //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef CGUNPACK_H
#define CGUNPACK_H

#include <TObject.h>
#include <list>
#include <vector>

extern int packageId;
extern int traceLevel;

class TH1;
class DFormat;
class CGRegVal;

typedef std::list<std::vector<unsigned int> > VectorList;

class CGUnpack : public TObject
{
 
public:
   CGUnpack();
   virtual ~CGUnpack();
   
   int          GetErrorCode()                       const {return errCode;}
   void         ClearErrCode()                             {errCode=0;}
   
   int          InitCVal();
   int          IsInit()                             const {return fBinit;}
   void         InitBSCMode(const UInt_t &mode);                                  // 0=ADC-FPD, 1=ACD-MEM, 2=MEM-FPD, 3=ADC-FPD
   void         OpenConnection() throw(signed int);
   void         CloseConnection();
   int          EventComming();
   int          EventReadyToRead()                         {return ((triggerBITS >> 0x2) == 3);}
   int          WhichChTrigged();
   void         ResetTrigger();
   void         ResetSoftwareTrigger();
   void         SetSoftwareTrigger();
   void         SetUnpacStatus(Int_t stat)                 {funpacTrue   = stat;}  // 1-on, 0-off
   void         SetTriggerMode(Int_t stat)                 {ftriggerMode = stat;}  // 1-signal, 0-software
   int          ChannelNR(UInt_t &chanel);
   bool         ReadDataWithSoftwareTrigger(UInt_t &chanel);
   bool         ReadDataWithSignalTrigger(UInt_t &chanel);
   void         FillHistograms(TH1 *h[], UInt_t _caseMode);
   VectorList & ReadAllChannels(UInt_t _caseMode);
   VectorList & GetBufferList();                                                   // return flocalBuffer list
      
   void         Write(UInt_t &chanel);                                            // Write pedestal values

   CGRegVal     GetRegistersValues();
   CGRegVal     PrintRegisters();
   CGRegVal    *GetRegPointer()                            {return fregInfo;}
  
private: 
   void         CreateDataArray();
   void         DeleteArray();
   UShort_t     DecToHex(UShort_t &vdec) const;
   int          ReadDataLength(UInt_t &chanel);
   void         ReadDataMemory(UInt_t &chanel); 
   void         ReadPedestMemory(UInt_t &chanel);

   
public:
   // trigger
   UInt_t       triggerBITS;
   
private:
   VectorList   flocalBuffer;          // output data handler
   VectorList::iterator fitlocBuff;    // list iterator
   UInt_t       unpmemdata[1024];      // unpacked spectrum
   UInt_t       memdata[1024];         // raw spectrum
   UInt_t       fLASTBIN;
   UInt_t       fBinit;
   Int_t        ftriggerMode;
   Int_t        funpacTrue;
   
   
protected:
   UInt_t       errCode;
   //usb var
   UShort_t     data_r;
   CGRegVal    *fregInfo;
   DFormat     *dataIn; 
   
   ClassDef(CGUnpack, 1)
};
#endif
