//----------------------------------------------------------------------------
//  exec code, v 1.0                                          2007/04/16    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2007                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <string>
#include <pthread.h>

#include "CMacroDef.h"
#include "COnLineExeption.h"
#include "CDataStructure.h"

#include "CRootFileHandler.h"

#include "CGoofieBoard.h"
#include "CService.h"
#include "CReadOutBoards.h"
#include "COnLineAna.h"

using namespace std;

int main(int argc, char **argv) 
{
  // get configuration files location
  string locationHardwareFName = "ConfigFiles/hardwareTRDsetup.conf";
  string locationSoftwareFName = "ConfigFiles/softwareTRDsetup.conf";

  if(argc>=3) {
    locationHardwareFName = argv[1];
    locationSoftwareFName = argv[2];
  }
  
  // start true application
  CService *pWrapper   = new CService();                         //queue, flags, mutex, etc..
  pWrapper->InitializeSetUp(locationSoftwareFName);
     
  CReadOutBoards *ROB = new CReadOutBoards(pWrapper);            // hardware access 
  pthread_t *ptrob    = ROB->StartROB(locationHardwareFName);    // link to hardware configuration file
  
  COnLineAna     *ONA = new COnLineAna(pWrapper);                // data analysis and DIM server part
  pthread_t *ptona    = ONA->StartONA(locationSoftwareFName);    // link to software configuration file

  // wait until threads finish
  pthread_join(*ptona, NULL);
  pthread_join(*ptrob, NULL);

  DEBUGINFO("Debuge Info => main => finishing application, attempting to delete objects ",0)
  
  // delete objects
  if(ONA) {
     delete ONA;
     ONA=NULL;
  }
  if(ROB) {
     delete ROB;
     ROB=NULL;
  }
  if(pWrapper) {
     delete pWrapper;
     pWrapper=NULL;
  }

  DEBUGINFO("Debuge Info => main => all objects deleted",0)
  
  return 0;
}
